import { languageDataProp, defineLanguageFacet, syntaxTree, delimitedIndent, indentNodeProp, Language, LRLanguage, foldNodeProp, flatIndent, continuedIndent, foldInside, LanguageSupport } from '@codemirror/language';
import { NodeProp, Tree, NodeType, Parser, NodeSet, parseMixed } from '@lezer/common';
import { ContextTracker, ExternalTokenizer, LRParser } from '@lezer/lr';
import { styleTags, tags } from '@lezer/highlight';
import { snippetCompletion, completeFromList } from '@codemirror/autocomplete';

// Note that 'eval' and 'do' are not in this list as they can accept a block.
const namedUnaryOperators = new Set([
    'chr',
    'defined',
    'delete',
    'evalbytes',
    'exists',
    'exp',
    'gethostbyname',
    'getnetbyname',
    'getpgrp',
    'gmtime',
    'hex',
    'int',
    'lc',
    'lcfirst',
    'length',
    'localtime',
    'lock',
    'oct',
    'ord',
    'pos',
    'quotemeta',
    'ref',
    'scalar',
    'select',
    'uc',
    'ucfirst',
    'undef',
    'untie'
]);
// The list operators that can operate on a block (grep, map, join, sort, and unpack) are handled separately.
const listOperators = new Set([
    'chomp',
    'chop',
    'crypt',
    'die',
    'getpriority',
    'index',
    'pack',
    'pipe',
    'reverse',
    'rindex',
    'setpgrp',
    'setpriority',
    'split',
    'sprintf',
    'substr',
    'tie',
    'vec',
    'warn'
]);

const pgVariables = new Set([
    'BR',
    'BRBR',
    'PAR',
    'LQ',
    'RQ',
    'BM',
    'EM',
    'BDM',
    'EDM',
    'LTS',
    'GTS',
    'LTE',
    'GTE',
    'SOL',
    'SOLUTION',
    'HINT',
    'COMMENT',
    'US',
    'SPACE',
    'NBSP',
    'NDASH',
    'MDASH',
    'BLABEL',
    'ELABEL',
    'BBOLD',
    'EBOLD',
    'BITALIC',
    'EITALIC',
    'BUL',
    'EUL',
    'BCENTER',
    'ECENTER',
    'BLTR',
    'ELTR',
    'BKBD',
    'EKBD',
    'HR',
    'LBRACE',
    'RBRACE',
    'LB',
    'RB',
    'DOLLAR',
    'PERCENT',
    'CARET',
    'PI',
    'E',
    'LATEX',
    'TEX',
    'APOS',
    'showPartialCorrectAnswers',
    'refreshCachedImages',
    'ITEM',
    'ITEMSEP',
    'displayMode',
    'problemSeed'
]);
// Note: Keep these in alphabetical order within sections to that
// it is easier to compare to the pgOperatorCompletions map.
const pgOperators = new Set([
    // PG.pl
    'ADD_CSS_FILE',
    'ADD_JS_FILE',
    'ANS',
    'AskSage',
    'DOCUMENT',
    'ENDDOCUMENT',
    'HEADER_TEXT',
    'insertGraph',
    'loadMacros',
    'maketext',
    'NAMED_ANS',
    'POST_HEADER_TEXT',
    'SET_PROBLEM_LANGUAGE',
    'SET_PROBLEM_TEXTDIRECTION',
    // core/Value.pl
    'ColumnVector',
    'Complex',
    'Compute',
    'Context',
    'Formula',
    'Infinity',
    'Interval',
    'List',
    'Matrix',
    'Point',
    'Real',
    'Set',
    'String',
    'Union',
    'Vector',
    // core/PGcommonFunctions.pl
    'abs',
    'acos',
    'acosh',
    'acot',
    'acoth',
    'acsc',
    'acsch',
    'arccos',
    'arccosh',
    'arccot',
    'arccoth',
    'arccsc',
    'arccsch',
    'arcsec',
    'arcsech',
    'arcsin',
    'arcsinh',
    'arctan',
    'arctanh',
    'asec',
    'asech',
    'asin',
    'asinh',
    'atan',
    'atan2',
    'atanh',
    'C',
    'Comb',
    'cos',
    'cosh',
    'cot',
    'coth',
    'csc',
    'csch',
    'ln',
    'log',
    'logten',
    'P',
    'Perm',
    'sec',
    'sech',
    'sgn',
    'sin',
    'sinh',
    'sqrt',
    'tan',
    'tanh',
    // core/PGauxiliaryFunctions.pl
    'ceil',
    'fact',
    'floor',
    'gcd',
    'gcf',
    'isPrime',
    'max',
    'min',
    'preformat',
    'random_coprime',
    'random_pairwise_coprime',
    'random_subset',
    'reduce',
    'repeated',
    'Round',
    'round',
    'step',
    // core/PGbasicmacros.pl
    'begintable',
    'closeDiv',
    'closeSpan',
    'COMMENT',
    'embedPDF',
    'endtable',
    'FEQ',
    'helpLink',
    'HINT',
    'htmlLink',
    'iframe',
    'image',
    'knowlLink',
    'lex_sort',
    'list_random',
    'MODES',
    'non_zero_random',
    'num_sort',
    'OL',
    'openDiv',
    'openSpan',
    'PGsort',
    'random',
    'row',
    'SOLUTION',
    'STATEMENT',
    'TEXT',
    'uniq',
    'video',
    // core/PGessaymacros.pl
    'essay_box',
    'essay_cmp',
    'explanation_box',
    // core/RserveClient.pl
    'rserve_eval',
    'rserve_finish',
    'rserve_finish_plot',
    'rserve_get_file',
    'rserve_query',
    'rserve_start',
    'rserve_start_plot',
    // core/externalData.pl
    'store_number',
    'store_number_list',
    'store_string',
    // core/sage.pl
    'Sage',
    // core/weightedGrader.pl
    'CREDIT_ANS',
    'install_weighted_grader',
    'NAMED_WEIGHTED_ANS',
    'WEIGHTED_ANS',
    // context/*.pl
    'Assignment',
    'Boolean',
    'Congruence',
    'Currency',
    'Cycle',
    'Fraction',
    'Inequality',
    'Integer',
    'Letters',
    'Ordering',
    'Partition',
    'Percent',
    'Permutation',
    'phi',
    'PiecewiseFunction',
    'primeFactorization',
    'randomPrime',
    'Reaction',
    'root',
    'ScientificNotation',
    'SetBuilder',
    'tau',
    // parser/*.pl
    'AutoStrings',
    'CheckboxList',
    'DefineStrings',
    'DifferenceQuotient',
    'DropDown',
    'DropDownTF',
    'FormulaAnyVar',
    'FormulaUpToConstant',
    'FormulaWithUnits',
    'ImplicitEquation',
    'ImplicitPlane',
    'LinearInequality',
    'LinearRelation',
    'MultiAnswer',
    'NumberWithUnits',
    'OneOf',
    'ParametricLine',
    'ParametricPlane',
    'parserFunction',
    'PopUp',
    'RadioButtons',
    'RadioMultiAnswer',
    'SolutionFor',
    'WordCompletion',
    // graph/*.pl
    'add_functions',
    'closed_circle',
    'createLaTeXImage',
    'createTikZImage',
    'CylindricalPlot3D',
    'Graph3D',
    'GraphTool',
    'init_graph',
    'Live3Ddata',
    'Live3Dfile',
    'LiveGraphics3D',
    'open_circle',
    'ParametricCurve3D',
    'ParametricSurface3D',
    'plot_functions',
    'RectangularPlot3DAnnularDomain',
    'RectangularPlot3DRectangularDomain',
    'VectorField2D',
    'VectorField3D',
    // math/*.pl
    'apply_fraction_to_matrix_entries',
    'change_matrix_entry',
    'DraggableProof',
    'DraggableSubsets',
    'elem_matrix_row_add',
    'elem_matrix_row_mult',
    'elem_matrix_row_switch',
    'factoringMethods',
    'GL2Z',
    'GL3Z',
    'GL4Z',
    'lp_current_value',
    'lp_display',
    'lp_display_mm',
    'lp_display_pivot',
    'lp_pivot_element',
    'lp_solve',
    'random_diag_matrix',
    'random_inv_matrix',
    'rcef',
    'row_add',
    'row_mult',
    'row_switch',
    'rref',
    'SL2Z',
    'SL3Z',
    'SL4Z',
    // misc/*.pl
    'DataTable',
    'LayoutTable',
    'QuickMatrixEntry',
    'randomPerson'
]);
// All PG operators in the set above that are not in the map below will get the default template which is
// `${operator}(\${})\${}`.  So the map below is for operators for which a different template than the default or
// multiple templates are deisred. Note that anything in the map below and not in the set above will be ignored. So make
// sure anything in this map is also in the set above.
const pgOperatorCompletions = new Map([
    // PG.pl
    ['DOCUMENT', ['DOCUMENT();']],
    ['ENDDOCUMENT', ['ENDDOCUMENT();']],
    ['loadMacros', ['loadMacros(qw{\n\tPGstandard.pl\n\tPGML.pl${}\n\tPGcourse.pl\n});${}', 'loadMacros(${});${}']],
    // core/PGbasicmacros.pl
    ['list_random', ['list_random(${list})${}']],
    ['non_zero_random', ['non_zero_random(${start}, ${end})${}', 'non_zero_random(${start}, ${end}, ${increment})${}']],
    ['random', ['random(${start}, ${end})${}', 'random(${start}, ${end}, ${increment})${}']]
]);

// This file was generated by lezer-generator. You probably shouldn't edit it.
const automaticSemicolon = 272,
  statementEnd = 273,
  UnrestrictedIdentifier = 1,
  SpecialScalarVariable = 2,
  NamedUnaryOperator = 3,
  ListOperator = 4,
  PGOperator = 5,
  HeredocStartIdentifier = 6,
  LaTeXImageCodeStart = 7,
  uninterpolatedHeredocStart = 274,
  interpolatedHeredocStart = 275,
  HeredocEndIdentifier = 8,
  InterpolatedStringContent = 9,
  EscapeSequence = 10,
  afterInterpolation = 276,
  packageNamePart = 277,
  afterPackageName = 278,
  QuoteLikeStartDelimiter = 11,
  QuoteLikeSeparatorDelimiter = 12,
  QuoteLikeEndDelimiter = 13,
  StringContent = 14,
  QWElement = 15,
  patternMatchStart = 279,
  RegexOptions = 16,
  regexEnd = 280,
  PodDirective = 17,
  PodContent = 18,
  PodCut = 19,
  endDataBlock = 281,
  BeginPG = 20,
  PGMLContent = 21,
  PGTextContent = 22,
  EndPG = 23,
  PGVariable = 24,
  q = 47,
  qq = 50,
  qw = 66,
  m = 79,
  qr = 83,
  s = 85,
  tr = 87,
  y = 88,
  Prototype = 108,
  ENDDOCUMENT = 164;

// ' ', \t
const isHWhitespace = (ch) => ch == 32 || ch == 9;
// ' ', \t, \n, \r
const isWhitespace = (ch) => isHWhitespace(ch) || ch == 10 || ch == 13;
// Returns the position of the first non-whitespace character in the text string.
const skipSpace = (text, i = 0) => {
    while (i < text.length && isWhitespace(text.charCodeAt(i)))
        ++i;
    return i;
};
const isUpperCaseASCIILetter = (ch) => ch >= 65 && ch <= 90;
const isLowerCaseASCIILetter = (ch) => ch >= 97 && ch <= 122;
const isASCIILetter = (ch) => isLowerCaseASCIILetter(ch) || isUpperCaseASCIILetter(ch);
const isDigit = (ch) => ch >= 48 && ch <= 55;
const isIdentifierChar = (ch) => ch == 95 /* _ */ || isASCIILetter(ch) || isDigit(ch);
const isVariableStartChar = (ch) => ch == 95 /* _ */ || isASCIILetter(ch);
// !"$%&'()*+,-./0123456789:;<=>?@[\]`~
// For arrays this is only used for interpolation and in that case only @$, @+, @-, and @1 .. @9 are allowed.
const isSpecialVariableChar = (ch, arrayType = false) => arrayType
    ? ch == 36 || ch == 43 || ch == 45 || (ch >= 49 && ch <= 57)
    : (ch >= 33 && ch != 35 && ch <= 64) || ch == 91 || ch == 92 || ch == 93 || ch == 96 || ch == 126;
/* 0-9, a-f, A-F */
const isHex = (ch) => (ch >= 48 && ch <= 55) || (ch >= 97 && ch <= 102) || (ch >= 65 && ch <= 70);

const isRegexOptionChar = (ch, regexType) => {
    if (regexType === tr || regexType === y) {
        if (ch == 99 /* c */ || ch == 100 /* d */ || ch == 115 /* s */ || ch == 114 /* r */)
            return true;
        return false;
    }
    if (ch == 109 /* m */ ||
        ch == 115 /* s */ ||
        ch == 105 /* i */ ||
        ch == 120 /* x */ ||
        ch == 112 /* p */ ||
        ch == 111 /* o */ ||
        ch == 100 /* d */ ||
        ch == 117 /* u */ ||
        ch == 97 /* a */ ||
        ch == 108 /* l */ ||
        ch == 110 /* n */)
        return true;
    if ((regexType === m || regexType === s) && (ch == 103 /* g */ || ch == 99) /* c */)
        return true;
    if (regexType == s && (ch == 101 /* e */ || ch == 114) /* r */)
        return true;
    return false;
};
const gobbleWhitespace = (input) => {
    while (isWhitespace(input.next))
        input.advance();
};
const beginPGPrefix = [66, 69, 71, 73, 78, 95]; // BEGIN_
const pgVariants = [
    [80, 71, 77, 76, 95, 72, 73, 78, 84], // PGML_HINT
    [80, 71, 77, 76, 95, 83, 79, 76, 85, 84, 73, 79, 78], // PGML_SOLUTION
    [80, 71, 77, 76], // PGML
    [84, 69, 88, 84], // TEXT
    [72, 73, 78, 84], // HINT
    [83, 79, 76, 85, 84, 73, 79, 78] // SOLUTION
];
const endPGPrefix = [69, 78, 68, 95]; // END_
const tikz = [84, 73, 75, 90]; // TIKZ
const latexImage = [76, 65, 84, 69, 88, 95, 73, 77, 65, 71, 69]; // LATEX_IMAGE
const heredocQueue = [];
// Base class that all tracker contexts extend.
class Context {
    type;
    parent;
    stackPos;
    // Used by quote and quoteLike operators.
    startDelimiter;
    endDelimiter;
    // Used by quoteLike operators.
    quoteLikeType;
    nestLevel = 0;
    // Used by heredocs.
    tag;
    indented = false;
    inBody = false;
    newlinePos;
    // Used by any interpolating context.
    interpolating = false;
    constructor(type, parent = null, stackPos = -1, options = {}) {
        this.type = type;
        this.parent = parent;
        this.stackPos = stackPos;
        this.quoteLikeType = options.quoteLikeType;
        this.interpolating = options.interpolating ?? true;
        this.indented = options.indented ?? false;
        if (options.tag)
            this.tag = options.tag;
        if (typeof options.startDelimiter === 'number')
            this.setStartAndEndDelimiters(options.startDelimiter);
        if (typeof options.newlinePos === 'number')
            this.newlinePos = options.newlinePos;
    }
    setStartAndEndDelimiters(start) {
        this.startDelimiter = start;
        if (start == 40 /* ( */) {
            this.endDelimiter = 41 /* ) */;
        }
        else if (start == 60 /* < */) {
            this.endDelimiter = 62 /* > */;
        }
        else if (start == 91 /* [ */) {
            this.endDelimiter = 93 /* ] */;
        }
        else if (start == 123 /* { */) {
            this.endDelimiter = 125 /* } */;
        }
        else
            this.endDelimiter = start;
    }
    atEnd(input) {
        if (this.type === 'heredoc') {
            if (!this.tag || input.peek(-1) !== 10)
                return false;
            let pos = 0;
            if (this.indented) {
                let nextChar = input.peek(pos);
                while (isWhitespace(nextChar))
                    nextChar = input.peek(++pos);
            }
            for (const tagCh of this.tag) {
                if (input.peek(pos++) !== tagCh)
                    return false;
            }
            if (input.peek(pos) < 0 || input.peek(pos) == 10)
                return pos;
            return false;
        }
        else if (this.type === 'pg') {
            if (!this.tag || input.peek(-1) !== 10)
                return false;
            let pos = 0;
            let ch = input.peek(pos);
            while (isHWhitespace(ch))
                ch = input.peek(++pos);
            for (const tagCh of this.tag) {
                if (ch !== tagCh)
                    return false;
                ch = input.peek(++pos);
            }
            while (isHWhitespace(ch) || ch == 59 /* ; */)
                ch = input.peek(++pos);
            if (ch == 10 || ch < 0)
                return pos;
        }
        else {
            return input.next === this.endDelimiter ? 1 : false;
        }
    }
}
const contextTracker = new ContextTracker({
    start: new Context('root'),
    shift(context, term, stack, input) {
        if (term === q || term === qq || term === qw) {
            return new Context('quoteLike', context, stack.pos, { quoteLikeType: term });
        }
        else if (term === m || term === qr || term === s || term === tr || term === y) {
            return new Context('quoteLike&regex', context, stack.pos, { quoteLikeType: term });
        }
        else if (term === HeredocStartIdentifier) {
            let pos = 0;
            const indented = input.next == 126 && input.peek(1) != 126; /* ~ follows but not ~~ */
            if (indented)
                ++pos;
            let haveWhitespace = false;
            while (isWhitespace(input.peek(pos))) {
                haveWhitespace = true;
                ++pos;
            }
            const quote = input.peek(pos) == 39 /* ' */ || input.peek(pos) == 34 /* " */ ? input.peek(pos) : undefined;
            if (!quote && haveWhitespace)
                return context;
            if (quote)
                ++pos;
            const escapedTag = !quote && input.peek(pos) == 126 && input.peek(pos + 1) == 126; /* ~ */
            if (escapedTag)
                pos += 2;
            if (escapedTag && (haveWhitespace || isWhitespace(input.peek(pos))))
                return context;
            if (!isIdentifierChar(input.peek(pos)))
                return context;
            const tag = [input.peek(pos++)];
            for (;;) {
                const next = input.peek(pos++);
                if (!isIdentifierChar(next))
                    break;
                tag.push(next);
            }
            if (heredocQueue[0]?.stackPos !== stack.pos) {
                while (input.peek(pos) != 10 /* \n */ && input.peek(pos) >= 0)
                    ++pos;
                heredocQueue.unshift(new Context('heredoc', context, stack.pos, {
                    tag,
                    interpolating: (!quote || quote == 34 || quote == 96) && !escapedTag,
                    indented,
                    newlinePos: input.pos + pos
                }));
            }
            return context;
        }
        else if (term === LaTeXImageCodeStart) {
            let pos = 0;
            while (isHWhitespace(input.peek(pos)))
                ++pos;
            const isLatexImage = beginPGPrefix.concat(latexImage).every((l, index) => l == input.peek(pos + index));
            if (!isLatexImage && !beginPGPrefix.concat(tikz).every((l, index) => l == input.peek(pos + index)))
                return context;
            if (heredocQueue.at(-1)?.stackPos !== stack.pos) {
                while (input.peek(pos) != 10 /* \n */ && input.peek(pos) >= 0)
                    ++pos;
                heredocQueue.unshift(new Context('heredoc', context, stack.pos, {
                    tag: endPGPrefix.concat(isLatexImage ? latexImage : tikz),
                    interpolating: true,
                    newlinePos: input.pos + pos
                }));
            }
            return context;
        }
        else if (term === patternMatchStart) {
            if (input.next == 47 /* / */)
                return new Context('regex', context, stack.pos, { startDelimiter: 47, quoteLikeType: m });
        }
        else if (term === BeginPG) {
            let pos = -1;
            while (isHWhitespace(input.peek(++pos)))
                ;
            if (!beginPGPrefix.every((l, index) => l == input.peek(pos + index)))
                return context;
            pos += beginPGPrefix.length;
            const pgVariant = pgVariants.findIndex((variant) => variant.every((l, index) => l == input.peek(pos + index)));
            if (pgVariant === -1)
                return context;
            return new Context('pg', context, stack.pos, {
                tag: endPGPrefix.concat(pgVariants[pgVariant]),
                interpolating: false
            });
        }
        else if ((context.type !== 'quote' || input.next != context.endDelimiter) &&
            term !== InterpolatedStringContent &&
            context.type !== 'pg') {
            if (input.next == 34 /* " */) {
                return new Context('quote', context, stack.pos, { startDelimiter: input.next });
            }
        }
        if (context.type.startsWith('quoteLike') && term === QuoteLikeStartDelimiter) {
            let pos = 1;
            let startDelimiter = input.next;
            while (isWhitespace(startDelimiter))
                startDelimiter = input.peek(pos++);
            if (startDelimiter >= 0) {
                context.setStartAndEndDelimiters(startDelimiter);
                // Note that q and qw are not interpolated, but don't need to set this because they can't accept
                // interpolated content.
                if (context.quoteLikeType === tr ||
                    context.quoteLikeType === y ||
                    ((context.quoteLikeType === m || context.quoteLikeType === qr || context.quoteLikeType === s) &&
                        startDelimiter == 39) /* ' */)
                    context.interpolating = false;
                return context;
            }
        }
        if ((term === interpolatedHeredocStart || term === uninterpolatedHeredocStart) &&
            heredocQueue.slice(-1)[0]?.newlinePos !== context.newlinePos) {
            const heredocContext = heredocQueue.pop();
            if (heredocContext) {
                heredocContext.parent = context;
                heredocContext.inBody = true;
                return heredocContext;
            }
        }
        if (context.parent &&
            ((context.type === 'quote' && input.next === context.endDelimiter) ||
                (context.type === 'quoteLike' && term === QuoteLikeEndDelimiter) ||
                (context.type === 'regex' && term === regexEnd) ||
                (context.type === 'quoteLike&regex' && term === regexEnd) ||
                (context.type === 'heredoc' && term === HeredocEndIdentifier) ||
                (context.type === 'pg' && term === EndPG))) {
            return context.parent;
        }
        return context;
    },
    strict: false
});
const semicolon = new ExternalTokenizer((input, stack) => {
    if (stack.canShift(statementEnd) &&
        !heredocQueue.length &&
        (input.peek(-1) == 59 /* ; */ ||
            (input.peek(-1) != 59 /* ; */ && (input.next < 0 || input.next == 10))) /* \n */) {
        input.acceptToken(statementEnd, 0);
    }
    else if (stack.canShift(automaticSemicolon) &&
        input.next != 59 /* ; */ &&
        (input.next < 0 || input.next == 125) /* } */)
        input.acceptToken(automaticSemicolon);
});
const unrestrictedIdentifier = new ExternalTokenizer((input, stack) => {
    if (stack.canShift(UnrestrictedIdentifier)) {
        gobbleWhitespace(input);
        if (input.next < 0 || isASCIILetter(input.next) || input.next == 95 /* _ */)
            return;
        while (input.next >= 0 && isIdentifierChar(input.next))
            input.advance();
        input.acceptToken(UnrestrictedIdentifier);
    }
});
// Note that is only to pick up special variables that won't be considered as a ScalarVariable already.
const specialScalarVariable = new ExternalTokenizer((input, stack) => {
    if (stack.canShift(SpecialScalarVariable) && input.next == 36 /* $ */) {
        if (stack.canShift(Prototype))
            return;
        const first = input.peek(1);
        const second = input.peek(2);
        if (first == 123 /* { */ && isSpecialVariableChar(second) && input.peek(3) == 125 /* } */) {
            input.acceptToken(SpecialScalarVariable, 4);
            return;
        }
        if (first == 123 /* { */ && second == 94 /* ^ */) {
            let pos = 3, ch;
            while ((isUpperCaseASCIILetter((ch = input.peek(pos))) || ch == 95) /* _ */ && ch != 125 /* } */)
                ++pos;
            if (ch == 125) {
                input.acceptToken(SpecialScalarVariable, pos + 1);
                return;
            }
        }
        if (first == 94 /* ^ */ && (isUpperCaseASCIILetter(second) || second == 95) /* _ */) {
            input.acceptToken(SpecialScalarVariable, 3);
            return;
        }
        if (!isSpecialVariableChar(first))
            return;
        if (first == 36 /* $ */ && isIdentifierChar(second))
            return;
        input.acceptToken(SpecialScalarVariable, 2);
        return;
    }
});
// Finds the longest word coming up in the stream (word characters include underscores here).  Returns an array
// containing the word and the ascii character code of the next character after it.
const peekWord = (input) => {
    let pos = 0;
    let word = '', nextChar;
    while (isASCIILetter((nextChar = input.peek(pos))) || nextChar == 95 /* _ */) {
        ++pos;
        word += String.fromCharCode(nextChar);
    }
    return [word, nextChar];
};
// Finds the longest lower case word coming up in the stream.  Returns an array
// containing the word and the ascii character code of the next character after it.
const peekLCWord = (input) => {
    let pos = 0;
    let word = '', nextChar;
    while (isLowerCaseASCIILetter((nextChar = input.peek(pos)))) {
        ++pos;
        word += String.fromCharCode(nextChar);
    }
    return [word, nextChar];
};
const builtinOperator = new ExternalTokenizer((input, stack) => {
    if (stack.canShift(PGOperator)) {
        const [word, nextChar] = peekWord(input);
        if (word.startsWith('ENDDOCUMENT'))
            input.acceptToken(ENDDOCUMENT, 11);
        else if (pgOperators.has(word) && !isIdentifierChar(nextChar))
            input.acceptToken(PGOperator, word.length);
    }
    if (stack.canShift(NamedUnaryOperator)) {
        const [word, nextChar] = peekLCWord(input);
        if (namedUnaryOperators.has(word) && !isIdentifierChar(nextChar))
            input.acceptToken(NamedUnaryOperator, word.length);
    }
    if (stack.canShift(ListOperator)) {
        const [word, nextChar] = peekLCWord(input);
        if (listOperators.has(word) && !isIdentifierChar(nextChar))
            input.acceptToken(ListOperator, word.length);
    }
});
const heredoc = new ExternalTokenizer((input, stack) => {
    if (stack.canShift(HeredocStartIdentifier)) {
        const indented = input.next == 126 && input.peek(1) != 126; /* ~ follows but not ~~ */
        if (indented)
            input.advance();
        gobbleWhitespace(input);
        const quote = input.next == 39 /* ' */ || input.next == 34 /* " */ ? input.next : undefined;
        if (!quote && isWhitespace(input.peek(-1)))
            return;
        if (quote)
            input.advance();
        const escapedTag = !quote && input.next == 126 && input.peek(1) == 126; /* ~ */
        if (escapedTag && (isWhitespace(input.peek(-1)) || isWhitespace(input.peek(2))))
            return;
        if (escapedTag)
            input.advance(2);
        if (!isIdentifierChar(input.next))
            return;
        for (;;) {
            input.advance();
            if (!isIdentifierChar(input.next))
                break;
        }
        if (quote) {
            if (input.next != quote)
                return;
            input.advance();
        }
        input.acceptToken(HeredocStartIdentifier);
        return;
    }
    if (stack.canShift(LaTeXImageCodeStart)) {
        if (input.peek(-2) != 45 /* - */ || input.peek(-1) != 62 /* > */)
            return;
        while (isHWhitespace(input.next))
            input.advance();
        const isLatexImage = beginPGPrefix.concat(latexImage).every((l, index) => l == input.peek(index));
        if (isLatexImage || beginPGPrefix.concat(tikz).every((l, index) => l == input.peek(index))) {
            const tokenLength = beginPGPrefix.length + (isLatexImage ? latexImage.length : tikz.length);
            // Verify that only horizontal whitespace and semicolons are on the remainder of the line.
            let pos = tokenLength;
            let ch = input.peek(pos);
            while (ch >= 0 && (isHWhitespace(ch) || ch == 59) /* ; */)
                ch = input.peek(++pos);
            if (ch <= 0 || ch == 10 /* \n */)
                input.acceptToken(LaTeXImageCodeStart, tokenLength);
        }
    }
    if (!(stack.context instanceof Context))
        return;
    if (input.next == 10 /* \n */ &&
        heredocQueue.length &&
        heredocQueue.slice(-1)[0]?.newlinePos !== stack.context.newlinePos &&
        (stack.canShift(uninterpolatedHeredocStart) || stack.canShift(interpolatedHeredocStart))) {
        if (heredocQueue.slice(-1)[0]?.interpolating)
            input.acceptToken(interpolatedHeredocStart, 1);
        else
            input.acceptToken(uninterpolatedHeredocStart, 1);
        return;
    }
    if (stack.context.type !== 'heredoc' || !stack.context.inBody)
        return;
    if (stack.context.interpolating) {
        const endCount = stack.context.atEnd(input);
        if (endCount) {
            input.acceptToken(HeredocEndIdentifier, endCount);
            return;
        }
        else if (input.next < 0) {
            input.acceptToken(HeredocEndIdentifier);
            return;
        }
    }
    else {
        const endCount = stack.context.atEnd(input);
        if (endCount) {
            input.acceptToken(HeredocEndIdentifier, endCount);
            return;
        }
        for (;;) {
            if (input.next < 0) {
                input.acceptToken(HeredocEndIdentifier);
                return;
            }
            if (input.next != 10 /* \n */) {
                input.advance();
                continue;
            }
            input.advance();
            const endCount = stack.context.atEnd(input);
            if (endCount) {
                input.acceptToken(StringContent);
                return;
            }
            input.advance();
        }
    }
}, { contextual: true });
const scanEscape = (input) => {
    const after = input.peek(2);
    // Restricted range octal character
    if (after >= 48 && after <= 55 /* 0-7 */) {
        let size = 3, next;
        while (size < 6 && (next = input.peek(size)) >= 48 && next <= 55)
            ++size;
        return size;
    }
    // Restricted range hexidecimal character
    if (after == 120 /* x */ && isHex(input.peek(3)))
        return isHex(input.peek(4)) ? 5 : 4;
    // Hexidecimal character
    if (after == 120 /* x */ && input.peek(3) == 123 /* { */) {
        // FIXME: There could be optional blanks at the beginning and end inside the braces.
        for (let size = 4;; ++size) {
            const next = input.peek(size);
            if (next == 125 /* } */)
                return size + 1;
            if (!isHex(next))
                break;
        }
    }
    // This could be any named unicode character or character sequence.
    if (after == 78 /* N */ && input.peek(3) == 123 /* { */) {
        for (let size = 4;; ++size) {
            const next = input.peek(size);
            if (next == 125 /* } */)
                return size + 1;
            if (next < 0)
                break;
        }
    }
    // Octal character
    if (after == 111 /* o */ && input.peek(3) == 123 /* { */) {
        for (let size = 4;; ++size) {
            const next = input.peek(size);
            if (next == 125 /* } */)
                return size + 1;
            if (next < 48 || next > 55 /* not 0-7 */)
                break;
        }
    }
    return 3;
};
const interpolated = new ExternalTokenizer((input, stack) => {
    if (!(stack.context instanceof Context) || !stack.context.interpolating)
        return;
    let content = false;
    for (;; content = true) {
        if (heredocQueue.length &&
            (stack.context.type !== 'heredoc' ||
                heredocQueue.slice(-1)[0]?.newlinePos !== stack.context.newlinePos) &&
            input.next == 10 /* \n */) {
            break;
        }
        if ((stack.context.nestLevel == 0 && stack.context.atEnd(input)) ||
            input.next < 0 ||
            ((input.next == 36 /* $ */ || input.next == 64) /* @ */ &&
                (isVariableStartChar(input.peek(1)) ||
                    input.peek(1) == 123 /* { */ ||
                    (isSpecialVariableChar(input.peek(1), input.next == 64 /* @ */) &&
                        (stack.context.nestLevel > 0 || input.peek(1) !== stack.context.endDelimiter))))) {
            break;
        }
        if (stack.context.type.startsWith('quoteLike') &&
            stack.context.startDelimiter !== stack.context.endDelimiter &&
            input.next === stack.context.startDelimiter) {
            ++stack.context.nestLevel;
        }
        else if (stack.context.nestLevel > 0 && input.next === stack.context.endDelimiter) {
            --stack.context.nestLevel;
        }
        else if (input.next == 126 /* ~ */ && input.peek(1) == 126) {
            const escaped = scanEscape(input);
            if (escaped) {
                if (content)
                    break;
                else {
                    input.acceptToken(EscapeSequence, escaped);
                    return;
                }
            }
        }
        else if (!content &&
            (input.next == 91 /* [ */ ||
                input.next == 123 /* { */ ||
                (input.next == 45 /* - */ &&
                    input.peek(1) == 62 /* > */ &&
                    (input.peek(2) == 91 /* [ */ || input.peek(2) == 123))) /* { */ &&
            stack.canShift(afterInterpolation)) {
            input.acceptToken(afterInterpolation);
            return;
        }
        else if (!content &&
            input.next == 58 /* : */ &&
            input.peek(1) == 58 &&
            (stack.canShift(packageNamePart) || stack.canShift(afterPackageName))) {
            let pos = 2, ch;
            while (isIdentifierChar((ch = input.peek(pos))))
                ++pos;
            if (ch == 58 && input.peek(pos + 1) == 58)
                input.acceptToken(packageNamePart);
            else
                input.acceptToken(afterPackageName);
            return;
        }
        input.advance();
    }
    if (content)
        input.acceptToken(InterpolatedStringContent);
}, { contextual: true });
// Note that the paired delimiters enabled by the extra_paired_delimiters
// feature are not dealt with by the quote-like tokenizers.
const quoteLikeOperator = new ExternalTokenizer((input, stack) => {
    if (!(stack.context instanceof Context) || !stack.context.type.startsWith('quoteLike'))
        return;
    if (!stack.context.startDelimiter) {
        const haveWhitespace = isWhitespace(input.next);
        let pos = 1;
        let nextChar = input.next;
        while (isWhitespace(nextChar))
            nextChar = input.peek(pos++);
        if (stack.canShift(QuoteLikeStartDelimiter) && nextChar >= 0 && (!haveWhitespace || nextChar !== 35)) {
            input.acceptToken(QuoteLikeStartDelimiter, pos);
            return;
        }
        return;
    }
    if (stack.canShift(QWElement) && isWhitespace(input.next))
        gobbleWhitespace(input);
    if (stack.canShift(QuoteLikeSeparatorDelimiter) &&
        stack.context.startDelimiter === stack.context.endDelimiter &&
        input.next === stack.context.endDelimiter) {
        input.acceptToken(QuoteLikeSeparatorDelimiter, 1);
        return;
    }
    else if (stack.canShift(QuoteLikeEndDelimiter) &&
        stack.context.type === 'quoteLike&regex' &&
        stack.context.startDelimiter !== stack.context.endDelimiter &&
        input.next === stack.context.endDelimiter) {
        input.acceptToken(QuoteLikeEndDelimiter, 1);
        return;
    }
    if (stack.canShift(QuoteLikeStartDelimiter) &&
        stack.context.type === 'quoteLike&regex' &&
        stack.context.startDelimiter !== stack.context.endDelimiter) {
        let pos = 1;
        let nextChar = input.next;
        while (isWhitespace(nextChar))
            nextChar = input.peek(pos++);
        if (nextChar === stack.context.startDelimiter) {
            input.acceptToken(QuoteLikeStartDelimiter, pos);
            return;
        }
    }
    if (input.next === stack.context.endDelimiter) {
        input.acceptToken(QuoteLikeEndDelimiter, 1);
        return;
    }
    if (input.next >= 0) {
        if (stack.canShift(StringContent)) {
            while (input.next >= 0 &&
                (stack.context.nestLevel > 0 ||
                    input.next !== stack.context.endDelimiter ||
                    (input.next === stack.context.endDelimiter && input.peek(-2) == 126 && input.peek(-1) == 126))) {
                if (stack.context.startDelimiter !== stack.context.endDelimiter) {
                    if (input.next === stack.context.startDelimiter)
                        ++stack.context.nestLevel;
                    if (input.next === stack.context.endDelimiter)
                        --stack.context.nestLevel;
                }
                input.advance();
            }
            input.acceptToken(StringContent);
            return;
        }
        else if (stack.canShift(QWElement)) {
            while (input.next >= 0 &&
                !isWhitespace(input.next) &&
                (stack.context.nestLevel > 0 ||
                    input.next !== stack.context.endDelimiter ||
                    (input.next === stack.context.endDelimiter && input.peek(-1) == 126 && input.peek(-1) == 126))) {
                if (stack.context.startDelimiter !== stack.context.endDelimiter) {
                    if (input.next === stack.context.startDelimiter)
                        ++stack.context.nestLevel;
                    if (input.next === stack.context.endDelimiter)
                        --stack.context.nestLevel;
                }
                input.advance();
            }
            input.acceptToken(QWElement);
            return;
        }
    }
}, { contextual: true });
const regex = new ExternalTokenizer((input, stack) => {
    if (stack.canShift(patternMatchStart) && input.next == 47 /* / */) {
        const divisionToken = stack.parser.nodeSet.types.find((t) => t.name == '/')?.id;
        if (typeof divisionToken === 'number' && !stack.canShift(divisionToken))
            input.acceptToken(patternMatchStart);
    }
    if (!(stack.context instanceof Context) || !stack.context.type.endsWith('regex'))
        return;
    if (stack.canShift(RegexOptions) &&
        stack.context.quoteLikeType &&
        isRegexOptionChar(input.next, stack.context.quoteLikeType)) {
        while (input.next >= 0 && isRegexOptionChar(input.next, stack.context.quoteLikeType))
            input.advance();
        input.acceptToken(RegexOptions);
        return;
    }
    if (stack.canShift(regexEnd)) {
        input.acceptToken(regexEnd);
        return;
    }
}, { contextual: true });
const pod = new ExternalTokenizer((input, stack) => {
    if (stack.canShift(PodDirective)) {
        if ((input.peek(-1) == 10 /* \n */ || input.peek(-1) < 0) &&
            input.next == 61 /* = */ &&
            isASCIILetter(input.peek(1))) {
            while (input.next >= 0 && input.next != 10)
                input.advance();
            input.acceptToken(PodDirective);
            return;
        }
    }
    if (stack.canShift(PodContent)) {
        while (input.next >= 0) {
            if (input.next != 10) {
                input.advance();
                continue;
            }
            input.advance();
            if (input.peek(0) == 61 /* = */ &&
                input.peek(1) == 99 /* c */ &&
                input.peek(2) == 117 /* u */ &&
                input.peek(3) == 116 /* t */ &&
                (input.peek(4) == 10 || input.peek(4) < 0))
                break;
        }
        input.acceptToken(PodContent);
        return;
    }
    if (stack.canShift(PodCut)) {
        if ((input.peek(-1) == 10 || input.peek(-1) < 0) &&
            input.peek(0) == 61 /* = */ &&
            input.peek(1) == 99 /* c */ &&
            input.peek(2) == 117 /* u */ &&
            input.peek(3) == 116 /* t */ &&
            (input.peek(4) == 10 || input.peek(4) < 0)) {
            input.acceptToken(PodCut, 4);
            return;
        }
    }
});
const endData = new ExternalTokenizer((input, stack) => {
    if (stack.canShift(endDataBlock)) {
        while (input.advance() >= 0)
            ;
        input.acceptToken(endDataBlock);
    }
});
const pgText$1 = new ExternalTokenizer((input, stack) => {
    if (stack.canShift(BeginPG)) {
        let ch = input.peek(-1);
        if (ch >= 0 && ch != 10)
            return;
        let pos = 0;
        ch = input.next;
        while (isHWhitespace(ch))
            ch = input.peek(++pos);
        if (!beginPGPrefix.every((l, index) => l == input.peek(pos + index)))
            return;
        pos += beginPGPrefix.length;
        const pgVariant = pgVariants.findIndex((variant) => variant.every((l, index) => l == input.peek(pos + index)));
        if (pgVariant === -1)
            return;
        pos += pgVariants[pgVariant].length;
        ch = input.peek(pos);
        while (isHWhitespace(ch) || ch == 59 /* ; */)
            ch = input.peek(++pos);
        if (ch == 10 || ch < 0)
            input.acceptToken(BeginPG, pos);
    }
    if (!(stack.context instanceof Context) || stack.context.type !== 'pg' || !stack.context.tag)
        return;
    if (stack.canShift(PGMLContent) || stack.canShift(PGTextContent)) {
        if (input.next != 10)
            return;
        // eslint-disable-next-line @typescript-eslint/no-unnecessary-condition
        while (input.next >= 0) {
            if (stack.context.atEnd(input))
                break;
            input.advance();
        }
        if (stack.context.tag[4] == 80)
            input.acceptToken(PGMLContent);
        else
            input.acceptToken(PGTextContent);
    }
    else if (stack.canShift(EndPG)) {
        const endPos = stack.context.atEnd(input);
        if (endPos)
            input.acceptToken(EndPG, endPos);
    }
}, { contextual: true });
const pgVariable = (name) => {
    return pgVariables.has(name.replace(/^\$/, '')) || pgVariables.has(name.replace(/^\$\{\s*(\w*)\s*\}/, '$1'))
        ? PGVariable
        : -1;
};

const pgHighlighting = styleTags({
    'do continue else elsif for foreach goto if last next redo return unless until when while': tags.controlKeyword,
    'bless local my our state sub': tags.definitionKeyword,
    package: tags.moduleKeyword,
    prototype: tags.modifier,
    'NamedUnaryOperator ListOperator Eval each grep': tags.function(tags.keyword),
    'join keys map pop push shift sort splice time times': tags.function(tags.keyword),
    'unpack unshift values wantarray': tags.special(tags.keyword),
    'ENDDOCUMENT PGOperator': tags.special(tags.function(tags.operatorKeyword)),
    'BeginPG EndPG': tags.keyword,
    'BEGIN CHECK END INIT UNITCHECK': tags.processingInstruction,
    '__FILE__ __LINE__ __PACKAGE__ __SUB__': tags.literal,
    'ScalarVariable SpecialScalarVariable ArrayVariable HashVariable': tags.variableName,
    'ArrayLength/Identifier Prototype Constant/Identifier': tags.variableName,
    'PackageVariable/PackageVariableName/... PackageName/ScalarVariable': tags.variableName,
    PGVariable: tags.special(tags.variableName),
    'PackageName/Identifier PackageName/UnrestrictedIdentifier': tags.namespace,
    'PackageName/ArrayVariable PackageName/HashVariable SUPER': tags.namespace,
    'NamedType/...': tags.typeName,
    Name: tags.name,
    'Label/Identifier LabelName/Identifier STDIN STDERR STDOUT IOOperator/Identifier': tags.labelName,
    'MemberExpression/Identifier': tags.propertyName,
    'MemberExpression/ScalarVariable': tags.special(tags.propertyName),
    'FunctionName/PackageName/... FunctionName/Identifier': tags.function(tags.definition(tags.variableName)),
    UpdateOp: tags.updateOperator,
    'ArithOp "*" % "/"': tags.arithmeticOperator,
    'LogicOp and not or xor : FileTestOp': tags.logicOperator,
    BitOp: tags.bitwiseOperator,
    '<<': tags.special(tags.bitwiseOperator),
    'CompareOp < lt gt le ge eq ne cmp': tags.compareOperator,
    '=': tags.definitionOperator,
    '$ $# @ % & "*" ArrowOperator \\': tags.derefOperator,
    'ScalarDereference/{ ScalarDereference/}': tags.derefOperator,
    'ArrayDereference/{ ArrayDereference/}': tags.derefOperator,
    'HashDereference/{ HashDereference/}': tags.derefOperator,
    'FunctionDereference/{ FunctionDereference/}': tags.derefOperator,
    'TypeGlobDereference/{ TypeGlobDereference/}': tags.derefOperator,
    'ConcatOp BindingOp RangeOp RefOp x': tags.operator,
    Comment: tags.lineComment,
    Integer: tags.integer,
    Float: tags.float,
    'StringSingleQuoted StringDoubleQuoted q qq qx */StringContent */InterpolatedStringContent': tags.string,
    '*/QuoteLikeStartDelimiter QuoteLikeSeparatorDelimiter */QuoteLikeEndDelimiter': tags.string,
    'qw QWListContent/... Pair/Identifier HashAccessVariable/Identifier Version': tags.string,
    'HeredocInitializer/... HeredocEndIdentifier Glob LaTeXImageCodeStart': tags.string,
    'm qr s tr y RegexOptions': tags.special(tags.string),
    'PodStatement EndDataStatement/... EndDocument': tags.blockComment,
    EscapeSequence: tags.escape,
    'Comma FatComma': tags.punctuation,
    '( )': tags.paren,
    '[ ]': tags.squareBracket,
    '{ }': tags.brace,
    '; :: :': tags.separator
});

// This file was generated by lezer-generator. You probably shouldn't edit it.
const spec_Identifier = {__proto__:null,q:95, qq:101, __FILE__:113, __LINE__:115, __PACKAGE__:117, qw:133, m:159, qr:167, s:171, tr:175, y:177, our:203, state:205, my:207, local:209, sub:213, prototype:227, package:233, return:253, if:255, unless:257, while:259, until:261, for:263, foreach:265, elsif:273, else:277, continue:287, BEGIN:305, UNITCHECK:307, CHECK:309, INIT:311, END:313, __END__:323, __DATA__:325, ENDDOCUMENT:329, next:339, last:341, redo:343, x:353, lt:369, gt:371, le:373, ge:375, eq:379, ne:381, cmp:385, and:397, or:399, xor:401, not:409, time:417, times:419, wantarray:421, grep:427, map:429, join:431, sort:433, unpack:435, bless:447, SUPER:451, eval:753, do:469, goto:473, push:481, unshift:483, splice:485, pop:489, shift:491, each:495, values:497, keys:499};
const parser = LRParser.deserialize({
  version: 14,
  states: "#/pQcQ!>rOOP'[O`OOO*iQ!LcO'#IfOOQa'#If'#IfO+RQ!LcO'#DVO.YQMxO'#DUOOQa'#Ii'#IiOOO!b'#Il'#IlO._ONzO'#D[O/_ONnO'#D^OOO!b'#Io'#IoO/fONzO'#D_O/qQ!LcO'#IkOOQa'#Ik'#IkO3SO`O'#DaOOQa'#Ij'#IjOOQa'#Ip'#IpOOQa'#Iq'#IqOOQa'#Ir'#IrOOQa'#De'#DeO:cQ# eO'#EcO=cQ!LcO'#IhO>PQ!!zO'#DmOOO!b'#Iv'#IvOBmONzO'#DoOOQa'#Is'#IsOBxQMxO'#DrOGqQ!!zO'#DwOGxQ!>rO'#ErOLpQ!!zO'#GmOLwQ!LcO'#J]OOO!b'#Ix'#IxO! yONzO'#D|O!!UOMhO'#EOOOO!b'#Iy'#IyO!!ZONzO'#EQOOO!b'#Iz'#IzO!!fONzO'#ESOOO!b'#I|'#I|OOO!b'#I}'#I}O!!qONzO'#EUO!!|QMxO'#EXO!#[QMxO'#EZO!#jQMxO'#E]O!#xQNQO'#E_O!$^QMxO'#EaOOQd'#JP'#JPOOQd'#JQ'#JQOOQd'#JR'#JROOQd'#JS'#JSOOQd'#JO'#JOO!%TQM|O'#EdOOQ`'#JV'#JVOOQ`'#JY'#JYO!%[QMxO'#EsO!%aQMyO'#J[O!%oQMyO'#EyO!&YQMxO'#EiOOQ$c'#J^'#J^O!+TQ!!{O'#E}OOQ$b'#Ja'#JaOOQ$b'#Jb'#JbOOQ$b'#Jc'#JcOOQ$b'#Jd'#JdOOQ$b'#Je'#JeOOQ$b'#Jf'#JfO!+qQMxO'#E|O!,VQMxO'#FWO!,VQMxO'#F]O!,VQMxO'#F^O!,[QMxO'#FfO!,VQMxO'#FbO!,mQ$IdO'#FdO!-XQ!>sO'#FfO!3jQMyO'#FhO!%oQMyO'#FgOOQ`'#Jk'#JkOOQ`'#Jl'#JlOOQ`'#Jm'#JmOOQ`'#Jn'#JnOOQ`'#Jo'#JoO!3uQMxO'#FiOOQ@Y'#Fo'#FoOOQ@Y'#Fp'#FpO!3zO&jO'#FqOOQ,f'#Jq'#JqOOQ,f'#Jr'#JrO!4PQ!+OO'#FrOOQ,f'#Js'#JsO!4UQ!+OO'#FuO!4ZO7[O'#FxOOQ@Y'#JX'#JXOOQ$b'#KU'#KUOCWQ!!zO'#GmOCWQ!!zO'#GmOCWQ!!zO'#GqOOQa'#KV'#KVOOQa'#KW'#KWOOQa'#KX'#KXO!4cQ!LcO'#GsO!7tQ# eO'#GwOOQ$b'#KZ'#KZOOQ$b'#K['#K[OOQ$b'#K]'#K]OOQ$b'#K^'#K^OOQ$b'#K_'#K_OOQ$b'#KY'#KYO!:iQ!!zO'#GxO!CjQ!!zO'#GxO!H[Q!!zO'#HPOOQ$b'#HP'#HPO!:iQ!!zO'#HOOOQa'#HQ'#HQOOQ`'#K`'#K`O!HcQMxO'#HSOOQ`'#Kc'#KcO!H}QMxO'#HXO!IYQMxO'#HXO!3uQMxO'#HZOOQ$c'#H^'#H^O# dQ# eO'#H]OOQ$c'#Ke'#KeO#(rQ# eO'#H_OOQ$b'#Kf'#KfO#.cQ!!zO'#HaOOQ`'#Kh'#KhOOQ`'#Ki'#KiOOQ`'#Kj'#KjOOQ`'#Kg'#KgO#.jQMxO'#HeOOQa'#Km'#KmOOQa'#Kn'#KnOOQa'#Kl'#KlO#.uQ!LcO'#HiOOQ`'#Kq'#KqOOQ`'#Kr'#KrOOQ`'#Ks'#KsOOQ`'#Kp'#KpO#2^QMxO'#HlOOQa'#Ih'#IhOOQ@Y'#IU'#IUQcQ!>rOOO#2oO!rO'#CvO#2wONnO'#CwPOOO)CDW)CDWO#3OQ!!zO'#IwOOQa,5:f,5:fO#8WQM|O,5:fOOQa,5:g,5:gO#<uQ!!zO'#IgO#<|QMzO'#HsO#=UQMxO,59qO#=^QMzO,59pOOOO'#Ht'#HtO#=fOMhO'#ImO#=qO!bO,59vO#=yO!bO,59vO#?`O!!QO'#IfO#>OO!!QO'#IfO#?jOpO'#C{OOQa,59x,59xO#?oONnO,59xO#?vO!!QO'#IeOOO#h'#Ie'#IeOOO#h'#Hq'#HqO#AWO!!QO,59yO#A_O!bO,59yO#AdQM|O,5=pOOQa,59{,59{OCWQ!!zO,5:TO#A{QMzO'#H|O#BTQ!!zO,5;aOOQ`,5;y,5;yO#FtQ!>rO,5>lO#F{Q!LcO'#IfO#G]Q# eO'#EcO#GpQ!LbO,5?wO>PQ!!zO'#DmO#J]QMxO,5:XO#JbQ!!zO'#DxO!$lQM|O'#EdO#JiQMxO'#EiO#JtO!bO'#DqOOQa,5:Z,5:ZO#J|O!bO,5:ZOOQa,5:^,5:^O#3OQ!!zO,5:^O#KRQ!LbO'#ItO#K]QMxO,5:cO#KbQ!LcO'#ItO#KiQMxO,5:dO#KnQMyO'#JuOOQa'#Jv'#JvOOQa'#Jw'#JwOOQa'#Jx'#JxO#KxQMyO'#FzOOQ@Y'#Ju'#JuOOQ@Y'#H{'#H{O#LVQ!>rO,5;^OOQ@Z,5;^,5;^O#JbQ!!zO,5:dO#MzQ!LcO,5=XO$ yQ!!zO'#J`O!%oQMyO,5;hOCWQ!!zO,5<jOOQ$b'#Jy'#JyOCWQ!!zO,5<lO$&dQ!!zO,5<oOOQ$b'#Jz'#JzOOQ$b'#J{'#J{OOQ$b'#J|'#J|OOQ$b'#J}'#J}OCWQ!!zO'#ISO$)aQ!LcO,5<oOOQ$b'#KO'#KOOOQ$b'#KP'#KPOCWQ!!zO'#ITO$)}Q!LcO,5<oOOQ$b'#KQ'#KQOOQ$b'#Ge'#GeOOQ$b'#KR'#KROOQ$b'#KS'#KSOOQ$b'#KT'#KTOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOCWQ!!zO,5<oOOQa,5=],5=]O$*kQ!!zO,5=}O$*rO!!QO,5:hO$*|O!bO,5:hO$+RONnO,5:jO$+YO!!QO,5:lO$+dO!bO,5:lO$+iO!!QO,5:nO$+vO!bO,5:nO$+{O!bO,5:pO$,QO!bO,5:pOOQa,5:s,5:sO#3OQ!!zO,5:sOOQa,5:u,5:uO#3OQ!!zO,5:uOOQa,5:w,5:wO#3OQ!!zO,5:wOOQa,5:y,5:yO#3OQ!!zO,5:yO$3aQ# eO,5=kO$,VQ# eO,5=kOOQ$b,5=k,5=kOOQa,5:{,5:{O#3OQ!!zO,5:{O$4lQM|O,5;OOOQa,5;O,5;OO$4sQMxO,5;fO$4xQ$IeO'#EuO$5ZQ$IeO,5;_OOQ$c'#Iu'#IuO$5iQ!!{O'#IOO$6]QMyO,5?vO$6kOaO'#EwOOQ@Z'#JZ'#JZOOQ@Y,5;e,5;eO$6vQ(ChO'#EkO$7OQMxO'#EmOOQa'#Hz'#HzO#JoQMxO,5;TOOQa'#E{'#E{O$7WQMyO,5;fO!3uQMxO,5<TO$7iQ!>rO'#ErOOQa,5;T,5;TO$7yQMxO,5;TO$:eQ!LcO'#J]O$;XQMyO'#J_OOQa,5;i,5;iO>PQ!!zO'#DmO!3uQMxO,5;rO!3uQMxO,5;wO!3uQMxO,5;xO!,VQMxO,5;xO!,VQMxO,5;|O$;yQ$IdO,5<OO$BsQ!>sO,5<QO!3uQMxO,5;|O$BzQ!!{O,5;}O!3uQMxO,5<OO!,VQMxO,5<OO$CUQMxO,5<OO$C^QMxO,5<OOOQ`'#Jj'#JjO!3uQMxO'#F`OOQ@Y,5<Q,5<QO$<eQ!>sO,5<QO$CoQMxO'#IQOOQa-E<O-E<OOOQ@Y,5<R,5<ROOQ@Y,5<T,5<TO$CtO&jO,5<]OOQ@Y,5<^,5<^OOQ@Y,5<a,5<aO$CyO7[O,5<dO$DOO7[O,5<eO$DTQ!LcO,5=XO$FSQ!LcO,5=]O$IbQMxO,5=_OOQa,5=c,5=cO$IgQ!LcO'#JgOOQa'#FV'#FVOOQa,5=d,5=dO!&jQ!!zO,5=uOOQa,5=j,5=jO$JgQMxO'#DxOOQa'#Kb'#KbOOQa'#Ka'#KaO$KQQMxO'#KaOOQa,5=n,5=nOCWQ!!zO'#KbO$KXQ!LcO'#EuO$NmQ!LcO,5=sO%$OQMxO,5=sOOQa,5=w,5=wOOQa,5=y,5=yO%+RQ# eO,5={O%-rQ!LcO,5={O%/RQMxO'#KkO%/ZQMxO'#KkOOQa,5>P,5>POOQa'#Ko'#KoO%/cQMxO'#KoOOQa,5>T,5>TOOQa'#Kt'#KtO%/kQMxO'#KtOOQa,5>W,5>WOOQ@Y-E<S-E<SOOO#i,59b,59bO%/yO`O,59bO%0OONnO,59cOOO#i,59c,59cO%2iQ!LbO,5?cOOQa1G0Q1G0QOOQa1G0R1G0RO%2pQ!>sO1G1}OOQa'#HW'#HWO%9OQ!LcO1G3[O%<aQMxO1G3[O%<fQ# dO,5?RO%>tQ!LbO,5?ROOQ`,5>_,5>_OOQ`-E;q-E;qOOQa'#DW'#DWOOQa1G/[1G/[OOOO-E;r-E;rO%>{ONzO,5?XOOO!b'#Hv'#HvO%?ZO!bO1G/bOOQa1G/b1G/bO%?ZO!bO1G/bO%?cOMhO'#HrO%?hOpO,59hOOO#h,59m,59mO%?pOMhO,59mO%?xOMhO,59mOOO#h,59l,59lO%@TOMhO,59gOOQa1G/d1G/dOOO#h-E;o-E;oOOQa1G/e1G/eO%@YO!!QO1G/eO%@YO!!QO1G/eO%@aQ!LcO1G/oOOQ$c,5>h,5>hOOQ$c-E;z-E;zO#KnQMyO'#IROOQ@Y'#IR'#IRO%ApQ!>rO1G4WOOQa1G4W1G4WO#7iQM|O,5:fO%AwQ!!{O'#HwO%BbQMxO,5?`POQa1G5c1G5cOOQa1G/s1G/sOOO!b'#Hx'#HxO%BsO!bO,5:]OOQa,5:],5:]OOQa1G/u1G/uO%B{Q!LbO1G/xOOQa1G/}1G/}OOQa1G0O1G0OOOQ@Y,5@a,5@aOOQ@Y,5<f,5<fO!%oQMyO,5<fO#KnQMyO,5<fOOQ@Y-E;y-E;yOOQ@Z1G0x1G0xO%CSQMxO1G0OOOQa,5?z,5?zOOQ@Y1G1S1G1SO%CXQ!LcO1G2UO%DhQ!LcO1G2WO%ISQ!LcO1G2ZOCWQ!!zO1G2WO%KaQ!LcO,5>nOOQa-E<Q-E<QO%NpQ!LcO,5>oOOQa-E<R-E<RO&!PQ!LcO1G2ZO&%]Q!LcO1G2ZO&%jQ!LcO1G2ZO&(vQ!LcO1G2ZO&)ZQ!LcO1G2ZO&,yQ!LcO1G2ZO&-QQ!LcO1G2ZO&0dQ!LcO1G2ZO&0kQ!LcO1G2ZO&0rQ!LcO1G2ZO&2RQ!LcO1G2ZO&3bQ!LbO1G3iOCWQ!!zO1G3iO&3iO!bO1G0SO&3nO#tO1G0SO&3vO!!QO1G0SO&3}O!!QO1G0SO&4XO#tO1G0UO&4aONnO1G0UO&4hO!bO1G0WO&4mO#tO1G0WO&4uO!!QO1G0WO&4|O!!QO1G0WOOO#h'#I{'#I{O&5WONzO'#I{O&5cO!bO1G0YO&5kO!!QO1G0YO&5uO!!QO1G0YO&6PO!!QO1G0YO&5cO!bO1G0[O&6^O!bO1G0[O&6cQ!LbO1G0_O&6jQ!LbO1G0aO&6qQ!LbO1G0cO&6xQ!LbO1G0eO&7PQ# eO,5;aO&9qQ!LbO1G0gO*pQMxO'#DVOOQ`'#JU'#JUO&:ZQM|O'#JUO&:bQMxO'#JTO&:mQMxO1G0jO$7WQMyO1G1QO&:rQ$IeO,5;aOOQ@Y1G0y1G0yO&;TQMyO1G0yOOQa,5>j,5>jOOQa-E;|-E;|OOOa'#H}'#H}O&;`OaO,5;cOOQ@Z,5;c,5;cOOQa,5;V,5;VO&;kQMxO,5;VO&;pQMyO,5;XOOQ`'#JW'#JWO&<RQMxO,5;[OOQa-E;x-E;xO$7yQMxO1G0oO#JoQMxO1G1QOOQ@Y1G1Q1G1QO&<WQMyO1G1QOOQ@Y1G1o1G1oO&<fQMxO'#EmO$7yQMxO1G0oOOQa1G0o1G0oO&<kQMyO,5?yO&=]QMxO,5:XO&=bQ!>rO1G1^O&CpQ!>rO1G1cO&JOQ!>rO1G1dO!3uQMxO1G1dO!3uQMxO1G1hO$BzQ!!{O1G1iO!3uQMxO1G1jO!,VQMxO1G1jO'!ZQMxO1G1jO'!cQMxO1G1jOOQ@Y1G1l1G1lO'!tQ!>sO1G1lO')SQ!>rO1G1hO'/_Q!LcO1G1iO!%oQMyO1G1iO'/iQ!>rO1G1jO'5tQMxO1G1jOOQ@Z,5;z,5;zOOQ@Y1G1w1G1wOOQ@Y1G2O1G2OOOQ@Y1G2P1G2POOQa1G2y1G2yO'5yQ# eO'#HwO'9PQ!LcO,5@RO'<QQ!LcO'#J[OOQa1G3a1G3aO#JbQ!!zO,5:dPCWQ!!zO,5@{O'?`QMxO,5@{O'@SQ!LcO,5@|OOQa1G3_1G3_OOQ`'#HY'#HYO'B}QMxO1G3_O!:iQ!!zO,5AVO%/RQMxO,5AVO'CYQMxO,5AZO'C_QMxO,5A`OOO#i1G.|1G.|OOO#i1G.}1G.}OOQa1G4}1G4}O'CdQ!LcO'#KaOOQ@Y'#Jt'#JtOOQ@Y7+'i7+'iOOQa7+(v7+(vO'F`QMxO7+(vOOQa1G4m1G4mO'FeOMhO'#HuOOO!b'#Hu'#HuO'FpONzO1G4sOOO!b-E;t-E;tOOQa7+$|7+$|O'GOO!bO7+$|O'GWOMjO,5>^OOOp-E;p-E;pOOO#h1G/X1G/XOOO#h1G/W1G/WO%?pOMhO1G/XO'G`OMjO1G/ROOQa7+%P7+%PO'GhO!!QO7+%POOQ@Y,5>m,5>mOOQ@Y-E<P-E<POOQa7+)r7+)rO'GoQ!LcO,5>cOOQa-E;u-E;uOOO!b-E;v-E;vOOQa1G/w1G/wOOQa7+%d7+%dOOQ@Y1G2Q1G2QO!%oQMyO1G2QO'HoQ!LcO7+%jO'8nQMyO,5@RO( ZQ!LcO7+'rOCWQ!!zO7+)TO(!jQ!LcO7+)TO(#yO#tO7+%nOOQa7+%n7+%nO(#|O#tO7+%nO($RO!bO7+%nO($WO!!QO7+%nOOQa7+%p7+%pO($_O#tO7+%pO($dO#tO7+%pO($lO#tO7+%rOOQa7+%r7+%rO($oO#tO7+%rO($tO!bO7+%rO($yO!!QO7+%rOOO#h,5?g,5?gO(%QO!bO,5?gO(%VO!!QO7+%tO(%aO!bO7+%tO(%fO#tO7+%tO(%nO!!QO7+%tO&5cO!bO7+%tO&5uO!!QO7+%tO(%uO!bO7+%vO&5cO!bO7+%vOOQa7+%y7+%yOOQa7+%{7+%{OOQa7+%}7+%}OOQa7+&P7+&POOQa7+&R7+&RO(%zQMxO,5?pO(&PQM|O'#HyO(&^QMxO,5?oOOQa7+&U7+&UO#JoQMxO7+&lOOQ@Y7+&l7+&lO&<WQMyO7+&lOOQ@Y7+&e7+&eOOOa-E;{-E;{OOQ@Z1G0}1G0}OOQa1G0q1G0qOOQa1G0s1G0sOOQa1G0v1G0vO$7yQMxO7+&ZOOQa7+&Z7+&ZO&<WQMyO7+&lOOQa1G0u1G0uOOQ`'#Jh'#JhO!,VQMxO'#FXOOQ@Y'#IP'#IPO(&iQ!>rO7+&xOOQ`'#Ji'#JiO!3uQMxO'#FZOOQ@Y7+&x7+&xO(,wQ!>rO7+&}OOQ@Y7+&}7+&}OOQ@Y7+'O7+'OO(3VQ!>rO7+'OO(9bQ!>rO7+'SO'/_Q!LcO7+'TO!%oQMyO7+'TO(?mQ!>rO7+'UO!3uQMxO7+'UO!,VQMxO7+'UO(ExQMxO7+'UOOQ@Y7+'W7+'WOOQ@Y7+'S7+'SO#3OQ!!zO7+'TO(E}QMxO7+'TOOQ@Y7+'U7+'UO(FSQ# eO'#IOO(HwQ!LcO,5?vO(LVQMxO1G0OO(L[Q!LcO1G6gOOQa1G6g1G6gO#3OQ!!zO1G6gO) jQ!LcO7+(yOOQa1G6q1G6qO$ yQ!!zO1G6qOOQa1G6u1G6uOOQa1G6z1G6zO)${Q!LcO<<LbOOO!b,5>a,5>aOOO!b-E;s-E;sOOQa<<Hh<<HhOOOp1G3x1G3xOOO#h7+$s7+$sOOO#h7+$r7+$rOOO#h'#DP'#DPOOO#h7+$m7+$mOOQa<<Hk<<HkOOQ@Y7+'l7+'lO)(oQ!LcO1G/sO)+TQ!LcO<<LoOOQa<<IY<<IYO),dO#tO<<IYO),iO#tO<<IYOOQa<<I[<<I[O),qO#tO<<I[OOQa<<I^<<I^O),vO#tO<<I^O),{O#tO<<I^OOO#h1G5R1G5RO)-TO!bO<<I`O)-YO#tO<<I`O)-bO!!QO<<I`OOQa<<I`<<I`O)-]O#tO<<I`O)-iO!!QO<<I`O)-sO!bO<<IbO)-xO!bO<<IbOOQ`1G5[1G5[OOQ`,5>e,5>eOOQ`-E;w-E;wO&<WQMyO<<JWO&<WQMyO<<JWOOQ@Y<<JW<<JWOOQa<<Iu<<IuO!3uQMxO,5;sOOQ@Y-E;}-E;}OOQ@Y<<Jd<<JdOOQ@Y,5;u,5;uOOQ@Y<<Ji<<JiOOQ@Y<<Jj<<JjOOQ@Y<<Jn<<JnO#3OQ!!zO<<JoO)-}QMxO<<JoOOQ@Y<<Jp<<JpO).SQ!>rO<<JpO!3uQMxO<<JpO!,VQMxO<<JpO)4_Q!LcO<<JoO!3uQMxO<<JoOOQa7+%j7+%jO)4iQ!LbO7+,ROOQa<<Le<<LeO)4pQMxO7+,]OOQaANA|ANA|OOQaAN>tAN>tO)4uO#tOAN>tOOQaAN>vAN>vOOQaAN>xAN>xO)4zO#tOAN>xO)5PO#tOAN>zOOQaAN>zAN>zO)5SO#tOAN>zO)5XO!bOAN>zO)5^O!!QOAN>zO)5eO#tOAN>|O)5mO!bOAN>|O&<WQMyOAN?rOOQ@YAN?rAN?rOOQ@Y1G1_1G1_O)4_Q!LcOAN@ZO!3uQMxOAN@ZOOQ@YAN@[AN@[O)5rQ!>rOAN@[O!3uQMxOAN@[O#3OQ!!zOAN@ZOOQ@YAN@ZAN@ZOOQa<= m<= mOOQa<= w<= wOOQaG24`G24`OOQaG24dG24dOOQaG24fG24fO);}O#tOG24fO)<SO#tOG24fOOQaG24hG24hO)<[O#tOG24hO)<aO#tOG24hOOQ@YG25^G25^O#3OQ!!zOG25uOOQ@YG25uG25uOOQ@YG25vG25vO)<iQ!>rOG25vO)BtQ!LbOG25uOOQaLD*QLD*QO)B{O#tOLD*QOOQaLD*SLD*SO)CQO#tOLD*SO)CVQ!LbOLD+aOOQ@YLD+bLD+bO!3uQMxOLD+aOOQa!$'Ml!$'MlOOQa!$'Mn!$'MnO!3uQMxO!$'N{OOQ@Y!$'N{!$'N{OOQ@Y!)9Dg!)9DgO)C^QMxO'#EXO)ClQMxO'#EZO!HcQMxO'#HSO)CzQ# eO'#H]O)DkQ# eO'#H_O)E[Q!!zO'#HaO#3OQ!!zO'#GmO#3OQ!!zO,5:TO)EcQ# eO'#EcO#3OQ!!zO,5<jO#3OQ!!zO,5<lO)HQQ!!zO,5<oO#3OQ!!zO'#ISO)JeQ!LcO,5<oO#3OQ!!zO'#ITO)KRQ!LcO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO#3OQ!!zO,5<oO)KoQ!LcO'#J]O*#WQ!LcO,5=XO*#wQ!LcO'#JgO*$[Q# eO,5={O*&fQ!LcO,5={O*']Q!LcO1G/oO*'|Q!LcO1G2UO*(mQ!LcO1G2WO*,PQ!LcO1G2ZO#3OQ!!zO1G2WO*,WQ!LcO,5>nO*,zQ!LcO,5>oO*-qQ!LcO1G2ZO*0eQ!LcO1G2ZO*0rQ!LcO1G2ZO*3fQ!LcO1G2ZO*3yQ!LcO1G2ZO*7PQ!LcO1G2ZO*7WQ!LcO1G2ZO*:QQ!LcO1G2ZO*:XQ!LcO1G2ZO*:`Q!LcO1G2ZO*;PQ!LcO1G2ZO#3OQ!!zO1G3iO*;pQMxO,5:XO#3OQ!!zO,5@{O*;uQ!LbO,5@|O*<SQ!LcO7+'rO#3OQ!!zO7+)TO*<sQ!LcO7+)TO*=jQ!LcO1G/sO*@XQ!LcO<<LoO*AOQ!LcO'#IhO*EmQ!!zO'#GmO#3OQ!!zO'#GmO#3OQ!!zO'#GqO*J[Q# eO'#GwO*LjQ!!zO,5=}O>PQ!!zO'#DmOCWQ!!zO'#KbO*LqQ!LbO1G3iO*LxQ!!zO,5AVO+#cQMxO,5:XO*LxQ!!zO'#GxO+(OQ!!zO'#GxO*LxQ!!zO'#HOO%/RQMxO'#KkO>PQ!!zO'#DmO+(VQMxO'#He",
  stateData: "+)P~OiOS'VOS'WOS&|PQ&}PQ~OQUOR#WOS#_OT#bOa!vOd!|OhQOlQOmROnROrdOwlO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`fO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO#h!VO#l!uO#r![O#s!^O#t!_O#u!`O#v!aO#w!bO#x!cO$^!nO$_!oO$`!pO$a!qO$b!rO$g!wO$h!xO$j!zO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO(d!tO)W#kO~O&|$SO&}$TO~Ow$ZO!j$VOq'YX!U'YX!^'YX!b'YX!m'YX!s'YX#Q'YX#S'YX#U'YX#l'YX#s'YX#t'YX#u'YX#v'YX#w'YX#x'YX$s'YX$u'YX$v'YX$x'YX$y'YX$z'YX${'YX$|'YX$}'YX%O'YX%P'YX%Q'YX%R'YX%S'YX%T'YX%U'YX%V'YX%W'YX%Y'YX%Z'YX%['YX%]'YX%^'YX%_'YX%`'YX%f'YX&W'YX&z'YX!h'YX~Ov$XO~P'dOq$[OqyX!^']X!b']X!_']X~Ov']Xw']X!U']X!j']X!m']X!s']X#Q']X#S']X#U']X#l']X#s']X#t']X#u']X#v']X#w']X#x']X$s']X$u']X$v']X$x']X$y']X$z']X${']X$|']X$}']X%O']X%P']X%Q']X%R']X%S']X%T']X%U']X%V']X%W']X%Y']X%Z']X%[']X%]']X%^']X%_']X%`']X%f']X&W']X&z']X!i']X!h']X#b']X~P*pOq$^O~OZ$aO'V$_O'W$_O~OQ$iOX$iOY$iOh$dOl$cOmROnRO!|1TO#O1UO&|$SO&}$TO~O'b$fO~P.jOZ$kO'V$_O'W$_O~Ov$mOv'_Xw'_X!U'_X!^'_X!b'_X!j'_X!m'_X!s'_X#Q'_X#S'_X#U'_X#l'_X#s'_X#t'_X#u'_X#v'_X#w'_X#x'_X$s'_X$u'_X$v'_X$x'_X$y'_X$z'_X${'_X$|'_X$}'_X%O'_X%P'_X%Q'_X%R'_X%S'_X%T'_X%U'_X%V'_X%W'_X%Y'_X%Z'_X%['_X%]'_X%^'_X%_'_X%`'_X%f'_X&W'_X&z'_X!_'_X!i'_X!h'_X#b'_X~OU$nO~Oq$pO!^$oOQ%sXR%sXS%sXT%sXh%sXh%|Xl%sXl%|Xm%sXn%sXr%sXr%|Xv#iX{%sX|%sX}%sX!P%sX!S%sX!U%sX!V%sX!W%sX!Y%sX!Z%sX![%sX!`%sX!b#VX!d%sX!g%sX!j%sX!m%sX!q%sX!s#VX!u%sX!w%sX!y%sX!z%sX!|%sX#O%sX#Q%sX#S%sX#U%sX#X%sX#Y%sX#Z%sX#[%sX#^%sX#l#VX#s#VX#t#VX#u#VX#v#VX#w#VX#x#VX$s#VX$u#VX$v#VX$x#VX$y#VX$z%sX${#VX$|#VX$}#VX%O#VX%P#VX%Q#VX%R#VX%S#VX%T#VX%U#VX%V#VX%W#VX%Y#VX%Z#VX%[#VX%]#VX%^#VX%_#VX%`#VX%b%sX%c%sX%d%sX%f%sX%h%sX%i%sX%j%sX%m%sX%n%sX%o%sX%p%sX%q%sX%w%sX%y%sX&S%sX&U%sX&W#VX&Y%sX&Z%sX&[%sX&^%sX&_%sX&a%sX&b%sX&c%sX&z#VX'R%sX'b%sX)W%sX!h#VX~Ow$sO#b$rO~P3XOv'[Xw'[X!U'[X!j'[X!m'[X!s'[X#Q'[X#S'[X#U'[X#l'[X$s'[X$u'[X$v'[X$x'[X$y'[X$z'[X${'[X$|'[X$}'[X%O'[X%P'[X%Q'[X%R'[X%S'[X%T'[X%U'[X%V'[X%W'[X%Y'[X%Z'[X%['[X%]'[X%^'[X%_'[X%`'[X%f'[X&W'[X&z'[X!_'[X!i'[X!h'[X#b'[X~O!^$oO!b'[X#s'[X#t'[X#u'[X#v'[X#w'[X#x'[X~P:mOQUOR#WOS#_OT#bOh$tOl$tOmROnROr$uOw$yO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`$wO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO!_'hP~OZ$|O'V$_O'W$_O~Oh%POl%POr%POw%QO~OQUOR#WOS#_OT#bOh$tOl$tOmROnROr$uOw$yO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`$wO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~O!i'hP~PCWO$o%WO$p%XO$q%YO!h'hP~PcOQUOR#WOS#_OT#bOh$tOl$tOmROnROr$uO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`$wO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~Ow%`O~PHYOv$XOw$ZO!U%zO!j$VO!m%yO!s%xO#Q%xO#S%sO#U%xO#s!^O#t!_O#u!`O#v!aO#w!bO#x!cO$s%dO$u%fO$v%eO$x%wO$y%wO$z%yO${%yO$|%zO$}%lO%O%hO%P%iO%Q%jO%R%kO%S%pO%T%nO%U%oO%V%{O%W%rO%Y%}O%Z&OO%[&PO%]&QO%^%tO%_%uO%`%vO%f&TO&W&UO!^(PX!b(PX#l(PX&z(PX~OZ&VO'V$_O'W$_O~O!s&XO~OZ&YO'V$_O'W$_O~OZ&[O'V$_O'W$_O~OZ&^O'V$_O'W$_O~Oh&`Ol&`Ow&aO!|yO~Oh&bOl&bOw&cO!|yO~Oh&dOl&dOw&eO!|yO~OT&jOh&fOl&fOr&hOw&gO!|yO~Oh&kOl&kOw&lO!|yO~OQUOhROlROmROnRO{&nO!`&mO~O#^!UO~P!$lOr&pO~O!^&rO!b&rO#l(OX&z(OX~O#l&uO&z&vO~O$^!nO$_!oO$`!pO$a!qO$b!rO~Or&|Ow'PO!`&xO#b&yO~P!%wOQUOR#WOS#_OT#bOh$tOl$tOmROnROr$uOw$yO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`fO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~O#s(RP#t(RP#u(RP#v(RP#w(RP#x(RP#l(RP&z(RP~P!&jO#s!^O#t!_O#u!`O#v!aO#w!bO#x!cO~O!`'VO~Ow'PO#u!`O#v!aO#w!bO#x!cO~Oh'bOl'bO!`'`O#X!OO#Y!PO#Z!QO#[!RO$X'dO~O#l&uO$T'eO&z&vOQ$YXR$YXS$YXT$YXa$YXd$YXh$YXl$YXm$YXn$YXr$YXw$YX{$YX|$YX}$YX!P$YX!S$YX!U$YX!V$YX!W$YX!Y$YX!Z$YX![$YX!`$YX!d$YX!g$YX!j$YX!m$YX!q$YX!u$YX!w$YX!y$YX!z$YX!|$YX#O$YX#Q$YX#S$YX#U$YX#X$YX#Y$YX#Z$YX#[$YX#^$YX#h$YX#l$YX#r$YX#s$YX#t$YX#u$YX#v$YX#w$YX#x$YX$^$YX$_$YX$`$YX$a$YX$b$YX$g$YX$h$YX$j$YX$z$YX%b$YX%c$YX%d$YX%f$YX%h$YX%i$YX%j$YX%m$YX%n$YX%o$YX%p$YX%q$YX%w$YX%y$YX&S$YX&U$YX&Y$YX&Z$YX&[$YX&^$YX&_$YX&a$YX&b$YX&c$YX&y$YX'R$YX'b$YX(d$YX)W$YX!h$YX$o$YX$p$YX$q$YX~Or'iO#l$[X&z$[X~Ow'PO~Ob'mO~O'T'nO~O'T'oO~Oe'pOf'qO~O!`'tOv%gXw%gX!U%gX!^%gX!b%gX!j%gX!m%gX!s%gX#Q%gX#S%gX#U%gX#l%gX#s%gX#t%gX#u%gX#v%gX#w%gX#x%gX$s%gX$u%gX$v%gX$x%gX$y%gX$z%gX${%gX$|%gX$}%gX%O%gX%P%gX%Q%gX%R%gX%S%gX%T%gX%U%gX%V%gX%W%gX%Y%gX%Z%gX%[%gX%]%gX%^%gX%_%gX%`%gX%f%gX&W%gX&z%gX!_%gX!i%gX!h%gX#b%gX~Ov%kX!^%kX!b%kX!s%kX#l%kX#s%kX#t%kX#u%kX#v%kX#w%kX#x%kX$s%kX$u%kX$v%kX$x%kX$y%kX${%kX$|%kX$}%kX%O%kX%P%kX%Q%kX%R%kX%S%kX%T%kX%U%kX%V%kX%W%kX%Y%kX%Z%kX%[%kX%]%kX%^%kX%_%kX%`%kX&W%kX&z%kX!_%kX!i%kX!h%kX#b%kX~P!&jOQUOR#WOS#_OT#bOh$tOl$tOmROnROr$uOw$yO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`'VO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~OQUOR#WOS#_OT#bOh$tOl$tOmROnROr$uO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`'VO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~OwlO~P!?SOQ%sXR%sXS%sXT%sXh%sXl%sXm%sXn%sXr%sXw%sX{%sX|%sX}%sX!P%sX!S%sX!U%sX!V%sX!W%sX!Y%sX!Z%sX![%sX!`%sX!d%sX!g%sX!j%sX!m%sX!q%sX!u%sX!w%sX!y%sX!z%sX!|%sX#O%sX#Q%sX#S%sX#U%sX#X%sX#Y%sX#Z%sX#[%sX#^%sX$z%sX%b%sX%c%sX%d%sX%f%sX%h%sX%i%sX%j%sX%m%sX%n%sX%o%sX%p%sX%q%sX%w%sX%y%sX&S%sX&U%sX&Y%sX&Z%sX&[%sX&^%sX&_%sX&a%sX&b%sX&c%sX'R%sX'b%sX)W%sX~Ov$mO~P!CqOh'|Ol'|Ow$yO!`(OO!jkO!m'{O#^!UO%c(QO~Oh(SOl(SOr(RO~Oq(TO~OQUOT#bOh$tOl$tOmROnROwlO{$PO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`fO!dgO!gjO!jkO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO%d#OO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kOv&PX!s&PX#l&PX$s&PX$u&PX$v&PX$x&PX$y&PX${&PX$|&PX$}&PX%O&PX%P&PX%Q&PX%R&PX%S&PX%T&PX%U&PX%V&PX%W&PX%Y&PX%Z&PX%[&PX%]&PX%^&PX%_&PX%`&PX&W&PX&z&PX!_&PX!i&PX!h&PX#b&PX~OR#WOS#_Or$uO|eO!mmO$z#PO%b#PO%c#PO%f#RO!^&PX!b&PX#s&PX#t&PX#u&PX#v&PX#w&PX#x&PX~P!I_OQUOT#bOh$tOl$tOmROnROwlO{$PO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`fO!dgO!gjO!jkO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO%d#OO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kOv&RX!s&RX#l&RX$s&RX$u&RX$v&RX$x&RX$y&RX${&RX$|&RX$}&RX%O&RX%P&RX%Q&RX%R&RX%S&RX%T&RX%U&RX%V&RX%W&RX%Y&RX%Z&RX%[&RX%]&RX%^&RX%_&RX%`&RX&W&RX&z&RX!_&RX!i&RX!h&RX#b&RX~OR#WOS#_Or$uO|eO!mmO$z#PO%b#PO%c#PO%f#RO!^&RX!b&RX#s&RX#t&RX#u&RX#v&RX#w&RX#x&RX~P#!mOQUOR#WOS#_OT#bOh$tOl$tOmROnROw$yO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`$wO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~Or(WO~P#){Om(YO!`(ZO#OzO~Om(]O!`(^O#OzOv&]Xw&]X!U&]X!^&]X!b&]X!j&]X!m&]X!s&]X#Q&]X#S&]X#U&]X#l&]X#s&]X#t&]X#u&]X#v&]X#w&]X#x&]X$s&]X$u&]X$v&]X$x&]X$y&]X$z&]X${&]X$|&]X$}&]X%O&]X%P&]X%Q&]X%R&]X%S&]X%T&]X%U&]X%V&]X%W&]X%Y&]X%Z&]X%[&]X%]&]X%^&]X%_&]X%`&]X%f&]X&W&]X&z&]X!_&]X!i&]X!h&]X#b&]X~Om(`On(`O!`(aO#OzO#Q{O~OW(dO^(eO~OW(gO~P.jOQUOR2gOS2nOT#bOh$tOl$tOmROnROr1]Ow$yO{$PO|2cO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`$wO!dgO!gjO!jkO!m2dO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z1ZO%b1ZO%c1ZO%d#OO%f2fO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~OQ(mOh(mOl(mOr(lOw$ZO!`'VO!j$VO!|yO%y#gO~OV(kO~P#7iOQUOR2gOS2nOT#bOh$tOl$tOmROnROw$yO{$PO|2cO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`$wO!dgO!gjO!jkO!m2dO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z1ZO%b1ZO%c1ZO%d#OO%f2fO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~Or(oO~P#8_OP(qOr(qO~Oq$[Oqya~OP(sOr(sO~Oi(vO'V$_O'W$_O~O](yO^(wO~OZ(zO~Ov)OOw$ZO!j$VO'O)POQ'YXX'YXY'YXh'YXl'YXm'YXn'YX!|'YX#O'YX&|'YX&}'YX'b'YXW'YX]'YX!s'YX['YX~O'P({O'QpX~P#>OO'Q)RO~O'b)SO~P.jOv)OOw$ZO!j$VO'O)POQ'XXX'XXY'XXh'XXl'XXm'XXn'XX!|'XX#O'XX&|'XX&}'XX'b'XXW'XX]'XX!s'XX['XX~O])UO~P.jOZ)WO~OQ(mOh(mOl(mOr(lO!`'VO!|yO%y#gO~OP)YOr)YO~Oq$pOQ#iaR#iaS#iaT#iah#ial#iam#ian#iar#iav#iaw#ia{#ia|#ia}#ia!P#ia!S#ia!U#ia!V#ia!W#ia!Y#ia!Z#ia![#ia!`#ia!d#ia!g#ia!j#ia!m#ia!q#ia!u#ia!w#ia!y#ia!z#ia!|#ia#O#ia#Q#ia#S#ia#U#ia#X#ia#Y#ia#Z#ia#[#ia#^#ia$z#ia%b#ia%c#ia%d#ia%f#ia%h#ia%i#ia%j#ia%m#ia%n#ia%o#ia%p#ia%q#ia%w#ia%y#ia&S#ia&U#ia&Y#ia&Z#ia&[#ia&^#ia&_#ia&a#ia&b#ia&c#ia'R#ia'b#ia)W#ia~O!h)_O~PcOv)`O!_'YX!i'YX#b'YX~P'dOw%sXw&OX!_#VX!i#VX#b#VX~P3XOv)`Ow$ZO!U%zO!^&rO!b&rO!j$VO!m%yO!s%xO#Q%xO#S%sO#U%xO$s%dO$u%fO$v%eO$x%wO$y%wO$z%yO${%yO$|%zO$}%lO%O%hO%P%iO%Q%jO%R%kO%S%pO%T%nO%U%oO%V%{O%W%rO%Y%}O%Z&OO%[&PO%]&QO%^%tO%_%uO%`%vO%f&TO&W&UO!_'hX~O!_)dO~O!h'hP~PCWOw'PO!`&xO#b&yO~O])gO_)eO~OZ$|O~O!i'hX!h'hX~P#GpO!i)jO~O!h'hX~PLwO!h)kO~O#l&uO&z&vO~P!+qOr)oO#l&uO&z&vO~P!+qO!h)qO$o%WO$p%XO$q%YO~PcOv)`Ow$ZO!j$VO%f&TO!^%aa!b%aa#l%aa#s%aa#t%aa#u%aa#v%aa#w%aa#x%aa%^%aa%_%aa%`%aa&z%aa!_%aa!i%aa!h%aa#b%aa~O!U%aa!m%aa!s%aa#Q%aa#S%aa#U%aa$s%aa$u%aa$v%aa$x%aa$y%aa$z%aa${%aa$|%aa$}%aa%O%aa%P%aa%Q%aa%R%aa%S%aa%T%aa%U%aa%V%aa%W%aa%Y%aa%Z%aa%[%aa%]%aa&W%aa~P#LgOQUOR#WOS#_OT#bOh$tOl$tOmROnROr$uOw$yO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`2iO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~O$s)xO~PCWOv$waw$wa!U$wa!^$wa!b$wa!j$wa!m$wa!s$wa#Q$wa#S$wa#U$wa#l$wa#s$wa#t$wa#u$wa#v$wa#w$wa#x$wa$s$wa$u$wa$v$wa$x$wa$y$wa$z$wa${$wa$|$wa%V$wa%W$wa%Y$wa%Z$wa%[$wa%]$wa%^$wa%_$wa%`$wa%f$wa&W$wa&z$wa!_$wa!i$wa!h$wa#b$wa~O$}%lO%O%hO%P%iO%Q%jO%R%kO%S$wa%T$wa%U$wa~P$&kO%S%pO%T%nO%U%oO$}$wa%O$wa%P$wa%Q$wa%R$wa~P$&kO#b*ZO~P#3OO]*]O^*[O~P.jOZ*_O~O!s*`O~P.jO]*cO^*bO~P.jOZ*eO~O[*fO]*gO^*hO~P.jOZ*kO~O^*lO~OZ*mO~OQ%saR%saS%saT%sah%sal%sam%san%sar%sav%uaw%sa{%sa|%sa}%sa!P%sa!S%sa!U%sa!V%sa!W%sa!Y%sa!Z%sa![%sa!^%ua!`%sa!b%ua!d%sa!g%sa!j%sa!m%sa!q%sa!s%ua!u%sa!w%sa!y%sa!z%sa!|%sa#O%sa#Q%sa#S%sa#U%sa#X%sa#Y%sa#Z%sa#[%sa#^%sa#l%ua#s%ua#t%ua#u%ua#v%ua#w%ua#x%ua$s%ua$u%ua$v%ua$x%ua$y%ua$z%sa${%ua$|%ua$}%ua%O%ua%P%ua%Q%ua%R%ua%S%ua%T%ua%U%ua%V%ua%W%ua%Y%ua%Z%ua%[%ua%]%ua%^%ua%_%ua%`%ua%b%sa%c%sa%d%sa%f%sa%h%sa%i%sa%j%sa%m%sa%n%sa%o%sa%p%sa%q%sa%w%sa%y%sa&S%sa&U%sa&W%ua&Y%sa&Z%sa&[%sa&^%sa&_%sa&a%sa&b%sa&c%sa&z%ua'R%sa'b%sa)W%sa!_%ua!i%ua!h%ua#b%ua~Oq$pOw%ua!U%ua!j%ua!m%ua#Q%ua#S%ua#U%ua$z%ua%f%ua~P$,VOQUOhROlROmROnRO{*uO!`*vO~O!_'wP~P$4TOr&|O~Oq$pOw#iX#j#iX#l#iX&z#iX~Ow'PO#j*|O#l&uO&z&vO~O!^&rX!b&rX#l&rX&z&rX#s&rX#t&rX#u&rX#v&rX#w&rX#x&rX~P!&jO!^&rO!b&rO#l(Oa&z(Oa~O&{+RO&|$SO&}$TO~O!_+SO#`+TO~Or+UO#e+VO~Ow'PO!`&xO#b&yO#l&uO&z&vO~O!h%_O$o%WO$p%XO$q%YO~PcOw'PO#b+_O~Ov)`Ow$ZO!U%zO!j$VO!m%yO!s%xO#Q%xO#S%sO#U%xO$s%dO$u%fO$v%eO$x%wO$y%wO$z%yO${%yO$|%zO$}%lO%O%hO%P%iO%Q%jO%R%kO%S%pO%T%nO%U%oO%V%{O%W%rO%Y%}O%Z&OO%[&PO%]&QO%^%tO%_%uO%`%vO%f&TO&W&UO~O!^(PX!b(PX#s(PX#t(PX#u(PX#v(PX#w(PX#x(PX#l(PX&z(PX~P$8RO!^&rO!b&rO#s(RX#t(RX#u(RX#v(RX#w(RX#x(RX#l(RX&z(RX~Oh+kOl+kO!`+iO#X!OO#Y!PO#Z!QO#[!RO$X+mO~O#l&uO&z&vOQ$YaR$YaS$YaT$Yaa$Yad$Yah$Yal$Yam$Yan$Yar$Yaw$Ya{$Ya|$Ya}$Ya!P$Ya!S$Ya!U$Ya!V$Ya!W$Ya!Y$Ya!Z$Ya![$Ya!`$Ya!d$Ya!g$Ya!j$Ya!m$Ya!q$Ya!u$Ya!w$Ya!y$Ya!z$Ya!|$Ya#O$Ya#Q$Ya#S$Ya#U$Ya#X$Ya#Y$Ya#Z$Ya#[$Ya#^$Ya#h$Ya#l$Ya#r$Ya#s$Ya#t$Ya#u$Ya#v$Ya#w$Ya#x$Ya$^$Ya$_$Ya$`$Ya$a$Ya$b$Ya$g$Ya$h$Ya$j$Ya$z$Ya%b$Ya%c$Ya%d$Ya%f$Ya%h$Ya%i$Ya%j$Ya%m$Ya%n$Ya%o$Ya%p$Ya%q$Ya%w$Ya%y$Ya&S$Ya&U$Ya&Y$Ya&Z$Ya&[$Ya&^$Ya&_$Ya&a$Ya&b$Ya&c$Ya&y$Ya'R$Ya'b$Ya(d$Ya)W$Ya!h$Ya$o$Ya$p$Ya$q$Ya~O$T'eO~P$<eO#l&uO&z&vO~P>POh+kOl+kO~On+kO#X!OO#Y!PO#Z!QO#[!RO~Ow$sO~Oc+vO~Og+wO~Og+xO~O!U%zO!m%yO!s%xO#Q%xO#S%sO#U%xO$s%dO$u%fO$v%eO$x%wO$y%wO$z%yO${%yO$|%zO$}%lO%O%hO%P%iO%Q%jO%R%kO%S%pO%T%nO%U%oO%V%{O%W%rO%Y%}O%Z&OO%[&PO%]&QO&W&UO~P#LgOv)`Ow$ZO!j$VO!U%ea!^%ea!b%ea!m%ea!s%ea#Q%ea#S%ea#U%ea#l%ea#s%ea#t%ea#u%ea#v%ea#w%ea#x%ea$s%ea$u%ea$v%ea$x%ea$y%ea$z%ea${%ea$|%ea$}%ea%O%ea%P%ea%Q%ea%R%ea%S%ea%T%ea%U%ea%V%ea%W%ea%Y%ea%Z%ea%[%ea%]%ea%^%ea%_%ea%`%ea%f%ea&W%ea&z%ea!_%ea!i%ea!h%ea#b%ea~O!_+yO~O!^&rO!b&rO#l(ZX#s(ZX#t(ZX#u(ZX#v(ZX#w(ZX#x(ZX&z(ZX!_(ZX!i(ZX!h(ZX#b(ZX~P$8ROw,OO~Oh'|Ol'|Ow$yO!jkO!m'{O#^!UO~O%c2jO~P$JlOq$pOv#iXw#iX!U#iX!^#iX!`#iX!b#iX!j#iX!m#iX!s#iX#Q#iX#S#iX#U#iX#l#iX#s#iX#t#iX#u#iX#v#iX#w#iX#x#iX$s#iX$u#iX$v#iX$x#iX$y#iX$z#iX${#iX$|#iX$}#iX%O#iX%P#iX%Q#iX%R#iX%S#iX%T#iX%U#iX%V#iX%W#iX%Y#iX%Z#iX%[#iX%]#iX%^#iX%_#iX%`#iX%f#iX&W#iX&z#iX!_#iX!i#iX!h#iX#b#iX~O!`'VOv%{aw%{a!U%{a!^%{a!b%{a!j%{a!m%{a!s%{a#Q%{a#S%{a#U%{a#l%{a#s%{a#t%{a#u%{a#v%{a#w%{a#x%{a$s%{a$u%{a$v%{a$x%{a$y%{a$z%{a${%{a$|%{a$}%{a%O%{a%P%{a%Q%{a%R%{a%S%{a%T%{a%U%{a%V%{a%W%{a%Y%{a%Z%{a%[%{a%]%{a%^%{a%_%{a%`%{a%f%{a&W%{a&z%{a!_%{a!i%{a!h%{a#b%{a~Or,TO~Oq$pOQ%sXR%sXS%sXT%sXh%sXh%|Xl%sXl%|Xm%sXn%sXr%sXr%|Xv&Taw&Ta{%sX|%sX}%sX!P%sX!S%sX!U&Ta!V%sX!W%sX!Y%sX!Z%sX![%sX!`%sX!d%sX!g%sX!j&Ta!m&Ta!q%sX!s&Ta!u%sX!w%sX!y%sX!z%sX!|%sX#O%sX#Q&Ta#S&Ta#U&Ta#X%sX#Y%sX#Z%sX#[%sX#^%sX#l&Ta$s&Ta$u&Ta$v&Ta$x&Ta$y&Ta$z&Ta${&Ta$|&Ta$}&Ta%O&Ta%P&Ta%Q&Ta%R&Ta%S&Ta%T&Ta%U&Ta%V&Ta%W&Ta%Y&Ta%Z&Ta%[&Ta%]&Ta%^&Ta%_&Ta%`&Ta%b%sX%c%sX%d%sX%f&Ta%h%sX%i%sX%j%sX%m%sX%n%sX%o%sX%p%sX%q%sX%w%sX%y%sX&S%sX&U%sX&W&Ta&Y%sX&Z%sX&[%sX&^%sX&_%sX&a%sX&b%sX&c%sX&z&Ta'R%sX'b%sX)W%sX!_&Ta!i&Ta!h&Ta#b&Ta~O!^&Ta!b&Ta#s&Ta#t&Ta#u&Ta#v&Ta#w&Ta#x&Ta~P%$TOv)`Ow$ZO!U%zO!j$VO!m%yO!s%xO#Q%xO#S%sO#U%xO$v%eO$x%wO$y%wO$z%yO${%yO$|%zO$}%lO%O%hO%P%iO%Q%jO%R%kO%S%pO%T%nO%U%oO%V%{O%W%rO%Y%}O%Z&OO%[&PO%]&QO%f&TO&W&UO~O!^&Ta!b&Ta#l&Ta#s&Ta#t&Ta#u&Ta#v&Ta#w&Ta#x&Ta$s&Ta$u&Ta%^&Ta%_&Ta%`&Ta&z&Ta!_&Ta!i&Ta!h&Ta#b&Ta~P%+oO!^&rO!b&rO~Om,WO#OzO~Om,XO#OzO~Om,YOn,YO#OzO#Q{O~OW,ZO~OW,[O~P.jOv)`Ow$ZO!U1hO!j$VO!m1gO!s1fO#Q1fO#S%sO#U1fO$s1^O$u1_O$v%eO$x1eO$y1eO$z1gO${1gO$|1hO$}1aO%O%hO%P%iO%Q%jO%R%kO%S1cO%T%nO%U%oO%V1iO%W%rO%Y1kO%Z1lO%[1mO%]1nO%^%tO%_%uO%`%vO%f&TO&W2hO~O!i,]O~P%0VO#l&uO&z&vOQ(hPR(hPS(hPT(hPa(hPd(hPh(hPl(hPm(hPn(hPr(hPw(hP{(hP|(hP}(hP!P(hP!S(hP!U(hP!V(hP!W(hP!Y(hP!Z(hP![(hP!`(hP!d(hP!g(hP!j(hP!m(hP!q(hP!u(hP!w(hP!y(hP!z(hP!|(hP#O(hP#Q(hP#S(hP#U(hP#X(hP#Y(hP#Z(hP#[(hP#^(hP#h(hP#l(hP#r(hP#s(hP#t(hP#u(hP#v(hP#w(hP#x(hP$^(hP$_(hP$`(hP$a(hP$b(hP$g(hP$h(hP$j(hP$z(hP%b(hP%c(hP%d(hP%f(hP%h(hP%i(hP%j(hP%m(hP%n(hP%o(hP%p(hP%q(hP%w(hP%y(hP&S(hP&U(hP&Y(hP&Z(hP&[(hP&^(hP&_(hP&a(hP&b(hP&c(hP&y(hP'R(hP'b(hP(d(hP)W(hP!h(hP$o(hP$p(hP$q(hP~O!`'VOv%xiw%xi!U%xi!^%xi!b%xi!j%xi!m%xi!s%xi#Q%xi#S%xi#U%xi#l%xi#s%xi#t%xi#u%xi#v%xi#w%xi#x%xi$s%xi$u%xi$v%xi$x%xi$y%xi$z%xi${%xi$|%xi$}%xi%O%xi%P%xi%Q%xi%R%xi%S%xi%T%xi%U%xi%V%xi%W%xi%Y%xi%Z%xi%[%xi%]%xi%^%xi%_%xi%`%xi%f%xi&W%xi&z%xi!_%xi!i%xi!h%xi#b%xi~Oq,bO~Oq$pO!^1[O!h,cOh%|Xl%|Xr%|Xv#iXw&OX!s#VX$s#VX$u#VX$v#VX$x#VX$y#VX${#VX$|#VX$}#VX%O#VX%P#VX%Q#VX%R#VX%S#VX%T#VX%U#VX%V#VX%W#VX%Y#VX%Z#VX%[#VX%]#VX%^#VX%_#VX%`#VX&W#VX~P!CqO!h,cO~P%0VOi,eO'V$_O'W$_OZ'aa~O],hO^(wO~Oq,jO~O'P({O'Qpa~Ow$ZO!j$VO~Ov,nOw$ZO!j$VO~Oq,oO~O],pO~P.jO$s%dO$u%fO!^!]i!b!]i#l!]i#s!]i#t!]i#u!]i#v!]i#w!]i#x!]i%^!]i%_!]i%`!]i&z!]i!_!]i!i!]i!h!]i#b!]i~P%+oO!h,tO~PcO!^&kX!_&kX!b&kX!i&kX!h&kX#l&kX&z&kX~PCWO!^&rO!b&rO!_'ha!i'ha!h'ha~O],xO_)eO~O!h,yO~P%0VO!h,|O~O$s%dO$u%fO!^$ri!b$ri#l$ri#s$ri#t$ri#u$ri#v$ri#w$ri#x$ri%^$ri%_$ri%`$ri&z$ri!_$ri!i$ri!h$ri#b$ri~P%+oO$s%dO$u%fO!^$ti!b$ti#l$ti#s$ti#t$ti#u$ti#v$ti#w$ti#x$ti%^$ti%_$ti%`$ti&z$ti!_$ti!i$ti!h$ti#b$ti~P%+oOv)`Ow$ZO!j$VO$x%wO%f&TO!U$wi!^$wi!b$wi!m$wi!s$wi#Q$wi#S$wi#U$wi#l$wi#s$wi#t$wi#u$wi#v$wi#w$wi#x$wi$s$wi$u$wi$v$wi$z$wi${$wi$|$wi$}$wi%O$wi%P$wi%Q$wi%R$wi%S$wi%T$wi%U$wi%V$wi%W$wi%Y$wi%Z$wi%[$wi%]$wi%^$wi%_$wi%`$wi&W$wi&z$wi!_$wi!i$wi!h$wi#b$wi~O$y%wO~P%EwOv)`Ow$ZO!j$VO$v%eO%f&TO#S&va#l&va$s&va$u&va$}&va%O&va%P&va%Q&va%R&va%S&va%T&va%U&va%V&va%W&va%Y&va%Z&va%[&va%]&va%^&va%_&va%`&va&W&va&z&va!_&va!i&va!h&va#b&va~O!U%zO!m%yO!s%xO#Q%xO#U%xO$x%wO$y%wO$z%yO${%yO$|%zO!^&va!b&va#s&va#t&va#u&va#v&va#w&va#x&va~P%IZOv)`Ow$ZO!j$VO$v%eO%O%hO%P%iO%Q%jO%R%kO%f&TO#S&wa#l&wa$s&wa$u&wa%S&wa%T&wa%U&wa%V&wa%W&wa%Y&wa%Z&wa%[&wa%]&wa%^&wa%_&wa%`&wa&W&wa&z&wa!_&wa!i&wa!h&wa#b&wa~O!U%zO!m%yO!s%xO#Q%xO#U%xO$x%wO$y%wO$z%yO${%yO$|%zO$}%lO!^&wa!b&wa#s&wa#t&wa#u&wa#v&wa#w&wa#x&wa~P%LmO$y$wi~P%EwOv)`Ow$ZO!j$VO!s%xO#Q%xO#U%xO$v%eO$x%wO$y%wO%f&TO!U$wi!^$wi!b$wi#S$wi#l$wi#s$wi#t$wi#u$wi#v$wi#w$wi#x$wi$s$wi$u$wi$|$wi$}$wi%O$wi%P$wi%Q$wi%R$wi%S$wi%T$wi%U$wi%V$wi%W$wi%Y$wi%Z$wi%[$wi%]$wi%^$wi%_$wi%`$wi&W$wi&z$wi!_$wi!i$wi!h$wi#b$wi~O!m$wi$z$wi${$wi~P&!WO!m%yO$z%yO${%yO~P&!WOv)`Ow$ZO!U%zO!j$VO!m%yO!s%xO#Q%xO#U%xO$v%eO$x%wO$y%wO$z%yO${%yO$|%zO$}%lO%O%hO%P%iO%Q%jO%R%kO%f&TO!^$wi!b$wi#S$wi#l$wi#s$wi#t$wi#u$wi#v$wi#w$wi#x$wi$s$wi$u$wi%Y$wi%Z$wi%[$wi%]$wi%^$wi%_$wi%`$wi&W$wi&z$wi!_$wi!i$wi!h$wi#b$wi~O%S$wi%T$wi%U$wi%V$wi%W$wi~P&%wO%S%pO%T%nO%U%oO%V%{O%W%rO~P&%wOv)`Ow$ZO!U%zO!j$VO!m%yO!s%xO#Q%xO#S%sO#U%xO$v%eO$x%wO$y%wO$z%yO${%yO$|%zO$}%lO%O%hO%P%iO%Q%jO%R%kO%S%pO%T%nO%U%oO%V%{O%W%rO%f&TO!^$wi!b$wi#l$wi#s$wi#t$wi#u$wi#v$wi#w$wi#x$wi$s$wi$u$wi%Z$wi%[$wi%]$wi%^$wi%_$wi%`$wi&W$wi&z$wi!_$wi!i$wi!h$wi#b$wi~O%Y$wi~P&)nO%Y%}O~P&)nOv)`Ow$ZO!U%zO!j$VO!m%yO!s%xO#Q%xO#S%sO#U%xO$v%eO$x%wO$y%wO$z%yO${%yO$|%zO$}%lO%O%hO%P%iO%Q%jO%R%kO%S%pO%T%nO%U%oO%V%{O%W%rO%Y%}O%Z&OO%f&TO!^$wi!b$wi#l$wi#s$wi#t$wi#u$wi#v$wi#w$wi#x$wi$s$wi$u$wi%]$wi%^$wi%_$wi%`$wi&W$wi&z$wi!_$wi!i$wi!h$wi#b$wi~O%[$wi~P&-XO%[&PO~P&-XO$s%dO$u%fO!^$wi!b$wi#l$wi#s$wi#t$wi#u$wi#v$wi#w$wi#x$wi%^$wi%_$wi%`$wi&z$wi!_$wi!i$wi!h$wi#b$wi~P%+oO$s%dO$u%fO%^%tO!^$wi!b$wi#l$wi#s$wi#t$wi#u$wi#v$wi#w$wi#x$wi%_$wi%`$wi&z$wi!_$wi!i$wi!h$wi#b$wi~P%+oO#b-PO~P%0VO]-RO~O`-TO'S-SO~O]-RO~P.jO]-RO^-UO~P.jO`-XO'S-WO~O!s-YO~P.jO]-ZO~O`-]O'S-[O~O]-ZO~P.jO]-ZO^-^O~P.jOZ-`O'V$_O'W$_O~O[*fO]*gO~O]-dO^-cO~P.jO[*fO]*gO~P.jO[*fO]*gO^-fO~P.jO^-iO~O!h-jO~P%0VO!h-kO~P%0VO!h-lO~P%0VO!h-mO~P%0VO!^#ia!b#ia!s#ia#l#ia#s#ia#t#ia#u#ia#v#ia#w#ia#x#ia$s#ia$u#ia$v#ia$x#ia$y#ia${#ia$|#ia$}#ia%O#ia%P#ia%Q#ia%R#ia%S#ia%T#ia%U#ia%V#ia%W#ia%Y#ia%Z#ia%[#ia%]#ia%^#ia%_#ia%`#ia&W#ia&z#ia!_#ia!i#ia!h#ia#b#ia~P#BTO!h-nO~P%0VOQUOhROlROmROnRO~O{-oO~P&9xO!^&rO!b&rO!_'wX~O!_-rO~Oq$pOw#ia#j#ia#l#ia&z#ia~Ow'PO#l&uO&z&vO~O&{-xO&|$SO&}$TO~O!_-yO~O!`'VO#b#aaw#aa#l#aa&z#aa~O!`&xO~Ow'PO#b+_O#l&uO&z&vO~Or+UO~O!^&rO!b&rO#s(Ra#t(Ra#u(Ra#v(Ra#w(Ra#x(Ra#l(Ra&z(Ra~O!_.PO~O#|.QO$O.UOQ#ziR#ziS#ziT#zia#zid#zih#zil#zim#zin#zir#ziw#zi{#zi|#zi}#zi!P#zi!S#zi!U#zi!V#zi!W#zi!Y#zi!Z#zi![#zi!`#zi!d#zi!g#zi!j#zi!m#zi!q#zi!u#zi!w#zi!y#zi!z#zi!|#zi#O#zi#Q#zi#S#zi#U#zi#X#zi#Y#zi#Z#zi#[#zi#^#zi#h#zi#l#zi#r#zi#s#zi#t#zi#u#zi#v#zi#w#zi#x#zi$^#zi$_#zi$`#zi$a#zi$b#zi$g#zi$h#zi$j#zi$z#zi%b#zi%c#zi%d#zi%f#zi%h#zi%i#zi%j#zi%m#zi%n#zi%o#zi%p#zi%q#zi%w#zi%y#zi&S#zi&U#zi&Y#zi&Z#zi&[#zi&^#zi&_#zi&a#zi&b#zi&c#zi&y#zi'R#zi'b#zi(d#zi)W#zi!h#zi$o#zi$p#zi$q#zi~O#|.QO$O.UOQ$PiR$PiS$PiT$Pia$Pid$Pih$Pil$Pim$Pin$Pir$Piw$Pi{$Pi|$Pi}$Pi!P$Pi!S$Pi!U$Pi!V$Pi!W$Pi!Y$Pi!Z$Pi![$Pi!`$Pi!d$Pi!g$Pi!j$Pi!m$Pi!q$Pi!u$Pi!w$Pi!y$Pi!z$Pi!|$Pi#O$Pi#Q$Pi#S$Pi#U$Pi#X$Pi#Y$Pi#Z$Pi#[$Pi#^$Pi#h$Pi#l$Pi#r$Pi#s$Pi#t$Pi#u$Pi#v$Pi#w$Pi#x$Pi$^$Pi$_$Pi$`$Pi$a$Pi$b$Pi$g$Pi$h$Pi$j$Pi$z$Pi%b$Pi%c$Pi%d$Pi%f$Pi%h$Pi%i$Pi%j$Pi%m$Pi%n$Pi%o$Pi%p$Pi%q$Pi%w$Pi%y$Pi&S$Pi&U$Pi&Y$Pi&Z$Pi&[$Pi&^$Pi&_$Pi&a$Pi&b$Pi&c$Pi&y$Pi'R$Pi'b$Pi(d$Pi)W$Pi!h$Pi$o$Pi$p$Pi$q$Pi~O$T'eOQ$QiR$QiS$QiT$Qia$Qid$Qih$Qil$Qim$Qin$Qir$Qiw$Qi{$Qi|$Qi}$Qi!P$Qi!S$Qi!U$Qi!V$Qi!W$Qi!Y$Qi!Z$Qi![$Qi!`$Qi!d$Qi!g$Qi!j$Qi!m$Qi!q$Qi!u$Qi!w$Qi!y$Qi!z$Qi!|$Qi#O$Qi#Q$Qi#S$Qi#U$Qi#X$Qi#Y$Qi#Z$Qi#[$Qi#^$Qi#h$Qi#l$Qi#r$Qi#s$Qi#t$Qi#u$Qi#v$Qi#w$Qi#x$Qi$^$Qi$_$Qi$`$Qi$a$Qi$b$Qi$g$Qi$h$Qi$j$Qi$z$Qi%b$Qi%c$Qi%d$Qi%f$Qi%h$Qi%i$Qi%j$Qi%m$Qi%n$Qi%o$Qi%p$Qi%q$Qi%w$Qi%y$Qi&S$Qi&U$Qi&Y$Qi&Z$Qi&[$Qi&^$Qi&_$Qi&a$Qi&b$Qi&c$Qi&y$Qi'R$Qi'b$Qi(d$Qi)W$Qi!h$Qi$o$Qi$p$Qi$q$Qi~Oh.bOl.bO~On.bO#X!OO#Y!PO#Z!QO#[!RO~O#l&uO&z&vOQ$YiR$YiS$YiT$Yia$Yid$Yih$Yil$Yim$Yin$Yir$Yiw$Yi{$Yi|$Yi}$Yi!P$Yi!S$Yi!U$Yi!V$Yi!W$Yi!Y$Yi!Z$Yi![$Yi!`$Yi!d$Yi!g$Yi!j$Yi!m$Yi!q$Yi!u$Yi!w$Yi!y$Yi!z$Yi!|$Yi#O$Yi#Q$Yi#S$Yi#U$Yi#X$Yi#Y$Yi#Z$Yi#[$Yi#^$Yi#h$Yi#l$Yi#r$Yi#s$Yi#t$Yi#u$Yi#v$Yi#w$Yi#x$Yi$^$Yi$_$Yi$`$Yi$a$Yi$b$Yi$g$Yi$h$Yi$j$Yi$z$Yi%b$Yi%c$Yi%d$Yi%f$Yi%h$Yi%i$Yi%j$Yi%m$Yi%n$Yi%o$Yi%p$Yi%q$Yi%w$Yi%y$Yi&S$Yi&U$Yi&Y$Yi&Z$Yi&[$Yi&^$Yi&_$Yi&a$Yi&b$Yi&c$Yi&y$Yi'R$Yi'b$Yi(d$Yi)W$Yi!h$Yi$o$Yi$p$Yi$q$Yi~O$T'eOQ$UiR$UiS$UiT$Uia$Uid$Uih$Uil$Uim$Uin$Uir$Uiw$Ui{$Ui|$Ui}$Ui!P$Ui!S$Ui!U$Ui!V$Ui!W$Ui!Y$Ui!Z$Ui![$Ui!`$Ui!d$Ui!g$Ui!j$Ui!m$Ui!q$Ui!u$Ui!w$Ui!y$Ui!z$Ui!|$Ui#O$Ui#Q$Ui#S$Ui#U$Ui#X$Ui#Y$Ui#Z$Ui#[$Ui#^$Ui#h$Ui#l$Ui#r$Ui#s$Ui#t$Ui#u$Ui#v$Ui#w$Ui#x$Ui$^$Ui$_$Ui$`$Ui$a$Ui$b$Ui$g$Ui$h$Ui$j$Ui$z$Ui%b$Ui%c$Ui%d$Ui%f$Ui%h$Ui%i$Ui%j$Ui%m$Ui%n$Ui%o$Ui%p$Ui%q$Ui%w$Ui%y$Ui&S$Ui&U$Ui&Y$Ui&Z$Ui&[$Ui&^$Ui&_$Ui&a$Ui&b$Ui&c$Ui&y$Ui'R$Ui'b$Ui(d$Ui)W$Ui!h$Ui$o$Ui$p$Ui$q$Ui~O#l&uO&z&vO~P#GpO$T'eOQ$WiR$WiS$WiT$Wia$Wid$Wih$Wil$Wim$Win$Wir$Wiw$Wi{$Wi|$Wi}$Wi!P$Wi!S$Wi!U$Wi!V$Wi!W$Wi!Y$Wi!Z$Wi![$Wi!`$Wi!d$Wi!g$Wi!j$Wi!m$Wi!q$Wi!u$Wi!w$Wi!y$Wi!z$Wi!|$Wi#O$Wi#Q$Wi#S$Wi#U$Wi#X$Wi#Y$Wi#Z$Wi#[$Wi#^$Wi#h$Wi#l$Wi#r$Wi#s$Wi#t$Wi#u$Wi#v$Wi#w$Wi#x$Wi$^$Wi$_$Wi$`$Wi$a$Wi$b$Wi$g$Wi$h$Wi$j$Wi$z$Wi%b$Wi%c$Wi%d$Wi%f$Wi%h$Wi%i$Wi%j$Wi%m$Wi%n$Wi%o$Wi%p$Wi%q$Wi%w$Wi%y$Wi&S$Wi&U$Wi&Y$Wi&Z$Wi&[$Wi&^$Wi&_$Wi&a$Wi&b$Wi&c$Wi&y$Wi'R$Wi'b$Wi(d$Wi)W$Wi!h$Wi$o$Wi$p$Wi$q$Wi~On.bO~Ov&kX!^&kX!b&kX!s&kX#l&kX#s&kX#t&kX#u&kX#v&kX#w&kX#x&kX$s&kX$u&kX$v&kX$x&kX$y&kX${&kX$|&kX$}&kX%O&kX%P&kX%Q&kX%R&kX%S&kX%T&kX%U&kX%V&kX%W&kX%Y&kX%Z&kX%[&kX%]&kX%^&kX%_&kX%`&kX&W&kX&z&kX!_&kX!i&kX!h&kX#b&kX~PCWO!^&rO!b&rO#l(Za&z(Za!_(Za~Ov(Zaw(Za!U(Za!j(Za!m(Za!s(Za#Q(Za#S(Za#U(Za#s(Za#t(Za#u(Za#v(Za#w(Za#x(Za$s(Za$u(Za$v(Za$x(Za$y(Za$z(Za${(Za$|(Za$}(Za%O(Za%P(Za%Q(Za%R(Za%S(Za%T(Za%U(Za%V(Za%W(Za%Y(Za%Z(Za%[(Za%](Za%^(Za%_(Za%`(Za%f(Za&W(Za!i(Za!h(Za#b(Za~P'8nO!^&rO!b&rOv(OXw(OX!U(OX!j(OX!m(OX!s(OX#Q(OX#S(OX#U(OX#l(OX#s(OX#t(OX#u(OX#v(OX#w(OX#x(OX$s(OX$u(OX$v(OX$x(OX$y(OX$z(OX${(OX$|(OX$}(OX%O(OX%P(OX%Q(OX%R(OX%S(OX%T(OX%U(OX%V(OX%W(OX%Y(OX%Z(OX%[(OX%](OX%^(OX%_(OX%`(OX%f(OX&W(OX&z(OX!_(OX!i(OX!h(OX#b(OX~O!^&rO!_.mO!b&rO~Ov)`Ow$ZO!j$VO%f&TO!^)Ua!b)Ua!_)Ua~O!U)Ua!m)Ua!s)Ua#Q)Ua#S)Ua#U)Ua#l)Ua#s)Ua#t)Ua#u)Ua#v)Ua#w)Ua#x)Ua$s)Ua$u)Ua$v)Ua$x)Ua$y)Ua$z)Ua${)Ua$|)Ua$})Ua%O)Ua%P)Ua%Q)Ua%R)Ua%S)Ua%T)Ua%U)Ua%V)Ua%W)Ua%Y)Ua%Z)Ua%[)Ua%])Ua%^)Ua%_)Ua%`)Ua&W)Ua&z)Ua!i)Ua!h)Ua#b)Ua~P'?kOh.oOl.oOr(RO~O!_.rO~O!_.sO~O!^&rO!b&rOv)TXw)TX!U)TX!i)TX!j)TX!m)TX!s)TX#Q)TX#S)TX#U)TX$s)TX$u)TX$v)TX$x)TX$y)TX$z)TX${)TX$|)TX$})TX%O)TX%P)TX%Q)TX%R)TX%S)TX%T)TX%U)TX%V)TX%W)TX%Y)TX%Z)TX%[)TX%])TX%^)TX%_)TX%`)TX%f)TX&W)TX!h)TX#b)TX#l)TX&z)TX!_)TX~Or(lO~Oi.uO'V$_O'W$_O~Oi,eO'V$_O'W$_OZ'ai~O].wO^(wO~OP.xOr.xO~OP.{Or.{O~O].}O~P.jO!^&ka!_&ka!b&ka!i&ka!h&ka#l&ka#s&ka#t&ka#u&ka#v&ka#w&ka#x&ka&z&ka#b&ka~P$8ROv!liv!lqw!liw!lq!U!li!U!lq!^!li!^!lq!b!li!b!lq!j!li!j!lq!m!li!m!lq!s!li!s!lq#Q!li#Q!lq#S!li#S!lq#U!li#U!lq#l!li#l!lq#s!li#s!lq#t!li#t!lq#u!li#u!lq#v!li#v!lq#w!li#w!lq#x!li#x!lq$s!li$s!lq$u!li$u!lq$v!li$v!lq$x!li$x!lq$y!li$y!lq$z!li$z!lq${!li${!lq$|!li$|!lq$}!li$}!lq%O!li%O!lq%P!li%P!lq%Q!li%Q!lq%R!li%R!lq%S!li%S!lq%T!li%T!lq%U!li%U!lq%V!li%V!lq%W!li%W!lq%Y!li%Y!lq%Z!li%Z!lq%[!li%[!lq%]!li%]!lq%^!li%^!lq%_!li%_!lq%`!li%`!lq%f!li%f!lq&W!li&W!lq&z!li&z!lq!_!li!_!lq!i!li!i!lq!h!li!h!lq#b!li#b!lq~O$s%dO$u%fO!^$tq!b$tq#l$tq#s$tq#t$tq#u$tq#v$tq#w$tq#x$tq%^$tq%_$tq%`$tq&z$tq!_$tq!i$tq!h$tq#b$tq~P%+oO!^&Vq!b&Vq#l&Vq#s&Vq#t&Vq#u&Vq#v&Vq#w&Vq#x&Vq$s&Vq$u&Vq%^&Vq%_&Vq%`&Vq&z&Vq!_&Vq!i&Vq!h&Vq#b&Vq~P%+oO`/SO'S/RO~O]/TO~O]/TO~P.jO'S/UO~O`/VO'S/UO~O`/XO'S/WO~O]/YO~O]/YO~P.jOZ/ZO~O]/]O^/[O~P.jO]/]O~O`/`O'S/_O~O]/]O~P.jO^/bO~O!_/dO~O!^&mX!_&mX!b&mX~P$4TO!^&rO!b&rO!_'wa~O#|.QO$O.UOQ#zqR#zqS#zqT#zqa#zqd#zqh#zql#zqm#zqn#zqr#zqw#zq{#zq|#zq}#zq!P#zq!S#zq!U#zq!V#zq!W#zq!Y#zq!Z#zq![#zq!`#zq!d#zq!g#zq!j#zq!m#zq!q#zq!u#zq!w#zq!y#zq!z#zq!|#zq#O#zq#Q#zq#S#zq#U#zq#X#zq#Y#zq#Z#zq#[#zq#^#zq#h#zq#l#zq#r#zq#s#zq#t#zq#u#zq#v#zq#w#zq#x#zq$^#zq$_#zq$`#zq$a#zq$b#zq$g#zq$h#zq$j#zq$z#zq%b#zq%c#zq%d#zq%f#zq%h#zq%i#zq%j#zq%m#zq%n#zq%o#zq%p#zq%q#zq%w#zq%y#zq&S#zq&U#zq&Y#zq&Z#zq&[#zq&^#zq&_#zq&a#zq&b#zq&c#zq&y#zq'R#zq'b#zq(d#zq)W#zq!h#zq$o#zq$p#zq$q#zq~O#|.QO$O.UOQ$PqR$PqS$PqT$Pqa$Pqd$Pqh$Pql$Pqm$Pqn$Pqr$Pqw$Pq{$Pq|$Pq}$Pq!P$Pq!S$Pq!U$Pq!V$Pq!W$Pq!Y$Pq!Z$Pq![$Pq!`$Pq!d$Pq!g$Pq!j$Pq!m$Pq!q$Pq!u$Pq!w$Pq!y$Pq!z$Pq!|$Pq#O$Pq#Q$Pq#S$Pq#U$Pq#X$Pq#Y$Pq#Z$Pq#[$Pq#^$Pq#h$Pq#l$Pq#r$Pq#s$Pq#t$Pq#u$Pq#v$Pq#w$Pq#x$Pq$^$Pq$_$Pq$`$Pq$a$Pq$b$Pq$g$Pq$h$Pq$j$Pq$z$Pq%b$Pq%c$Pq%d$Pq%f$Pq%h$Pq%i$Pq%j$Pq%m$Pq%n$Pq%o$Pq%p$Pq%q$Pq%w$Pq%y$Pq&S$Pq&U$Pq&Y$Pq&Z$Pq&[$Pq&^$Pq&_$Pq&a$Pq&b$Pq&c$Pq&y$Pq'R$Pq'b$Pq(d$Pq)W$Pq!h$Pq$o$Pq$p$Pq$q$Pq~O$T'eOQ$QqR$QqS$QqT$Qqa$Qqd$Qqh$Qql$Qqm$Qqn$Qqr$Qqw$Qq{$Qq|$Qq}$Qq!P$Qq!S$Qq!U$Qq!V$Qq!W$Qq!Y$Qq!Z$Qq![$Qq!`$Qq!d$Qq!g$Qq!j$Qq!m$Qq!q$Qq!u$Qq!w$Qq!y$Qq!z$Qq!|$Qq#O$Qq#Q$Qq#S$Qq#U$Qq#X$Qq#Y$Qq#Z$Qq#[$Qq#^$Qq#h$Qq#l$Qq#r$Qq#s$Qq#t$Qq#u$Qq#v$Qq#w$Qq#x$Qq$^$Qq$_$Qq$`$Qq$a$Qq$b$Qq$g$Qq$h$Qq$j$Qq$z$Qq%b$Qq%c$Qq%d$Qq%f$Qq%h$Qq%i$Qq%j$Qq%m$Qq%n$Qq%o$Qq%p$Qq%q$Qq%w$Qq%y$Qq&S$Qq&U$Qq&Y$Qq&Z$Qq&[$Qq&^$Qq&_$Qq&a$Qq&b$Qq&c$Qq&y$Qq'R$Qq'b$Qq(d$Qq)W$Qq!h$Qq$o$Qq$p$Qq$q$Qq~O$T'eOQ$UqR$UqS$UqT$Uqa$Uqd$Uqh$Uql$Uqm$Uqn$Uqr$Uqw$Uq{$Uq|$Uq}$Uq!P$Uq!S$Uq!U$Uq!V$Uq!W$Uq!Y$Uq!Z$Uq![$Uq!`$Uq!d$Uq!g$Uq!j$Uq!m$Uq!q$Uq!u$Uq!w$Uq!y$Uq!z$Uq!|$Uq#O$Uq#Q$Uq#S$Uq#U$Uq#X$Uq#Y$Uq#Z$Uq#[$Uq#^$Uq#h$Uq#l$Uq#r$Uq#s$Uq#t$Uq#u$Uq#v$Uq#w$Uq#x$Uq$^$Uq$_$Uq$`$Uq$a$Uq$b$Uq$g$Uq$h$Uq$j$Uq$z$Uq%b$Uq%c$Uq%d$Uq%f$Uq%h$Uq%i$Uq%j$Uq%m$Uq%n$Uq%o$Uq%p$Uq%q$Uq%w$Uq%y$Uq&S$Uq&U$Uq&Y$Uq&Z$Uq&[$Uq&^$Uq&_$Uq&a$Uq&b$Uq&c$Uq&y$Uq'R$Uq'b$Uq(d$Uq)W$Uq!h$Uq$o$Uq$p$Uq$q$Uq~O$T'eOQ$WqR$WqS$WqT$Wqa$Wqd$Wqh$Wql$Wqm$Wqn$Wqr$Wqw$Wq{$Wq|$Wq}$Wq!P$Wq!S$Wq!U$Wq!V$Wq!W$Wq!Y$Wq!Z$Wq![$Wq!`$Wq!d$Wq!g$Wq!j$Wq!m$Wq!q$Wq!u$Wq!w$Wq!y$Wq!z$Wq!|$Wq#O$Wq#Q$Wq#S$Wq#U$Wq#X$Wq#Y$Wq#Z$Wq#[$Wq#^$Wq#h$Wq#l$Wq#r$Wq#s$Wq#t$Wq#u$Wq#v$Wq#w$Wq#x$Wq$^$Wq$_$Wq$`$Wq$a$Wq$b$Wq$g$Wq$h$Wq$j$Wq$z$Wq%b$Wq%c$Wq%d$Wq%f$Wq%h$Wq%i$Wq%j$Wq%m$Wq%n$Wq%o$Wq%p$Wq%q$Wq%w$Wq%y$Wq&S$Wq&U$Wq&Y$Wq&Z$Wq&[$Wq&^$Wq&_$Wq&a$Wq&b$Wq&c$Wq&y$Wq'R$Wq'b$Wq(d$Wq)W$Wq!h$Wq$o$Wq$p$Wq$q$Wq~On/wO~O!_/yO~Ov&rX!^&rX!b&rX!s&rX#l&rX#s&rX#t&rX#u&rX#v&rX#w&rX#x&rX$s&rX$u&rX$v&rX$x&rX$y&rX${&rX$|&rX$}&rX%O&rX%P&rX%Q&rX%R&rX%S&rX%T&rX%U&rX%V&rX%W&rX%Y&rX%Z&rX%[&rX%]&rX%^&rX%_&rX%`&rX&W&rX&z&rX!_&rX!i&rX!h&rX#b&rX~P!&jO!^&rO!b&rOv(Oaw(Oa!U(Oa!j(Oa!m(Oa!s(Oa#Q(Oa#S(Oa#U(Oa#l(Oa#s(Oa#t(Oa#u(Oa#v(Oa#w(Oa#x(Oa$s(Oa$u(Oa$v(Oa$x(Oa$y(Oa$z(Oa${(Oa$|(Oa$}(Oa%O(Oa%P(Oa%Q(Oa%R(Oa%S(Oa%T(Oa%U(Oa%V(Oa%W(Oa%Y(Oa%Z(Oa%[(Oa%](Oa%^(Oa%_(Oa%`(Oa%f(Oa&W(Oa&z(Oa!_(Oa!i(Oa!h(Oa#b(Oa~O!h/zO~Ov)`Ow$ZO!j$VO!U)Ti!^)Ti!b)Ti!m)Ti!s)Ti#Q)Ti#S)Ti#U)Ti#l)Ti#s)Ti#t)Ti#u)Ti#v)Ti#w)Ti#x)Ti$s)Ti$u)Ti$v)Ti$x)Ti$y)Ti$z)Ti${)Ti$|)Ti$})Ti%O)Ti%P)Ti%Q)Ti%R)Ti%S)Ti%T)Ti%U)Ti%V)Ti%W)Ti%Y)Ti%Z)Ti%[)Ti%])Ti%^)Ti%_)Ti%`)Ti%f)Ti&W)Ti&z)Ti!_)Ti!i)Ti!h)Ti#b)Ti~O!`'VOv%{qw%{q!U%{q!^%{q!b%{q!j%{q!m%{q!s%{q#Q%{q#S%{q#U%{q#l%{q#s%{q#t%{q#u%{q#v%{q#w%{q#x%{q$s%{q$u%{q$v%{q$x%{q$y%{q$z%{q${%{q$|%{q$}%{q%O%{q%P%{q%Q%{q%R%{q%S%{q%T%{q%U%{q%V%{q%W%{q%Y%{q%Z%{q%[%{q%]%{q%^%{q%_%{q%`%{q%f%{q&W%{q&z%{q!_%{q!i%{q!h%{q#b%{q~O!`'VOv%xyw%xy!U%xy!^%xy!b%xy!j%xy!m%xy!s%xy#Q%xy#S%xy#U%xy#l%xy#s%xy#t%xy#u%xy#v%xy#w%xy#x%xy$s%xy$u%xy$v%xy$x%xy$y%xy$z%xy${%xy$|%xy$}%xy%O%xy%P%xy%Q%xy%R%xy%S%xy%T%xy%U%xy%V%xy%W%xy%Y%xy%Z%xy%[%xy%]%xy%^%xy%_%xy%`%xy%f%xy&W%xy&z%xy!_%xy!i%xy!h%xy#b%xy~O!^!ai!b!ai#l#ci&z#ci!_#ci~Ov!aiw!ai!U!ai!j!ai!m!ai!s!ai#Q!ai#S!ai#U!ai$s!ai$u!ai$v!ai$x!ai$y!ai$z!ai${!ai$|!ai$}!ai%O!ai%P!ai%Q!ai%R!ai%S!ai%T!ai%U!ai%V!ai%W!ai%Y!ai%Z!ai%[!ai%]!ai%^!ai%_!ai%`!ai%f!ai&W!ai~P)(^O!^&Vy!b&Vy#l&Vy#s&Vy#t&Vy#u&Vy#v&Vy#w&Vy#x&Vy$s&Vy$u&Vy%^&Vy%_&Vy%`&Vy&z&Vy!_&Vy!i&Vy!h&Vy#b&Vy~P%+oO'S0PO~O`0QO'S0PO~O'S0RO~O'S0SO~O`0TO'S0SO~O]0UO~O`0WO'S0VO~O]0UO~P.jO]0UO^0XO~P.jO]0ZO~O^0[O~O!_0aO~O$T'eOQ$WyR$WyS$WyT$Wya$Wyd$Wyh$Wyl$Wym$Wyn$Wyr$Wyw$Wy{$Wy|$Wy}$Wy!P$Wy!S$Wy!U$Wy!V$Wy!W$Wy!Y$Wy!Z$Wy![$Wy!`$Wy!d$Wy!g$Wy!j$Wy!m$Wy!q$Wy!u$Wy!w$Wy!y$Wy!z$Wy!|$Wy#O$Wy#Q$Wy#S$Wy#U$Wy#X$Wy#Y$Wy#Z$Wy#[$Wy#^$Wy#h$Wy#l$Wy#r$Wy#s$Wy#t$Wy#u$Wy#v$Wy#w$Wy#x$Wy$^$Wy$_$Wy$`$Wy$a$Wy$b$Wy$g$Wy$h$Wy$j$Wy$z$Wy%b$Wy%c$Wy%d$Wy%f$Wy%h$Wy%i$Wy%j$Wy%m$Wy%n$Wy%o$Wy%p$Wy%q$Wy%w$Wy%y$Wy&S$Wy&U$Wy&Y$Wy&Z$Wy&[$Wy&^$Wy&_$Wy&a$Wy&b$Wy&c$Wy&y$Wy'R$Wy'b$Wy(d$Wy)W$Wy!h$Wy$o$Wy$p$Wy$q$Wy~O#l&uO&z&vO~P%0VO!_0gO~P%0VO!_0hO~O'S0iO~O'S0jO~O`0lO'S0kO~O]0mO~O]0mO~P.jO`0oO'S0nO~O]0pO~O$T'eOQ$W!RR$W!RS$W!RT$W!Ra$W!Rd$W!Rh$W!Rl$W!Rm$W!Rn$W!Rr$W!Rw$W!R{$W!R|$W!R}$W!R!P$W!R!S$W!R!U$W!R!V$W!R!W$W!R!Y$W!R!Z$W!R![$W!R!`$W!R!d$W!R!g$W!R!j$W!R!m$W!R!q$W!R!u$W!R!w$W!R!y$W!R!z$W!R!|$W!R#O$W!R#Q$W!R#S$W!R#U$W!R#X$W!R#Y$W!R#Z$W!R#[$W!R#^$W!R#h$W!R#l$W!R#r$W!R#s$W!R#t$W!R#u$W!R#v$W!R#w$W!R#x$W!R$^$W!R$_$W!R$`$W!R$a$W!R$b$W!R$g$W!R$h$W!R$j$W!R$z$W!R%b$W!R%c$W!R%d$W!R%f$W!R%h$W!R%i$W!R%j$W!R%m$W!R%n$W!R%o$W!R%p$W!R%q$W!R%w$W!R%y$W!R&S$W!R&U$W!R&Y$W!R&Z$W!R&[$W!R&^$W!R&_$W!R&a$W!R&b$W!R&c$W!R&y$W!R'R$W!R'b$W!R(d$W!R)W$W!R!h$W!R$o$W!R$p$W!R$q$W!R~O'S0wO~O`0xO'S0wO~O'S0yO~O`0zO'S0yO~O$T'eOQ$W!ZR$W!ZS$W!ZT$W!Za$W!Zd$W!Zh$W!Zl$W!Zm$W!Zn$W!Zr$W!Zw$W!Z{$W!Z|$W!Z}$W!Z!P$W!Z!S$W!Z!U$W!Z!V$W!Z!W$W!Z!Y$W!Z!Z$W!Z![$W!Z!`$W!Z!d$W!Z!g$W!Z!j$W!Z!m$W!Z!q$W!Z!u$W!Z!w$W!Z!y$W!Z!z$W!Z!|$W!Z#O$W!Z#Q$W!Z#S$W!Z#U$W!Z#X$W!Z#Y$W!Z#Z$W!Z#[$W!Z#^$W!Z#h$W!Z#l$W!Z#r$W!Z#s$W!Z#t$W!Z#u$W!Z#v$W!Z#w$W!Z#x$W!Z$^$W!Z$_$W!Z$`$W!Z$a$W!Z$b$W!Z$g$W!Z$h$W!Z$j$W!Z$z$W!Z%b$W!Z%c$W!Z%d$W!Z%f$W!Z%h$W!Z%i$W!Z%j$W!Z%m$W!Z%n$W!Z%o$W!Z%p$W!Z%q$W!Z%w$W!Z%y$W!Z&S$W!Z&U$W!Z&Y$W!Z&Z$W!Z&[$W!Z&^$W!Z&_$W!Z&a$W!Z&b$W!Z&c$W!Z&y$W!Z'R$W!Z'b$W!Z(d$W!Z)W$W!Z!h$W!Z$o$W!Z$p$W!Z$q$W!Z~O!_0}O~P%0VO'S1OO~O'S1PO~O!_1QO~P%0VOh&`Ol&`Ow&aO!|1TO~Oh&bOl&bOw&cO!|1TO~OR2gOS2nOr1]O|2cO!m2dO$z1ZO%b1ZO%c1ZO%f2fO~P!I_OR2gOS2nOr1]O|2cO!m2dO$z1ZO%b1ZO%c1ZO%f2fO~P#!mOr1tO~P#8_Oq$pO!^1[Oh%|Xl%|Xr%|Xv#iXw&OX!i#VX!s#VX$s#VX$u#VX$v#VX$x#VX$y#VX${#VX$|#VX$}#VX%O#VX%P#VX%Q#VX%R#VX%S#VX%T#VX%U#VX%V#VX%W#VX%Y#VX%Z#VX%[#VX%]#VX%^#VX%_#VX%`#VX&W#VX!h#VX#b#VX#l#VX&z#VX!_#VX~P!CqO$s1zO~P#3OOv$waw$wa!U$wa!i$wa!j$wa!m$wa!s$wa#Q$wa#S$wa#U$wa$s$wa$u$wa$v$wa$x$wa$y$wa$z$wa${$wa$|$wa%V$wa%W$wa%Y$wa%Z$wa%[$wa%]$wa%^$wa%_$wa%`$wa%f$wa&W$wa!h$wa#b$wa#l$wa&z$wa!_$wa~O$}1aO%O%hO%P%iO%Q%jO%R%kO%S$wa%T$wa%U$wa~P)HXO%S1cO%T%nO%U%oO$}$wa%O$wa%P$wa%Q$wa%R$wa~P)HXOv)`Ow$ZO!j$VO!U(PX!^(PX!b(PX!m(PX!s(PX#Q(PX#S(PX#U(PX#l(PX#s(PX#t(PX#u(PX#v(PX#w(PX#x(PX$s(PX$u(PX$v(PX$x(PX$y(PX$z(PX${(PX$|(PX$}(PX%O(PX%P(PX%Q(PX%R(PX%S(PX%T(PX%U(PX%V(PX%W(PX%Y(PX%Z(PX%[(PX%](PX%^(PX%_(PX%`(PX%f(PX&W(PX&z(PX!_(PX!i(PX!h(PX#b(PX~Ov)`Ow$ZO!U1hO!j$VO!m1gO!s1fO#Q1fO#S%sO#U1fO$s1^O$u1_O$v%eO$x1eO$y1eO$z1gO${1gO$|1hO$}1aO%O%hO%P%iO%Q%jO%R%kO%S1cO%T%nO%U%oO%V1iO%W%rO%Y1kO%Z1lO%[1mO%]1nO%f&TO&W2hO~O!i%aa%^%aa%_%aa%`%aa!h%aa#b%aa#l%aa&z%aa!_%aa~P)N}O!^&rO!b&rO#l(ZX&z(ZX!_(ZX~P$8RO!^1[O~P%$TOv)`Ow$ZO!U1hO!j$VO!m1gO!s1fO#Q1fO#S%sO#U1fO$v%eO$x1eO$y1eO$z1gO${1gO$|1hO$}1aO%O%hO%P%iO%Q%jO%R%kO%S1cO%T%nO%U%oO%V1iO%W%rO%Y1kO%Z1lO%[1mO%]1nO%f&TO&W2hO~O!i&Ta$s&Ta$u&Ta%^&Ta%_&Ta%`&Ta!h&Ta#b&Ta#l&Ta&z&Ta!_&Ta~P*$cO!i!]i%^!]i%_!]i%`!]i!h!]i#b!]i#l!]i&z!]i!_!]i~P)N}O!i$ri%^$ri%_$ri%`$ri!h$ri#b$ri#l$ri&z$ri!_$ri~P)N}O!i$ti%^$ti%_$ti%`$ti!h$ti#b$ti#l$ti&z$ti!_$ti~P)N}Ov)`Ow$ZO!j$VO$x1eO%f&TO!U$wi!i$wi!m$wi!s$wi#Q$wi#S$wi#U$wi$s$wi$u$wi$v$wi$z$wi${$wi$|$wi$}$wi%O$wi%P$wi%Q$wi%R$wi%S$wi%T$wi%U$wi%V$wi%W$wi%Y$wi%Z$wi%[$wi%]$wi%^$wi%_$wi%`$wi&W$wi!h$wi#b$wi#l$wi&z$wi!_$wi~O$y1eO~P*)^O!U1hO!m1gO!s1fO#Q1fO#U1fO$x1eO$y1eO$z1gO${1gO$|1hO~P%IZO!U1hO!m1gO!s1fO#Q1fO#U1fO$x1eO$y1eO$z1gO${1gO$|1hO$}1aO~P%LmO$y$wi~P*)^Ov)`Ow$ZO!j$VO!s1fO#Q1fO#U1fO$v%eO$x1eO$y1eO%f&TO!U$wi!i$wi#S$wi$s$wi$u$wi$|$wi$}$wi%O$wi%P$wi%Q$wi%R$wi%S$wi%T$wi%U$wi%V$wi%W$wi%Y$wi%Z$wi%[$wi%]$wi%^$wi%_$wi%`$wi&W$wi!h$wi#b$wi#l$wi&z$wi!_$wi~O!m$wi$z$wi${$wi~P*-xO!m1gO$z1gO${1gO~P*-xOv)`Ow$ZO!U1hO!j$VO!m1gO!s1fO#Q1fO#U1fO$v%eO$x1eO$y1eO$z1gO${1gO$|1hO$}1aO%O%hO%P%iO%Q%jO%R%kO%f&TO!i$wi#S$wi$s$wi$u$wi%Y$wi%Z$wi%[$wi%]$wi%^$wi%_$wi%`$wi&W$wi!h$wi#b$wi#l$wi&z$wi!_$wi~O%S$wi%T$wi%U$wi%V$wi%W$wi~P*1PO%S1cO%T%nO%U%oO%V1iO%W%rO~P*1POv)`Ow$ZO!U1hO!j$VO!m1gO!s1fO#Q1fO#S%sO#U1fO$v%eO$x1eO$y1eO$z1gO${1gO$|1hO$}1aO%O%hO%P%iO%Q%jO%R%kO%S1cO%T%nO%U%oO%V1iO%W%rO%f&TO!i$wi$s$wi$u$wi%Z$wi%[$wi%]$wi%^$wi%_$wi%`$wi&W$wi!h$wi#b$wi#l$wi&z$wi!_$wi~O%Y$wi~P*4^O%Y1kO~P*4^Ov)`Ow$ZO!U1hO!j$VO!m1gO!s1fO#Q1fO#S%sO#U1fO$v%eO$x1eO$y1eO$z1gO${1gO$|1hO$}1aO%O%hO%P%iO%Q%jO%R%kO%S1cO%T%nO%U%oO%V1iO%W%rO%Y1kO%Z1lO%f&TO!i$wi$s$wi$u$wi%]$wi%^$wi%_$wi%`$wi&W$wi!h$wi#b$wi#l$wi&z$wi!_$wi~O%[$wi~P*7_O%[1mO~P*7_O!i$wi%^$wi%_$wi%`$wi!h$wi#b$wi#l$wi&z$wi!_$wi~P)N}O%^%tO!i$wi%_$wi%`$wi!h$wi#b$wi#l$wi&z$wi!_$wi~P)N}O!_/PO~O!^)Ua!_)Ua!b)Ua~P$8RO!i$tq%^$tq%_$tq%`$tq!h$tq#b$tq#l$tq&z$tq!_$tq~P)N}O!i&Vq$s&Vq$u&Vq%^&Vq%_&Vq%`&Vq!h&Vq#b&Vq#l&Vq&z&Vq!_&Vq~P*$cOv#ciw#ci!U#ci!i#ci!j#ci!m#ci!s#ci#Q#ci#S#ci#U#ci$s#ci$u#ci$v#ci$x#ci$y#ci$z#ci${#ci$|#ci$}#ci%O#ci%P#ci%Q#ci%R#ci%S#ci%T#ci%U#ci%V#ci%W#ci%Y#ci%Z#ci%[#ci%]#ci%^#ci%_#ci%`#ci%f#ci&W#ci!h#ci#b#ci~P)(^O!i&Vy$s&Vy$u&Vy%^&Vy%_&Vy%`&Vy!h&Vy#b&Vy#l&Vy&z&Vy!_&Vy~P*$cO!^1[O~P:mOQUOR2gOS2nOT#bOh$tOl$tOmROnROr1]O{$PO|2cO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`$wO!dgO!gjO!jkO!m2dO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z1ZO%b1ZO%c1ZO%d#OO%f2fO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~Ow%`O~P*AVOQUOR2gOS2nOT#bOh$tOl$tOmROnROr1]Ow$yO{$PO|2cO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!dgO!gjO!jkO!m2dO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z1ZO%b1ZO%c1ZO%d#OO%f2fO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~O!`fOv%kX!i%kX!s%kX$s%kX$u%kX$v%kX$x%kX$y%kX${%kX$|%kX$}%kX%O%kX%P%kX%Q%kX%R%kX%S%kX%T%kX%U%kX%V%kX%W%kX%Y%kX%Z%kX%[%kX%]%kX%^%kX%_%kX%`%kX&W%kX!h%kX#b%kX#l%kX&z%kX!_%kX~P*EtO#b2YO~P#3OO#b2_O~P%0VOQUOR#WOS#_OT#bOh$tOl$tOmROnROr$uOw$yO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`2rO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~O!_2aO~OQUOR#WOS#_OT#bOh$tOl$tOmROnROr$uO{$PO|eO}[O!PVO!SYO!U^O!V$PO!W$PO!Y`O!ZaO![bO!`2rO!dgO!gjO!jkO!mmO!qoO!urO!wtO!yvO!zwO!|yO#OzO#Q{O#S|O#U}O#X!OO#Y!PO#Z!QO#[!RO#^!UO$z#PO%b#PO%c#PO%d#OO%f#RO%h#SO%i#TO%j#UO%m#XO%n#YO%o#ZO%p#[O%q#]O%w#eO%y#gO&S#mO&U#oO&Y#qO&Z#rO&[#sO&^#vO&_#wO&a#zO&b#{O&c#|O'RqO'bXO)W#kO~OwlO~P+#hOm2qO!`(ZO#OzO~O}#j!W!V%cr|lmn{#`!g$u${%f$u!m$x$z!|#O#Q#S#U!s!j$z~",
  goto: "$@W)iPPPPPPPPPPPPPPPPPPPPPPPPPP)j)jPPP*a+PPP+o+r+rPP,b.|1hPPP1kP1k1kP4QPPP6gPPP6gPPP8|P8|P;c6gPPPP;i;iP6g6g6gP6gP6gP6gP6gPP>TPApPE[P6gP6gP6g6gPPPP;iPGwPHUPHiJYPJkM[PMeP!!WPM[M[!!zM[!#QPPPPPPP!#_M[!#sP!#yPM[M[!$V!$`PM[M[M[PM[M[!%RM[PPPPPM[M[M[M[PPM[PM[M[M[!%[PPP6gP6gPP6gPPPPPPPPPPPPPPP!%aPPPPPPP6gPPP6gP6gPPP6g6gPPPPP6g!&m6g!)U6gP6gP!+k6g!+s6g!.]6g!0r6gP6gP6gP6gPPP6gPP6gPPPP!3Z!4g!4m!4t!5W!5^!5h!5y!6P!6V!6z!7R!7g!7m!7{!8V!8c!8i!:P!;cPPPPPPPPPPPPPP!;i!<X!?e!Ah!GX6g4Q!Iy!L`P!Lx# _# _# _6g##t#$b#%^#'s#)v#,]#.r#1X#1k#1k#4Q#6u#6u#6u#6u#9`#9c#9i#<W#<Z#<h#<q#>k#>w#?h#?q#?t#@R#@e#@w#A^#As#As#BY#Bf#Bl#Br#CP#CP#CP#CP#CPP#C]#C]#Cf#Co#Cr#Cw#Cw#Cw#C|#Ef#Ef#Ef#Ef#Fz#Fz#H[#Ij#J[#J[#Jz#Mc#Mc#Mc$ x$$a$$a$$a$$a$$a$&v$)_$)c$)lP$,W$.o$1W$3o$3o$3o$6U$6Y$8o$8o$;U$;X$=n$=n$=n$@TQ$UP!Y$iX$T$g$k&V&X&Y&[(f)V)W*^*_*a*d*e*i*j*k,q-V-_-b-e-g/^/a0YT+P&u+Q!Z$iX$T$g$k&V&X&Y&[(f)V)W*^*_*a*d*e*i*j*k,q-V-_-b-e-g/^/a0Y!Z$eX$T$g$k&V&X&Y&[(f)V)W*^*_*a*d*e*i*j*k,q-V-_-b-e-g/^/a0YR.|,o!Z$hX$T$g$k&V&X&Y&[(f)V)W*^*_*a*d*e*i*j*k,q-V-_-b-e-g/^/a0Y&RUOfklm!T!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y$z%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&m&s'P'V'`'y(Q)^)a)x*Z*v+i+z,O,P,V-P-p.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r&RTOfklm!T!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y$z%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&m&s'P'V'`'y(Q)^)a)x*Z*v+i+z,O,P,V-P-p.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rR(t$^%w]Ofklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r%w_Ofklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r%w$POfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r%wiOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rQ$}hR)h%O%v$POfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rV'|#f(O1V%v$POfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r!Y$iX$T$g$k&V&X&Y&[(f)V)W*^*_*a*d*e*i*j*k,q-V-_-b-e-g/^/a0YS&`y1TS&bz1UQ&d{Q&f|Q&k}V(m$X$m)`%v$POfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r!Y$iX$T$g$k&V&X&Y&[(f)V)W*^*_*a*d*e*i*j*k,q-V-_-b-e-g/^/a0YQ(Y#uQ(]#yQ(`$OQ,W(ZQ,X(^Q,Y(aR2q2s%v$POfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rQ(`$OR,Y(aS'R!Z${Q+]&}Q-u*yR-{+Ws&z!Z${&{&}'R*y+Y+Z+]+`-s-u-|.O/g/h0]Q'W!eQ'X!fQ'Y!gQ'_!iQ'a!jW'x#_#`2n2oS'z#c2pU(m$X$m)`Q)s%bQ+g'ZQ+h'[S+j']'bQ,S(SQ,a(mQ-z+UQ.a+kS.p,V2lQ/k.RQ/v.bQ/|.oQ/}.qQ0O.tR0d/wS'R!Z${Q+Y&{Q+]&}S-u*y+ZR/g-s^!kOl$R$s%^'P)^S'Q!Z${Q'^!hQ'l!sU'y#`#j2oS(U#l1WS(V#n1XQ*{&qQ+[&}Q+^'OQ+a'RQ+d'WQ+e'XQ+f'YQ+p'_Q+s'aQ+u'fS-t*y+]Q-v*|S-}+Y+`Q.[+gQ.]+hQ.`+jS/i-u.OQ/j-|Q/n.VQ/u.aS0^/g/hQ0_/kQ0c/vQ0f/yQ0q0]Q0s0aQ0u0dQ1R0}R1S1Q_!}Ol$R$s%^'P)^%v#aOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rQ&i|Q&q!WQ(S#hR.o,U!c&v!Y!k!m%V%Z%c&q&}'^'`'h(k)[)n)o*y*|+]+i+o+q+r,{-u.O.^._/g/h/x0]0`Q&}!ZR*y&oS!dO$RU%Vl%^'PT)[$s)^W'x#_#`2n2oS'z#c2pQ)s%bS.p,V2lR/}.qX.S+d+e.T.XQ.W+dQ.Y+eQ/m.TR/o.X_!hOl$R$s%^'P)^Q'h!kQ+o'^Q.Z+fQ.e+pQ.h+sQ/p.[Q/q.]Q/t.`Q0b/uQ0t0cR0|0u_!mOl$R$s%^'P)^V%[l%^'P!S%|n$v%R%T'S'r'v(X)X)u)v*S*T*U*V*W*X+q,u-O-Q.^/Q1s2]!]1j(h(p)i*Y*n*o*p*q*s/x/{0`0v0{1r1u1v1w1x2S2T2U2V2W2X2^2`2b2k#x#cOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$o$s$w$y%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.i.q2i2j2l2n2o2p2r!}2p$V$Z%Q&U&a&c&e&g&l.f.n/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h%w#dOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rU(m$X$m)`R.t,b%v#hOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rR,U(T%w#jOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r#x#lOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$o$s$w$y%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.i.q2i2j2l2n2o2p2r!}1W$V$Z%Q&U&a&c&e&g&l.f.n/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2hQ$gXQ(f$Tl)T$g(f)V*^*a*d*j,q-V-_-e-g/^0YQ)V$kQ*^&VQ*a&XQ*d&YQ*j&[Q,q)WQ-V*_Q-_*eQ-e*iQ-g*kQ/^-bR0Y/aQ(|$cR,k(|S$]S*tR(r$]`$`WZhpsux*gS(u$`,dT,d(v,fQ,f(vR.v,fQ(x$aS,g(x,iR,i(zW)b$v%R+q.^Q+{'vU,v)b+{,}R,}1sQ)f$|R,w)fQ-q*wR/f-qS&{!Z${`+X&{+Z+`-s-|.O/h0]Q+Z&}Q+`'RQ-s*yQ-|+YQ.O+]Q/h-uR0]/gS%^l'PR)p%^[$qd$u(W(o1]1tU)Z$q*r*zS*r&h(RR*z&pQ+Q&uR-w+QQ&t!XU+O&t+b.jQ+b'TR.j+|Q.T+dQ.X+eT/l.T.X^!lOl$R$s%^'P)^R'j!lQ)^$sR,s)^!Y%mn$v%R%T'S'r'v(X)X)u)v){*Q*R*S*T*U*V*W*X+q,u-O-Q.^/Q1s2]S)z%m1b!c1b(h(p)i*Y*n*o*p*q*s/x/{0`0v0{1r1u1v1w1x1|2Q2R2S2T2U2V2W2X2^2`2b2k!U%qn$v%R%T'S'r'v(X)X)u)v*R*S*T*U*V*W*X+q,u-O-Q.^/Q1s2]S)|%q1d!_1d(h(p)i*Y*n*o*p*q*s/x/{0`0v0{1r1u1v1w1x2R2S2T2U2V2W2X2^2`2b2kQ$ROR(c$R!Z$jX$T$g$k&V&X&Y&[(f)V)W*^*_*a*d*e*i*j*k,q-V-_-b-e-g/^/a0Y%zSOfklm!T!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y$z%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r!Y$iX$T$g$k&V&X&Y&[(f)V)W*^*_*a*d*e*i*j*k,q-V-_-b-e-g/^/a0YV*t&m*v-p$`$YQn$t$v%R%T%a'S'r's'v(X(h(p)X)i)u)v)w)y){)}*O*P*Q*R*S*T*U*V*W*X*Y*n*o*p*q*s+q,R,u-O-Q.^.l/Q/x/{0`0v0{1q1r1s1u1v1w1x1y1{1|1}2O2P2Q2R2S2T2U2V2W2X2]2^2`2b2kS(j$X)`U(}$c$d$hS,l)O)PR.y,n[nO$R$s%^'P)^Q$vf`%Rk$w$y%`'V,O2i2rQ%TlW%am#P1Z2dS'S!]&sQ'r#QS's#R2f`'v#_#`#c,V2l2n2o2pQ(X#pQ(h$VQ(p$ZQ)X$oQ)i%QQ)u%dQ)v%fS)w%g%xQ)y%lQ){%pQ)}%wQ*O%yQ*P%zQ*Q%{Q*R%|Q*S%}Q*T&OQ*U&PQ*V&QQ*W&RQ*X&SQ*Y&UQ*n&aQ*o&cQ*p&eQ*q&gQ*s&lQ+q'`Q,R(QS,u)a+zQ-O)xQ-Q*ZQ.^+iS.l,P2[Q/Q-PQ/x.fQ/{.nQ0`/rQ0v0eQ0{0r`1q#W#l#n'y.i1W1X2gQ1r2eS1s%b.qQ1u1YQ1v1[Q1w1^Q1x1_S1y1`1fQ1{1aQ1|1cQ1}1eQ2O1gQ2P1hQ2Q1iQ2R1jQ2S1kQ2T1lQ2U1mQ2V1nQ2W1oQ2X1pQ2]2jQ2^1zQ2`2YQ2b2_R2k2h%v$POfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rS&n!T$zS*u&m-pR-o*v%wWOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rQ$bWQ$lZQ%OhQ&WpQ&ZsQ&]uQ&_xR-a*g%wZOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r%wcOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rS$xf$wQ%SkS%Ul$yQ)r%`U+c'V'`+iQ.k,OQ2Z2iR2m2rW&s!X&t'T+b^)a$v%R)b+q,}.^1sS+z'v+{Q,V(YS-p*w-qS.i+|.jQ.n,QQ.q,WQ2[,^R2l2q%whOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r$`$WQn$t$v%R%T%a'S'r's'v(X(h(p)X)i)u)v)w)y){)}*O*P*Q*R*S*T*U*V*W*X*Y*n*o*p*q*s+q,R,u-O-Q.^.l/Q/x/{0`0v0{1q1r1s1u1v1w1x1y1{1|1}2O2P2Q2R2S2T2U2V2W2X2]2^2`2b2kS(i$X)`U)Q$c$d$hS,m)O)PR.z,n%wpOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r%wsOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r%wuOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rQ*i&[U-b*h*j*kQ-h*lS/a-f-gR/c-i%wxOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r^!TOl$R$s%^'P)^%h$zfkm!]#P#Q#R#W#_#`#c#l#n#p$V$Z$o$w$y%Q%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'V'`'y(Q)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rQ'c!jQ+l']Q+t'dR.c+m&P!SOfklm!]!j#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V']'`'d'y(Q)^)a)x*Z+i+m+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rR*x&mQ*w&mR/e-p^!ZOl$R$s%^'P)^%n${fkm!]#P#Q#R#W#_#`#c#f#l#n#p$V$Z$o$w$y%Q%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'V'`'y(O(Q)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1V1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rR&o!TR+W&yS$QO$RU%]l%^'PT)]$s)^_!WOl$R$s%^'P)^Q&w!YQ'g!kQ'k!mQ)l%VQ)m%ZQ)t%cQ*{&qQ+[&}S+n'^'hQ+r'`Q,_(kQ,r)[S,z)n)oS-t*y+]Q-v*|Q._+iQ.d+oQ.f+qQ.g+rQ/O,{S/i-u.OQ/r.^Q/s._S0^/g/hQ0e/xQ0q0]R0r0`^!YOl$R$s%^'P)^R+}'y^!XOl$R$s%^'P)^Q'T!]S'u#W2gS(U#l1WS(V#n1XS*}&s.iR+|'y_!]Ol$R$s%^'P)^R'U!]Y%cn!d%T%V)[Q)n%ZR,{)o^!eOl$R$s%^'P)^_%bn!d%T%V%Z)[)o^!fOl$R$s%^'P)^_%bn!d%T%V%Z)[)o^!gOl$R$s%^'P)^^%bn!d%T%V%Z)[)oR'Z!h^!iOl$R$s%^'P)^^%bn!d%T%V%Z)[)oR'[!h^!jOl$R$s%^'P)^^%bn!d%T%V%Z)[)oR']!he'w#_#`#c%b,V.q2l2n2o2pX.R+d+e.T.XX.V+d+e.T.Xg'f!k'^+f+p+s.[.].`/u0c0u^!sOl$R$s%^'P)^R'O!Z_!yOl$R$s%^'P)^_!{Ol$R$s%^'P)^R,`(kV%]l%^'PV%Zl%^'P!`%gn$v%R%T'S'r'v(X)X)u)v)y){*O*P*Q*R*S*T*U*V*W*X+q,u-O-Q.^/Q1s2]!i1`(h(p)i*Y*n*o*p*q*s/x/{0`0v0{1r1u1v1w1x1{1|2O2P2Q2R2S2T2U2V2W2X2^2`2b2k![%ln$v%R%T%m'S'r'v(X)X)u)v){*Q*R*S*T*U*V*W*X+q,u-O-Q.^/Q1s2]!e1a(h(p)i*Y*n*o*p*q*s/x/{0`0v0{1b1r1u1v1w1x1|2Q2R2S2T2U2V2W2X2^2`2b2k!W%pn$v%R%T%q'S'r'v(X)X)u)v*R*S*T*U*V*W*X+q,u-O-Q.^/Q1s2]!a1c(h(p)i*Y*n*o*p*q*s/x/{0`0v0{1d1r1u1v1w1x2R2S2T2U2V2W2X2^2`2b2k!U%{n$v%R%T'S'r'v(X)X)u)v*R*S*T*U*V*W*X+q,u-O-Q.^/Q1s2]!_1i(h(p)i*Y*n*o*p*q*s/x/{0`0v0{1r1u1v1w1x2R2S2T2U2V2W2X2^2`2b2kh&Rn$v%R%T'S'v*X+q,u.^1s2]q1o(h(p)i*Y*n*o*p*q*s/x/{0`0v0{2X2kf&Sn$v%R%T'S'v+q,u.^1s2]o1p(h(p)i*Y*n*o*p*q*s/x/{0`0v0{2k#x#QOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$o$s$w$y%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.i.q2i2j2l2n2o2p2r!}2e$V$Z%Q&U&a&c&e&g&l.f.n/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h%w#VOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r#x#`Ofklm!]#P#Q#R#W#_#`#c#l#n#p$R$o$s$w$y%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.i.q2i2j2l2n2o2p2r!}2o$V$Z%Q&U&a&c&e&g&l.f.n/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h%w#^Ofklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r#x#fOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$o$s$w$y%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.i.q2i2j2l2n2o2p2r!}1V$V$Z%Q&U&a&c&e&g&l.f.n/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2hT(P#f1VQ'}#fQ,Q(OR,^1V%v#iOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rV(n$X$m)`#x#nOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$o$s$w$y%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.i.q2i2j2l2n2o2p2r!}1X$V$Z%Q&U&a&c&e&g&l.f.n/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h#x#pOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$o$s$w$y%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.i.q2i2j2l2n2o2p2r!}1Y$V$Z%Q&U&a&c&e&g&l.f.n/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h#x#uOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$o$s$w$y%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.i.q2i2j2l2n2o2p2r!}2s$V$Z%Q&U&a&c&e&g&l.f.n/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h%w#tOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rT([#u2s%w#yOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r%w#xOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rR(_#y%w$OOfklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2r%w#}Ofklm!]#P#Q#R#W#_#`#c#l#n#p$R$V$Z$o$s$w$y%Q%^%`%b%d%f%g%l%p%w%x%y%z%{%|%}&O&P&Q&R&S&U&a&c&e&g&l&s'P'V'`'y(Q)^)a)x*Z+i+z,O,P,V-P.f.i.n.q/r0e0r1W1X1Y1Z1[1^1_1`1a1c1e1f1g1h1i1j1k1l1m1n1o1p1z2Y2[2_2d2e2f2g2h2i2j2l2n2o2p2rR(b$O",
  nodeNames: "⚠ UnrestrictedIdentifier SpecialScalarVariable NamedUnaryOperator ListOperator PGOperator HeredocStartIdentifier LaTeXImageCodeStart HeredocEndIdentifier InterpolatedStringContent EscapeSequence QuoteLikeStartDelimiter QuoteLikeSeparatorDelimiter QuoteLikeEndDelimiter StringContent QWElement RegexOptions PodDirective PodContent PodCut BeginPG PGMLContent PGTextContent EndPG PGVariable Comment UninterpolatedHeredocBody InterpolatedHeredocBody ScalarVariable ArrayVariable HashVariable PackageVariable PackageName :: Identifier PackageVariableName ArrayAccessVariable HashAccessVariable ArrowOperator { PackageVariable PackageName PackageVariableName TypeGlob Option StringSingleQuoted StringQQuoted q StringDoubleQuoted StringQqQuoted qq HeredocInitializer << Integer Float SpecialLiteral __FILE__ __LINE__ __PACKAGE__ Pair FatComma ) ( List Comma QWList qw QWListContent ArrayLength $# } ] [ ArrayRef HashRef ArithOp ArrayAccessVariable HashAccessVariable PatternMatchM m PatternMatch / RegexQr qr SubstitutionS s TransliterationTrOrY tr y ScalarDereference $ ArrayDereference @ HashDereference % FunctionDereference & TypeGlobDereference * Constant VariableDeclaration our state my local AnonymousFunction sub FunctionPrototype Prototype FunctionAttribute : ParenthesizedArguments FunctionPrototypeAttribute prototype Block PackageStatement package PackageName Version StatementEnd ; ExpressionStatement FunctionDefinition FunctionName SingleLineStatement ReturnExpression return if unless while until for foreach Arguments IfStatement ElsifClause elsif ElseClause else UnlessStatement WhileStatement Label Continue continue UntilStatement ForStatement1 ForStatement2 ~~ StandaloneBlock NamedBlockStatement NamedBlock SpecialBlock BEGIN UNITCHECK CHECK INIT END Ellipsis EmptyStatement PodStatement EndDataStatement __END__ __DATA__ EndDocument ENDDOCUMENT LaTeXImageCode PGMLBlock PGTextBlock LoopControlStatement next last redo Assignment = UpdateExpression UpdateOp x BinaryExpression ArithOp BindingOp ArithOp ConcatOp BitOp CompareOp lt gt le ge CompareOp eq ne CompareOp cmp BitOp BitOp LogicOp LogicOp RangeOp and or xor UnaryExpression LogicOp RefOp not UpdateExpression ArithOp NoArgumentNamedUnaryOperator time times wantarray NamedUnaryExpression ListOperatorExpression grep map join sort unpack CallExpression FunctionName CallExpressionWithBareword FunctionName Bless bless MethodInvocation SUPER FunctionName IndirectMethodInvocation FunctionName ArrayFunction FunctionName EvalExpression Eval DoExpression do GotoExpression goto TernaryExpression LogicOp ArrayModifierFunction push unshift splice ArrayPopOrShift pop shift HashOrArrayDestructureFunction each values keys Program",
  maxTerm: 392,
  context: contextTracker,
  nodeProps: [
    ["group", -58,2,24,28,29,30,40,43,44,45,46,48,49,51,53,54,55,59,63,65,68,73,74,76,77,78,80,82,84,86,89,91,93,95,97,99,100,105,172,174,177,201,205,207,211,212,218,220,222,224,227,229,231,233,235,237,239,243,246,"Expression",-21,115,121,122,124,134,139,140,144,145,146,148,149,151,157,158,159,160,163,165,166,167,"Statement"],
    ["closedBy", 20,"EndPG",39,"}",62,")",72,"]"],
    ["openedBy", 23,"BeginPG",61,"(",70,"{",71,"["],
    ["isolate", -2,25,45,""]
  ],
  propSources: [pgHighlighting],
  skippedNodes: [0],
  repeatNodeCount: 20,
  tokenData: "LV~RyXY#rYZ#w]^#wpq#rqr#|rs$cst$htu%Puv)Rvw-]wx/Yxy1Vyz1[z{1a{|6O|}7o}!O7t!O!P8x!P!Q;r!Q!R<V!R![=m![!]@]!]!^@j!^!_Aa!_!`BR!`!aBf!a!bCO!b!cCT!c!}G]!}#OGn#O#P&[#P#QHe#Q#RI[#R#SIg#T#jG]#j#kI}#k#oG]#o#pKU#p#qKZ#q#rKi#r#sKn~#wO'V~~#|O'W~~$RQ%bR!_!`$X#r#s$^~$^O%S~~$cO$y~~$hO'b~~$mSi~OY$hZ;'S$h;'S;=`$y<%lO$h~$|P;=`<%l$h~%W_#`W!|~st&Vtu&[uv&[vw&[z{&[{|&[!]!^&[!b!c&[!c!}'P!}#O&[#O#P&[#P#Q&[#R#S'P#T#o'P#o#p'b~&[O!g~W&aY#`Wtu&[uv&[vw&[z{&[{|&[!]!^&[!b!c&[!}#O&[#O#P&[#P#Q&[~'USl~!Q!['P!c!}'P#R#S'P#T#o'P~'eVXY'bYZ'b]^'bpq'b!c!}'z#R#S'z#T#o'z~'}XXY(jYZ(j]^(jpq(j!Q!['z!c!}'z#R#S'z#T#o'z#q#r(|~(mTXY(jYZ(j]^(jpq(j#q#r(|~)ROl~~)Ya#`W#Q~qr*_tu&[uv&[vw&[z{&[{|*d}!O*_!]!^&[!_!`+Z!b!c&[!c!}+`!}#O&[#O#P&[#P#Q&[#R#S+`#T#o+`#o#p+q~*dOn~~*kYn~#`Wtu&[uv&[vw&[z{&[{|&[!]!^&[!b!c&[!}#O&[#O#P&[#P#Q&[~+`O$u~~+eSn~!Q![+`!c!}+`#R#S+`#T#o+`~+tVXY+qYZ+q]^+qpq+q!c!},Z#R#S,Z#T#o,Z~,^XXY,yYZ,y]^,ypq,y!Q![,Z!c!},Z#R#S,Z#T#o,Z#q#r*_~,|TXY,yYZ,y]^,ypq,y#q#r*_~-d[#`W#S~tu&[uv&[vw.Yz{&[{|&[!O!P/S!]!^&[!_!`+Z!b!c&[!}#O&[#O#P&[#P#Q&[~.aZ%ZQ#`Wtu&[uv&[vw&[z{&[{|&[!]!^&[!_!`+Z!b!c&[!}#O&[#O#P&[#P#Q&[~/VP!_!`+Z~/_V}~Ow/Ywx/tx#r/Y#r#s/y#s;'S/Y;'S;=`1P<%lO/Y~/yO}~~0OV}~Ow/Ywx/tx#r/Y#r#s0e#s;'S/Y;'S;=`1P<%lO/Y~0jV}~Ow/Ywx/Yx#r/Y#r#s0e#s;'S/Y;'S;=`1P<%lO/Y~1SP;=`<%l/Y~1[O!`~~1aO!_~~1h_#`W#U~tu&[uv&[vw&[z{2g{|&[!]!^&[!_!`+Z!b!c&[!c!}3a!}#O&[#O#P&[#P#Q&[#R#S3a#T#o3a#o#p4[~2nZ#`W$x~tu&[uv&[vw&[z{&[{|&[!]!^&[!_!`+Z!b!c&[!}#O&[#O#P&[#P#Q&[~3fT{~!Q![3a![!]3u!c!}3a#R#S3a#T#o3a~3xP![!]3{~4OS!c!}3a#R#S3a#T#o3a#o#p4[~4_VXY4[YZ4[]^4[pq4[!c!}4t#R#S4t#T#o4t~4wXXY5dYZ5d]^5dpq5d!Q![4t!c!}4t#R#S4t#T#o4t#q#r5v~5gTXY5dYZ5d]^5dpq5d#q#r5v~5{P{~![!]3u~6VZ#`W!m~tu&[uv&[vw&[z{&[{|6x!]!^&[!_!`+Z!b!c&[!}#O&[#O#P&[#P#Q&[~7PY#`W%f~tu&[uv&[vw&[z{&[{|&[!]!^&[!b!c&[!}#O&[#O#P&[#P#Q&[~7tO!b~~7yU$z~}!O8]!_!`+Z!`!a8b!c!}8g#R#S8g#T#o8g~8bO%f~~8gOv~~8lS|~!Q![8g!c!}8g#R#S8g#T#o8g~9PS#jS${Q!O!P9]!Q![:_!_!`+Z#R#S:PV9dR%]Q#jS!O!P9m!Q![:P#R#S:PV9vR(dP%]Q#jS!O!P:P!Q![:P#R#S:PS:UR#jS!O!P:P!Q![:P#R#S:P~:fT#jS!W~!O!P:P!Q![:_!g!h:u#R#S;d#X#Y:u~:xR{|;R}!O;R!Q![;X~;UP!Q![;X~;^Q!W~!Q![;X#R#S;R~;iR#jS!O!P:P!Q![:_#R#S:P~;wQ!s~!P!Q;}!_!`+Z~<SP%[~!_!`+Z~<^[#jS!V~!O!P:_!Q!Y=S!Y![=m!d!e>t!g!h:u!q!r?]!z!{?n#R#S>T#U#V>t#X#Y:u#c#d?]#l#m?n~=ZU#jS!V~!O!P:_!Q!Y=S!Y![=m!g!h:u#R#S>c#X#Y:u~=tT#jS!V~!O!P:_!Q![=m!g!h:u#R#S>T#X#Y:u~>YR#jS!O!P:P!Q![=m#R#S:P~>hS#jS!O!P:P!Q!Y=S!Y![=m#R#S:P~>wQ!Q!R>}!R!S>}~?SR!V~!Q!R>}!R!S>}#R#S>t~?`P!Q!Y?c~?hQ!V~!Q!Y?c#R#S?]~?qR!Q![?z!c!i?z#T#Z?z~@PS!V~!Q![?z!c!i?z#R#S?n#T#Z?z~@bP#b~![!]@e~@jOq~_@qY#lV#`Wtu&[uv&[vw&[z{&[{|&[!]!^&[!b!c&[!}#O&[#O#P&[#P#Q&[~AfQ$}~!^!_Al!_!`At~AqP!U~!_!`+Z~AyP$}~!`!aA|~BRO%V~~BWR$s~!_!`$X!`!aBa#r#s$^~BfO!^~~BkQ$}~!_!`Bq!`!aBv~BvO$}~~B{P$|~!_!`+Z~CTO&W~~C[b#`W#O~qrDdtu&[uv&[vw&[z{&[{|Di|!]Dd!]!^Di!^!bDd!b!cDi!c!}E`!}#ODi#O#PDi#P#QDi#Q#RDd#R#SE`#T#oE`#o#pEq~DiOm~~DpYm~#`Wtu&[uv&[vw&[z{&[{|&[!]!^&[!b!c&[!}#O&[#O#P&[#P#Q&[~EeSm~!Q![E`!c!}E`#R#SE`#T#oE`~EtVXYEqYZEq]^EqpqEq!c!}FZ#R#SFZ#T#oFZ~F^XXYFyYZFy]^FypqFy!Q![FZ!c!}FZ#R#SFZ#T#oFZ#q#rDd~F|TXYFyYZFy]^FypqFy#q#rDd~GbSr~!Q![G]!c!}G]#R#SG]#T#oG]~GuY#`W!j~tu&[uv&[vw&[z{&[{|&[!]!^&[!b!c&[!}#O&[#O#P&[#P#Q&[ZHlY!iR#`Wtu&[uv&[vw&[z{&[{|&[!]!^&[!b!c&[!}#O&[#O#P&[#P#Q&[~IaQ%Y~!O!P/S!_!`+Z~InT#jSr~!O!P:P!Q![Ig!c!}G]#R#SIg#T#oG]~JSTr~!O!PJc!Q![Jn!c!}G]#R#SG]#T#oG]SJhQ#jS!O!PJc!Q![Jc~JuT#jSr~!O!PJc!Q![Jn!c!}G]#R#SG]#T#oG]~KZOw~~K`R%Y~!O!P/S!_!`+Z#p#q;}~KnO!h~VKsQ%bR!O!PKy#r#sLORLOO%bRVLVO$XS%cR",
  tokenizers: [semicolon, unrestrictedIdentifier, specialScalarVariable, builtinOperator, heredoc, interpolated, quoteLikeOperator, regex, pod, endData, pgText$1, 0, 1, 2, 3],
  topRules: {"Program":[0,250]},
  dynamicPrecedences: {"125":1,"160":4,"163":4,"232":1,"286":1,"292":1,"295":1,"296":1,"297":1,"298":1,"302":1,"304":1,"305":1,"306":1,"308":1,"309":1,"311":1,"312":1,"313":1,"314":1,"317":1,"318":1,"320":1,"324":1,"327":1,"328":1,"329":1,"330":1,"331":1,"332":1,"334":1,"335":1,"336":1,"337":1,"338":1,"339":1,"340":1,"341":1,"343":1,"344":1,"345":1,"346":1,"348":1,"349":1,"350":1,"351":1,"352":1,"353":1,"354":1,"355":1,"356":1,"357":1,"358":1,"359":1,"360":1,"361":1,"362":1,"363":1,"364":1,"365":1,"367":1,"368":1,"369":1,"370":1,"371":1,"372":1,"375":1,"377":1,"378":1,"379":1,"380":1,"381":1,"382":1,"384":1,"385":1,"386":1,"388":1,"389":1,"390":1,"391":1},
  specialized: [{term: 28, get: (value, stack) => (pgVariable(value) << 1), external: pgVariable},{term: 34, get: (value) => spec_Identifier[value] || -1}],
  tokenPrec: 21500
});

const pgOperatorCompletionOptions = [];
for (const operator of pgOperators.values()) {
    const completions = pgOperatorCompletions.get(operator);
    if (completions) {
        for (const template of completions) {
            pgOperatorCompletionOptions.push(snippetCompletion(template, {
                label: operator,
                info: template
                    .replaceAll(/\${([^}]*)}/g, '$1')
                    .replaceAll(/\t/g, '')
                    .replaceAll(/\n/g, ' '),
                type: 'variable',
                section: { name: 'PG Methods', rank: 1 }
            }));
        }
    }
    else {
        pgOperatorCompletionOptions.push(snippetCompletion(`${operator}(\${})\${}`, {
            label: operator,
            info: `${operator}()`,
            type: 'variable',
            section: { name: 'PG Methods', rank: 1 }
        }));
    }
}
for (const pkg of ['Scaffold', 'Section']) {
    for (const position of ['Begin', 'End']) {
        pgOperatorCompletionOptions.push(snippetCompletion(`${pkg}::${position}(\${});\${}`, {
            label: `${pkg}::${position}`,
            info: `${pkg}::${position}();`,
            type: 'variable',
            section: { name: 'PG Methods', rank: 1 }
        }));
    }
}
for (const pkg of ['Label', 'Circle', 'Fun']) {
    pgOperatorCompletionOptions.push({ label: pkg, type: 'variable', section: { name: 'WWPlot Object', rank: 2 } });
}
const pgCompletion = (isTop = false) => {
    return (context) => {
        const nodeAt = syntaxTree(context.state).resolveInner(context.pos, -1);
        // If in one of the named nodes, then this returns and array whose first element is 2 if inside a complete
        // node and 1 if inside an incomplete node, and whose second element is the node.  It returns undefined if
        // in none of the named nodes..
        const inside = (nodeNames) => {
            for (let pos = nodeAt; pos; pos = pos.parent) {
                if ((nodeNames instanceof Array && nodeNames.includes(pos.name)) || nodeNames === pos.name)
                    return [pos.lastChild && pos.lastChild.name !== '⚠' ? 2 : 1, pos];
                if (pos.type.isTop)
                    break;
            }
        };
        if (isTop) {
            const [insideMethodInvocation, methodInvocation] = inside('MethodInvocation') ?? [];
            if (insideMethodInvocation &&
                methodInvocation &&
                (methodInvocation.parent?.name === 'ExpressionStatement' || context.explicit)) {
                const arrowOperator = methodInvocation.getChild('ArrowOperator');
                if (arrowOperator && context.matchBefore(/->\w*$/)) {
                    const before = context.matchBefore(/\w*/);
                    if (arrowOperator.to === context.pos || before) {
                        return {
                            from: before?.from ?? context.pos,
                            options: ['LATEX_IMAGE', 'TIKZ'].map((t, i) => snippetCompletion(`BEGIN_${t}\n\${}\nEND_${t}`, {
                                label: `BEGIN_${t}`,
                                type: 'interface',
                                boost: 99 - i
                            }))
                        };
                    }
                }
            }
        }
        if (context.matchBefore(/\$\w*$/)) {
            return completeFromList(Array.from(pgVariables.values()).map((label) => ({ label: `$${label}`, type: 'variable' })))(context);
        }
        const completionOptions = [];
        if (isTop &&
            !inside([
                'InterpolatedHeredocBody',
                'UninterpolatedHeredocBody',
                'PodStatement',
                'EndDataStatement',
                'EndDocument'
            ]) &&
            ((context.matchBefore(/^\s*\w*/) && context.explicit) || context.matchBefore(/^\s*B\w*/))) {
            completionOptions.push(...['PGML', 'PGML_HINT', 'PGML_SOLUTION', 'TEXT', 'HINT', 'SOLUTION'].map((t, i) => snippetCompletion(`BEGIN_${t}\n\${}\nEND_${t}`, {
                label: `BEGIN_${t}`,
                type: 'type',
                boost: 99 - i
            })));
        }
        if (inside(['CallExpression', 'FunctionName', 'Identifier']) &&
            !inside([
                'StringSingleQuoted',
                'StringQQuoted',
                'InterpolatedStringContent',
                'UninterpolatedHeredocBody',
                'MethodInvocation'
            ])) {
            completionOptions.push(...pgOperatorCompletionOptions);
        }
        return completeFromList(completionOptions)(context);
    };
};
const pgPerlParser = parser.configure({
    props: [languageDataProp.add({ Program: defineLanguageFacet({ autocomplete: pgCompletion() }) })]
});

class CompositeBlock {
    type;
    from;
    hash;
    end;
    children;
    positions;
    static create(type, from, parentHash, end) {
        const hash = (parentHash + (parentHash << 8) + type) | 0;
        return new CompositeBlock(type, from, hash, end, [], []);
    }
    hashProp;
    constructor(type, from, hash, end, children, positions) {
        this.type = type;
        this.from = from;
        this.hash = hash;
        this.end = end;
        this.children = children;
        this.positions = positions;
        this.hashProp = [[NodeProp.contextHash, hash]];
    }
    addChild(child, pos) {
        if (child.prop(NodeProp.contextHash) != this.hash)
            child = new Tree(child.type, child.children, child.positions, child.length, this.hashProp);
        this.children.push(child);
        this.positions.push(pos);
    }
    toTree(nodeSet, end = this.end) {
        const last = this.children.length - 1;
        if (last >= 0)
            end = Math.max(end, this.positions[last] + this.children[last].length + this.from);
        return new Tree(nodeSet.types[this.type], this.children, this.positions, end - this.from).balance({
            makeTree: (children, positions, length) => new Tree(NodeType.none, children, positions, length, this.hashProp)
        });
    }
}
class Buffer {
    nodeSet;
    content = [];
    nodes = [];
    constructor(nodeSet) {
        this.nodeSet = nodeSet;
    }
    write(type, from, to, children = 0) {
        this.content.push(type, from, to, 4 + children * 4);
        return this;
    }
    writeElements(elts, offset = 0) {
        for (const e of elts)
            e.writeTo(this, offset);
        return this;
    }
    finish(type, length) {
        return Tree.build({
            buffer: this.content,
            nodeSet: this.nodeSet,
            reused: this.nodes,
            topID: type,
            length
        });
    }
}
// Elements are used to compose syntax nodes during parsing.
class Element {
    type;
    from;
    to;
    children = [];
    constructor(
    // The node's [id](https://lezer.codemirror.net/docs/ref/#common.NodeType.id).
    type, 
    // The start of the node, as an offset from the start of the document.
    from, 
    // The end of the node.
    to, 
    // The node's child nodes
    children = []) {
        this.type = type;
        this.from = from;
        this.to = to;
        this.children.push(...children);
    }
    writeTo(buf, offset) {
        const startOff = buf.content.length;
        buf.writeElements(this.children, offset);
        buf.content.push(this.type, this.from + offset, this.to + offset, buf.content.length + 4 - startOff);
    }
    toTree(nodeSet) {
        return new Buffer(nodeSet).writeElements(this.children, -this.from).finish(this.type, this.to - this.from);
    }
}
class TreeElement {
    tree;
    from;
    constructor(tree, from) {
        this.tree = tree;
        this.from = from;
    }
    get to() {
        return this.from + this.tree.length;
    }
    get type() {
        return this.tree.type.id;
    }
    get children() {
        return [];
    }
    writeTo(buf, offset) {
        buf.nodes.push(this.tree);
        buf.content.push(buf.nodes.length - 1, this.from + offset, this.to + offset, -1);
    }
    toTree() {
        return this.tree;
    }
}
const elt = (type, from, to, children) => new Element(type, from, to, children);

const splitPatternParts = [
    '^(?:\\t| {4})+', // indent
    '\\[(?:[!<%@$#.]|::?:?|``?`?|\\|+ ?)', // open
    '\\[(?:_+|[ox^])\\]\\*?', // ansrule
    '(?:[!>%@$#.]|::?:?|``?`?| ?\\|+)\\]', // close
    ' {2,3}(?=\\n)', // linebreak
    '\\n+', // lineend
    '#+', // heading
    '(?:---+|===+)', // rule
    '(?:^|(?<=[\\t ]))(?:[-+o*]|(?:\\d+|[ivxl]+|[IVXL]+|[a-zA-Z])[.)]) +', // list
    '>> *| *<<', // align
    '```', // code
    ': {3}', // pre
    '[$@%]q[qr]?|\\bq[qr]?\\s+(?:#.*?(?:\\n\\s*)+)?(?!=>)(?=.)|\\bq[qr]?(?!=>)(?=\\W)', // quoted
    '\\*+|_+', // emphasis
    '\\[\\]', // noop
    '\\\\.|[{}[\\]()\'"]' // chars
];
const splitPattern = RegExp('(' + splitPatternParts.join('|') + ')', 'm');
const balanceAll = RegExp('[{\\[\'"]');
class PGMLParse {
    indent = 0;
    actualIndent = 0;
    atLineStart = 1;
    atBlockStart = 1;
    ignoreNL;
    input = '';
    pos = 0;
    block;
    root;
    split = [];
    i = 0;
    constructor(string) {
        this.Parse(this.Split(string));
    }
    Split(input) {
        this.input = input;
        const split = input.split(splitPattern);
        // Need to remove the last entry from split array when a split occurs at the end of the input.
        // Javascript adds that, but Perl doesn't (unless you add the optional limit -1 argument to split).
        if (split.slice(-1)[0] === '')
            split.pop();
        return split;
    }
    isLineEnd(block) {
        block = block?.prev;
        let i = this.i;
        while (i < this.split.length) {
            if (this.split[i++] !== '')
                return false;
            const token = this.split[i++] ?? '';
            if (/^\n+$/.test(token))
                break;
            if (/^ {2,3}$/.test(token))
                continue;
            if (!/^ +<<$/.test(token) || !block?.align)
                return false;
            block = block.prev;
        }
        return true;
    }
    nextChar(defaultNext) {
        return (this.split[this.i] ??
            this.split[this.i + 1] ??
            (typeof defaultNext === 'undefined' ? '' : defaultNext)).substring(0, 1);
    }
    prevChar(defaultPrev) {
        return (this.split[this.i - 2] ??
            this.split[this.i - 3] ??
            (typeof defaultPrev === 'undefined' ? '' : defaultPrev)).slice(-1);
    }
    Parse(split) {
        this.split = split;
        let block;
        this.i = 0;
        this.pos = 0;
        this.block = this.root = new Root(this);
        while (this.i < this.split.length) {
            block = this.block;
            this.pos += this.split[this.i]?.length ?? 0;
            this.Text(this.split[this.i++] ?? '');
            const token = this.split[this.i++];
            if (typeof token === 'undefined' || token === '')
                continue;
            if (block.terminator &&
                RegExp(`^${block.terminator instanceof RegExp ? block.terminator.source : block.terminator}$`).test(token))
                this.Terminate(token);
            else if (!block.containerEnd &&
                block.type !== 'balance' &&
                block.prev?.terminator &&
                RegExp(`^${block.prev.terminator instanceof RegExp ? block.prev.terminator.source : block.prev.terminator}$`).test(token)) {
                this.Terminate();
                this.Terminate(token);
            }
            else if (block.containerEnd &&
                RegExp(`^${block.containerEnd instanceof RegExp ? block.containerEnd.source : block.containerEnd}$`).test(token))
                this.EndContainer(token);
            else if (/^\[[@$]/.test(token) && (block.parseAll || block.parseSubstitutions))
                this.Begin(token);
            else if (token.startsWith('[%') && (block.parseAll || block.parseComments))
                this.Begin(token);
            else if (/^\\./.test(token) && (block.parseAll || block.parseSlashes))
                this.Slash(token);
            else if (/^\n$/.test(token))
                this.Break(token);
            else if (/^\n\n+$/.test(token))
                this.Par(token);
            else if (/^\*\*?$/.test(token) && !block.parseAll && block.parseSubstitutions)
                this.Star(token);
            else if (block.parseQuoted && token.startsWith('q'))
                this.Quoted(token);
            else if (block.balance && RegExp(`^${block.balance.source}`).test(token))
                this.Begin(token, token.substring(0, 1));
            else if (block.balance && RegExp(`${block.balance.source}$`).test(token))
                this.Begin(token, token.substring(token.length - 1));
            else if (block.parseAll)
                this.All(token);
            else if (/^[}\]]$/.test(token))
                this.Unbalanced(token);
            else
                this.Text(token);
            this.pos += this.split[this.i - 1]?.length ?? 0;
        }
        this.End('END_PGML');
        this.root.to = this.pos;
        delete this.root.parser;
    }
    All(token) {
        if (token.startsWith('[') && BlockDefs[token])
            this.Begin(token);
        else if (/^(?:\t| {4})/.test(token))
            this.Indent(token);
        else if (/\d+[.)] /.test(token))
            this.Bullet(token, 'numeric');
        else if (/[ivxl]+[.)] /.test(token))
            this.Bullet(token, 'roman');
        else if (/[a-z][.)] /.test(token))
            this.Bullet(token, 'alpha');
        else if (/[IVXL]+[.)] /.test(token))
            this.Bullet(token, 'Roman');
        else if (/[A-Z][.)] /.test(token))
            this.Bullet(token, 'Alpha');
        else if (/[-+o*] /.test(token))
            this.Bullet(token, 'bullet');
        else if (token.includes('{'))
            this.Brace(token);
        else if (token.includes('[]'))
            this.NOOP(token);
        else if (token.includes('[|'))
            this.Verbatim(token);
        else if (token.includes('[!'))
            this.Image(token);
        else if (/\[./.test(token))
            this.Answer(token);
        else if (token.includes('_'))
            this.Emphasis(token);
        else if (token.includes('*'))
            this.Star(token);
        else if (/["']/.test(token))
            this.Quote(token);
        else if (/^ {2,3}$/.test(token))
            this.ForceBreak(token);
        else if (token.includes('#'))
            this.Heading(token);
        else if (/-|=/.test(token))
            this.Rule(token);
        else if (token.includes('<<'))
            this.Center(token);
        else if (token.includes('>>'))
            this.Align(token);
        else if (token.includes('```'))
            this.Code(token);
        else if (token.includes(':   '))
            this.Preformatted(token);
        else
            this.Text(token);
    }
    Begin(token, id = token, options = {}) {
        const def = { ...BlockDefs[id], ...options, token };
        const type = BlockDefs[id]?.type ?? 'text';
        const blockClass = blockClasses.get(def.class ?? 'Block') ?? Block;
        delete def.type;
        delete def.class;
        const block = new blockClass(type, this.pos, def);
        const end = this.block?.isContainer ? this.block.terminator : this.block?.containerEnd;
        if (end)
            block.containerEnd = end;
        if (block.container && this.block?.type !== block.container)
            block.hasWarning = true;
        this.block?.pushItem(block);
        block.prev = this.block;
        this.block = block;
        this.atLineStart = 0;
        this.atBlockStart = 1;
    }
    End(action, endAt) {
        let block = this.block;
        if (block?.isContainer)
            return;
        const topItem = block?.topItem();
        if (topItem instanceof Block && topItem.type === 'break' && block?.type !== 'align')
            block?.popItem();
        while (block?.type !== 'root') {
            this.Terminate();
            if (endAt && endAt === block)
                return;
            block = this.block;
        }
    }
    Terminate(token) {
        const block = this.block;
        if (!block)
            return;
        const prev = block.prev;
        if (typeof token === 'string') {
            block.terminator = token;
            block.to += block.terminator.length;
            if (block.terminateMethod)
                this[block.terminateMethod](token);
        }
        if (prev && ((block.type !== 'pre' && block.prev?.type !== 'align') || prev.to < block.to))
            prev.to = block.to;
        delete block.prev;
        delete block.parseComments;
        delete block.parseSubstitutions;
        delete block.parseSlashes;
        delete block.parseAll;
        delete block.cancelUnbalanced;
        delete block.cancelNL;
        delete block.cancelPar;
        delete block.balance;
        delete block.terminateMethod;
        delete block.noIndent;
        delete block.ignoreIndent;
        this.block = prev;
        if (block.stack && block.combine)
            prev?.combineTopItems();
    }
    EndContainer(token) {
        while (!this.block?.isContainer)
            this.Terminate();
        this.Terminate(token);
    }
    Unbalanced(token) {
        this.Text(token);
    }
    Text(text, force) {
        if (text !== '' || force) {
            this.block?.pushText(text, force);
            this.atLineStart = this.atBlockStart = this.ignoreNL = 0;
        }
    }
    Item(type, token, options = {}) {
        const def = { ...options, token };
        const top = this.block?.topItem();
        this.block?.pushItem(new Item(type, top instanceof Item ? top.to : this.block.to, def));
        this.atBlockStart = 0;
    }
    Break(token) {
        if (this.ignoreNL) {
            if (this.block)
                this.block.to += 1;
            this.ignoreNL = 0;
        }
        else {
            this.Text(token);
            this.ignoreNL = 1;
        }
        this.atLineStart = 1;
        this.actualIndent = 0;
    }
    ForceBreak(token) {
        if (token === '   ') {
            this.End('forced break');
            this.Item('forced', token, { noIndent: 1 });
            this.indent = 0;
        }
        else {
            this.Item('break', token, { noIndent: 1 });
        }
        this.atLineStart = this.ignoreNL = 1;
        this.actualIndent = 0;
    }
    Par(token, endAt) {
        if (this.block?.allowPar) {
            this.Text('\n\n');
        }
        else {
            this.End(undefined, endAt);
            this.Item('par', token, { noIndent: 1 });
            this.atLineStart = this.ignoreNL = 1;
            this.indent = this.actualIndent = 0;
        }
    }
    Indent(token) {
        if (this.block && (this.block.ignoreIndent || this.atLineStart))
            this.block.to += token.length;
        if (this.block?.ignoreIndent)
            return;
        if (this.atLineStart) {
            const tabs = token.replace(/ {4}/g, '\t'); // turn spaces into tabs
            const indent = (this.actualIndent = tabs.length);
            const top = this.block?.topItem();
            if (top && top instanceof Text)
                top.to += token.length;
            if (indent !== this.indent) {
                this.End('indentation change');
                this.indent = indent;
            }
        }
        else {
            this.Text(token);
        }
    }
    Slash(token) {
        this.Item('slash', token);
    }
    Brace(token) {
        const top = this.block?.topItem();
        if (top instanceof Item && top.options)
            this.Begin(token, ' {');
        else
            this.Text(token);
    }
    Verbatim(token) {
        let bars = token;
        bars = bars.replace(/[^|]/g, '');
        bars = '\\' + bars.split('').join('\\');
        this.Begin(token, ' [|', { terminator: RegExp(` ?${bars}\\]`) });
    }
    Image(token) {
        this.Item('image', token);
    }
    Answer(token) {
        const def = { options: ['answer', 'width', 'name', 'cmp_options'] };
        if (token.endsWith('*'))
            def.hasStar = 1;
        this.Item('answer', token, def);
    }
    Emphasis(token) {
        const type = BlockDefs[token.substring(0, 1)]?.type;
        let block = this.block;
        if (block && block.type === type) {
            block.to += token.length;
            this.Terminate();
            return;
        }
        while (block?.type !== 'root') {
            if (block && block.prev?.type === type) {
                this.End(`end of ${type ?? ''}`, block);
                block.to += token.length;
                this.Terminate();
                return;
            }
            block = block?.prev;
        }
        if (!/\s/.test(this.nextChar(' ')) && (type === 'bold' || !/[a-z0-9]/i.test(this.prevChar(' ')))) {
            this.Begin(token, token.substring(0, 1));
        }
        else {
            this.Text(token);
        }
    }
    Star(token) {
        if (this.StarOption(token))
            return;
        if (this.block?.parseAll)
            this.Emphasis(token);
        else
            this.Text(token);
    }
    Rule(token) {
        if (this.atLineStart) {
            // check for line end or braces
            this.Item('rule', token, { options: ['width', 'height', 'size'] });
            this.ignoreNL = 1;
        }
        else {
            this.Text(token);
        }
    }
    Bullet(token, bullet) {
        if (!this.atLineStart) {
            this.Text(token);
            return;
        }
        if (bullet === 'bullet')
            bullet = { '*': 'disc', '+': 'square', o: 'circle', '-': 'bullet' }[token.substring(0, 1)] ?? 'bullet';
        let block = this.block;
        if (block?.type !== 'root' && !block?.align) {
            while (block?.type !== 'root' && !block?.prev?.align)
                block = block?.prev;
            this.End('start of list item', block);
        }
        this.indent = this.actualIndent;
        this.Begin('', 'list', { bullet });
        this.Begin(token, 'bullet');
    }
    Heading(token) {
        const n = token.length;
        if (n > 6) {
            this.Text(token);
            return;
        }
        let block = this.block;
        if (this.atLineStart) {
            if (block?.type !== 'root' && block?.type !== 'align') {
                while (block && block.type !== 'root' && block.prev?.type !== 'align')
                    block = block.prev;
                this.End('start of heading', block);
            }
            this.Begin(token, '#', { n });
        }
        else {
            while (block && (block.type !== 'heading' || block.n != n)) {
                if (block.type === 'root') {
                    this.Text(token);
                    return;
                }
                block = block.prev;
            }
            if (this.isLineEnd(block)) {
                if (block)
                    block.to += token.length;
                this.End('end of heading', block);
                if (block)
                    block.terminator = token;
                this.indent = 0;
            }
            else {
                this.Text(token);
            }
        }
    }
    Center(token) {
        let block = this.block;
        while (block && (!block.align || block.align !== 'right')) {
            if (block.type === 'root') {
                this.Text(token);
                return;
            }
            if (block.prev)
                block.prev.to = block.to;
            block = block.prev;
        }
        if (this.isLineEnd(block)) {
            if (block) {
                block.align = 'center';
                block.terminator = token;
                block.to += token.length;
            }
            this.End('end of centering', block);
        }
        else {
            this.Text(token);
        }
    }
    Align(token) {
        if (!this.atLineStart || (this.block?.type === 'align' && this.atBlockStart)) {
            this.Text(token);
            return;
        }
        this.End('start of aligned text');
        this.indent = this.actualIndent;
        this.Begin(token, '>>');
        this.atLineStart = this.ignoreNL = 1;
    }
    Code(token) {
        if (!this.atLineStart || (this.block?.type === 'code' && this.atBlockStart)) {
            this.Text(token);
            return;
        }
        this.End('start of preformatted code');
        this.indent = this.actualIndent;
        this.Begin(token, '```');
    }
    Preformatted(token) {
        if (!this.atLineStart || (this.block?.type === 'pre' && this.atBlockStart)) {
            this.Text(token);
            return;
        }
        this.End('start of preformatted text');
        this.indent = this.actualIndent;
        this.Begin(token, ':   ');
    }
    Quoted(token) {
        if (!this.split[this.i])
            this.pos += this.split[this.i + 1]?.length ?? 0;
        // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
        const next = (this.split[this.i] ? this.split[this.i] : this.split[++this.i]) ?? '';
        const quote = next.substring(0, 1);
        this.pos += 1;
        this.split[this.i] = next.substring(1);
        let pcount = 0;
        const open = /[({[<]/.test(quote) ? quote : '';
        let close = open || quote;
        close = close.replace(/[({[<]/, (match) => match === '(' ? ')' : match === '{' ? '}' : match === '[' ? ']' : match === '<' ? '>' : '');
        const qclose = `\\${close}`;
        this.Text(token + quote);
        while (this.i < this.split.length) {
            const text = this.split[this.i] ?? '';
            if (open && text === open) {
                ++pcount;
            }
            else if (open && text === close && pcount > 0) {
                --pcount;
            }
            else if (pcount == 0 && text !== qclose) {
                const i = text.indexOf(close);
                if (i > -1) {
                    this.Text(text.substring(0, i + 1));
                    this.pos += i + 1;
                    this.split[this.i] = text.substring(i + 1);
                    if (this.i % 2) {
                        this.pos += this.split[this.i]?.length ?? 0;
                        this.Text(this.split[this.i++] ?? '');
                    }
                    return;
                }
            }
            this.Text(text);
            this.pos += this.split[this.i]?.length ?? 0;
            ++this.i;
        }
    }
    Quote(token) {
        this.Item('quote', token);
    }
    NOOP(_token) {
        this.Text('', true);
    }
    terminateGetString(_token) {
        const block = this.block;
        if (!block)
            return;
        if (block.type === 'math' || block.type === 'image') {
            for (const child of block.stack ?? []) {
                if (child instanceof Text) {
                    const lastChild = block.children?.at(-1);
                    if (lastChild instanceof Text)
                        lastChild.pushItem(...child.stack);
                    else
                        block.pushChild(child);
                }
                else if (!(child instanceof Item)) {
                    const lastChild = block.children?.at(-1);
                    if (lastChild instanceof Text)
                        lastChild.pushText(child);
                    else
                        block.pushChild(new Text(lastChild?.to ?? block.to, child));
                }
                else {
                    block.pushChild(child);
                }
            }
            delete block.stack;
            for (const child of block.children ?? []) {
                if (child instanceof Text) {
                    child.stack = [child.stack?.join('') ?? ''];
                }
            }
        }
        else {
            const firstItem = block.stack?.at(0);
            block.textFrom = firstItem instanceof Item ? firstItem.from : block.to;
            block.text = this.stackString();
            delete block.stack;
        }
    }
    terminateBalance(_token) {
        const block = this.block;
        if (!block?.stack || block.stack.length == 0)
            this.Text('', true);
        const stackString = this.stackString();
        this.block = block?.prev;
        this.block?.popItem();
        if (block?.token === '"' || block?.token === "'") {
            this.Item('quote', block.token);
            this.Text(stackString);
            this.Item('quote', block.terminator);
        }
        else {
            this.Text((block?.token ?? '') + stackString + block?.terminator);
        }
    }
    terminateCode(_token) {
        if (!this.block)
            return;
        const top = this.block.topItem();
        if (top instanceof Text && top.type === 'text') {
            const topText = top.stack?.at(0);
            const pentUltimateText = top.stack?.at(1);
            if (typeof topText === 'string' &&
                /^ *[a-z0-9]+$/.test(topText) &&
                typeof pentUltimateText === 'string' &&
                /^\n+$/.test(pentUltimateText)) {
                this.block.class = topText.replace(/^ +/, '');
                top.stack?.shift();
                top.from += topText.length;
            }
        }
    }
    terminatePre(token) {
        if (this.block)
            this.block.terminator = ''; // the ending token is added to the text below
        if (this.block)
            this.block.to -= token.length;
        if (token.includes('\n\n')) {
            this.Par(token, this.block);
            this.block = this.block?.prev;
        }
        else {
            this.Text(token);
            this.atLineStart = 1;
            this.actualIndent = 0;
        }
    }
    terminateOptions(_token) {
        this.block = this.block?.prev;
        const optionBlock = this.block?.popItem();
        const block = this.block?.topItem();
        if (!(block instanceof Item) || !(optionBlock instanceof Item))
            return;
        const firstItem = optionBlock.stack?.at(0);
        optionBlock.textFrom = firstItem instanceof Item ? firstItem.from : optionBlock.from;
        optionBlock.text = this.stackText(optionBlock.stack);
        delete optionBlock.stack;
        block.pushOption(optionBlock);
        block.to = optionBlock.to;
    }
    StarOption(token) {
        const top = this.block?.topItem();
        if (!top || !(top instanceof Item))
            return false;
        if (token === '*' && top.allowStar) {
            top.hasStar = 1;
            top.to += 1;
            if (this.block)
                this.block.to = top.to;
            return true;
        }
        if (token === '**' && top.allowDblStar) {
            top.hasDblStar = true;
            top.to += 2;
            if (this.block)
                this.block.to = top.to;
            return true;
        }
        if (token === '***' && top.allowTriStar) {
            top.hasStar = 3;
            top.to += 3;
            if (this.block)
                this.block.to = top.to;
            return true;
        }
        return false;
    }
    stackText(stack) {
        if (!stack)
            return '';
        const strings = [];
        for (const item of stack) {
            if (!(item instanceof Item))
                continue;
            if (item.type === 'text')
                strings.push(this.replaceText(item));
            else if (item.type === 'quote')
                strings.push(this.replaceQuote(item));
            else if (item.type === 'variable')
                strings.push(this.replaceVariable(item));
            else if (item.type === 'command')
                strings.push(this.replaceCommand(item));
            else if (item.type === 'balance')
                strings.push(this.replaceBalance(item));
        }
        return strings.join('');
    }
    stackString() {
        return this.stackText(this.block?.stack);
    }
    replaceBalance(item) {
        return `${item.token ?? ''}${this.stackText(item.stack)}${item.terminator instanceof RegExp ? item.terminator.source : (item.terminator ?? '')}`;
    }
    replaceText(item) {
        return item.stack?.join('') ?? '';
    }
    replaceQuote(item) {
        return item.token ?? '';
    }
    replaceVariable(item) {
        return `$${item.text ?? ''}`;
    }
    replaceCommand(item) {
        return item.text ?? '';
    }
}
class Item {
    type;
    from;
    to;
    token;
    stack;
    children;
    optionStack;
    text;
    textFrom;
    hasWarning;
    bullet;
    hasDblStar;
    hasStar;
    indent;
    n;
    align;
    allowDblStar;
    allowPar;
    allowStar;
    allowTriStar;
    balance;
    cancelNL;
    cancelPar;
    cancelUnbalanced;
    class;
    combine;
    container;
    display;
    displaystyle;
    ignoreIndent;
    isContainer;
    noIndent;
    options;
    parseAll;
    parseComments;
    parseQuoted;
    parseSlashes;
    parseSubstitutions;
    parsed;
    terminateMethod;
    terminator;
    constructor(type, from, fields = {}) {
        this.type = type;
        this.from = from;
        Object.assign(this, fields);
        this.to = this.from + (this.token?.length ?? 0);
    }
    pushChild(child) {
        if (!(this.children instanceof Array))
            this.children = [];
        this.children.push(child);
    }
    pushOption(child) {
        if (!(this.optionStack instanceof Array))
            this.optionStack = [];
        this.optionStack.push(child);
    }
    quote(input) {
        input = input.replace(/\n/g, '\\n').replace(/\t/g, '\\t');
        return input;
    }
    toString() {
        return '';
    }
    stringifyObject(obj) {
        return ('{ ' +
            Object.entries(obj)
                .sort((a, b) => (a[0] < b[0] ? -1 : a[0] === b[0] ? 0 : 1))
                .map(([k, v]) => `${k}: ${typeof v === 'object'
                ? this.stringifyObject(v)
                : // eslint-disable-next-line @typescript-eslint/no-base-to-string
                    `'${this.quote(v.toString())}'`}`)
                .join(', ') +
            ' }');
    }
    show(indent) {
        indent ??= '';
        const strings = [];
        for (const id of Object.keys(this).sort()) {
            if (id === 'stack' || id === 'prev' || !Object.hasOwn(this, id))
                continue;
            const value = this[id];
            if (typeof value === 'undefined')
                continue;
            if (value instanceof Array && (id === 'children' || id === 'optionStack')) {
                strings.push(`${indent}${id}: [\n${indent}  {\n${value
                    .map((e) => e.show(`${indent}    `))
                    .join(`\n${indent}  },\n${indent}  {\n`)}\n${indent}  }\n${indent}]`);
            }
            else if (value instanceof Array) {
                strings.push(`${indent}${id}: [${value.map((e) => `'${this.quote(e.toString())}'`).join(', ')}]`);
            }
            else if (typeof value === 'object' && !(value instanceof RegExp)) {
                strings.push(`${indent}${id}: ${this.stringifyObject(value)}`);
            }
            else {
                strings.push(`${indent}${id}: '${this.quote(value.toString())}'`);
            }
        }
        return strings.join('\n');
    }
}
class Block extends Item {
    prev;
    containerEnd;
    constructor(type, from, fields = {}) {
        super(type, from, { ...fields, stack: [] });
        if ('prev' in fields)
            this.prev = fields.prev;
    }
    pushText(text, force) {
        if (text === '' && !force)
            return;
        const top = this.topItem();
        if (top instanceof Text) {
            top.pushText(text);
            this.to += text.length;
        }
        else {
            this.pushItem(new Text(this.to, text));
        }
    }
    pushItem(...items) {
        this.to = items.reduce((max, i) => {
            if (i instanceof Item && i.type === 'balance')
                return max;
            const to = i instanceof Item ? i.to : i.length;
            return to > max ? to : max;
        }, this.to);
        this.stack?.push(...items);
    }
    topItem(i = -1) {
        return this.stack?.at(i) ?? new Block('null', this.to);
    }
    popItem() {
        return this.stack?.pop();
    }
    // FIXME: There are a lot of hacks in this method.
    combineTopItems(i) {
        if (typeof i === 'undefined')
            i = -1;
        const top = this.topItem(i);
        let prev = this.topItem(i - 1);
        let par;
        for (let j = i - 1; prev.type === 'par' && top.combine?.par; --j) {
            par = prev;
            prev = this.topItem(j);
        }
        // Don't combine indents at the top level so that they can be easily separated into paragraphs.
        if (this instanceof Root && top.type === 'indent' && par && prev.type === 'indent')
            return;
        let id = top.combine?.[prev.type];
        let value;
        let inside = false;
        if (id) {
            if (typeof id !== 'string' && typeof id !== 'boolean') {
                [id, value] = Object.entries(id)[0];
                inside = true;
            }
            else {
                value = prev[id];
            }
            const topList = top instanceof Block ? (top.topItem().token?.substring(0, 2) ?? '') : '';
            const prevList = prev instanceof Block ? (prev.topItem().token?.substring(0, 2) ?? '') : '';
            if (top[id] === value ||
                (top.type === 'list' &&
                    top.bullet === 'roman' &&
                    prev.type === 'list' &&
                    prev.bullet === 'alpha' &&
                    ((topList === 'i.' && prevList === 'h.') ||
                        (topList === 'v.' && prevList === 'u.') ||
                        (topList === 'x.' && prevList === 'w.') ||
                        (topList === 'l.' && prevList === 'k.'))) ||
                (top.type === 'list' &&
                    top.bullet === 'Roman' &&
                    prev.type === 'list' &&
                    prev.bullet === 'Alpha' &&
                    ((topList === 'I.' && prevList === 'H.') ||
                        (topList === 'V.' && prevList === 'U.') ||
                        (topList === 'X.' && prevList === 'W.') ||
                        (topList === 'L.' && prevList === 'K.')))) {
                // Combine identical blocks
                if (inside)
                    prev = prev.topItem();
                this.stack?.splice(i, 1);
                if (par) {
                    this.stack?.splice(i, 1);
                    prev.pushItem(par);
                }
                i = -(top.stack?.length ?? 0);
                prev.pushItem(...(top.stack ?? []));
                if (prev instanceof Block && prev.topItem(i).combine)
                    prev.combineTopItems(i);
                return;
            }
            else if (top.type === 'indent' &&
                prev.type === 'indent' &&
                (top.indent ?? 0) > (prev.indent ?? 0) &&
                (prev.indent ?? 0) > 0) {
                // Move larger indentations into smaller ones
                this.stack?.splice(i, 1);
                if (par) {
                    this.stack?.splice(i, 1);
                    prev.pushItem(par);
                }
                if (typeof top.indent === 'undefined')
                    top.indent = 0;
                top.indent -= prev.indent ?? 0;
                prev.pushItem(top);
                prev.combineTopItems();
                return;
            }
            else if (id === 'indent' && top.type === 'indent' && prev.type === 'list') {
                prev = prev.topItem();
                if ((top.indent ?? 0) > value && value > 0) {
                    this.stack?.splice(i, 1);
                    if (par) {
                        this.stack?.splice(i, 1);
                        prev.pushItem(par);
                    }
                    if (typeof top.indent === 'undefined')
                        top.indent = 0;
                    top.indent -= value;
                    prev.pushItem(top);
                    prev.combineTopItems();
                }
            }
        }
    }
    show(indent) {
        indent ??= '';
        const strings = [super.show(indent)];
        if (this.stack) {
            strings.push(`${indent}stack: [`);
            for (let i = 0; i < this.stack.length; ++i) {
                const item = this.stack[i];
                if (item instanceof Item) {
                    strings.push(`${indent}  [ # ${i.toString()}`);
                    strings.push(item.show(`${indent}    `));
                    strings.push(`${indent}  ]`);
                }
                else {
                    strings.push(`${indent}  ${i.toString()}: '${item}'`);
                }
            }
            strings.push(`${indent}]`);
        }
        return strings.join('\n');
    }
}
class Root extends Block {
    parser;
    constructor(parser) {
        super('root', 0, { parseAll: true, parser });
        this.parser = parser;
    }
    pushItem(...items) {
        const parser = this.parser;
        if (!parser)
            return;
        let item;
        while ((item = items.pop())) {
            if (!item.noIndent || (parser.indent && item.noIndent < 0)) {
                parser.block = new Block('indent', item.from, {
                    prev: this,
                    indent: parser.indent,
                    parseAll: true,
                    combine: { indent: 'indent', list: { indent: 1 }, par: true }
                });
                parser.block.pushItem(item, ...items);
                items.length = 0;
                item = parser.block;
            }
            this.to = item.to;
            this.stack?.push(item);
        }
    }
}
class Table extends Block {
    pushItem(...items) {
        let item;
        while ((item = items.pop())) {
            if (item.type === 'text') {
                const text = item.stack?.join('') ?? '';
                this.to += text.length;
            }
            else if (item.type === 'table-cell' || item.type === 'options' || item.type === 'comment') {
                super.pushItem(item);
            }
        }
    }
}
class Text extends Item {
    constructor(from, ...texts) {
        super('text', from, { stack: [...texts], combine: { text: 'type' } });
        this.to += texts.join('').length;
    }
    pushText(...texts) {
        this.to += texts.join('').length;
        for (const text of texts) {
            if (text !== '')
                this.stack?.push(text);
        }
    }
    pushItem(...texts) {
        this.pushText(...texts);
    }
    show(indent) {
        indent ??= '';
        const strings = [super.show(indent)];
        if (this.stack)
            strings.push(`${indent}stack: [${this.stack
                .map((e) => (typeof e === 'string' ? `'${this.quote(e)}'` : 'bad item in Text stack'))
                .join(', ')}]`);
        return strings.join('\n');
    }
}
const blockClasses = new Map([
    ['Block', Block],
    ['Table', Table]
]);
const BlockDefs = {
    '[#': {
        type: 'table',
        class: 'Table',
        parseAll: true,
        ignoreIndent: true,
        allowPar: true,
        terminator: RegExp('#\\]'),
        allowStar: true,
        options: [
            'center',
            'caption',
            'horizontalrules',
            'texalignment',
            'align',
            'Xratio',
            'encase',
            'rowheaders',
            'headerrules',
            'valign',
            'padding',
            'tablecss',
            'captioncss',
            'columnscss',
            'datacss',
            'headercss',
            'allcellcss',
            'booktabs'
        ]
    },
    '[.': {
        type: 'table-cell',
        parseAll: true,
        isContainer: true,
        container: 'table',
        terminator: RegExp('\\.\\]'),
        allowStar: true,
        options: [
            'halign',
            'header',
            'color',
            'bgcolor',
            'b',
            'i',
            'm',
            'noencase',
            'colspan',
            'top',
            'bottom',
            'cellcss',
            'texpre',
            'texpost',
            'texencase',
            'rowcolor',
            'rowcss',
            'headerrow',
            'rowtop',
            'rowbottom',
            'valign',
            'rows'
        ]
    },
    '[:': {
        type: 'math',
        parseComments: true,
        parseSubstitutions: true,
        terminator: RegExp(':\\]'),
        terminateMethod: 'terminateGetString',
        parsed: true,
        allowStar: true,
        allowDblStar: true,
        allowTriStar: true,
        options: ['context', 'reduced']
    },
    '[::': {
        type: 'math',
        parseComments: true,
        parseSubstitutions: true,
        terminator: RegExp('::\\]'),
        terminateMethod: 'terminateGetString',
        parsed: true,
        allowStar: true,
        allowDblStar: true,
        allowTriStar: true,
        displaystyle: true,
        options: ['context', 'reduced']
    },
    '[:::': {
        type: 'math',
        parseComments: true,
        parseSubstitutions: true,
        terminator: RegExp(':::\\]'),
        terminateMethod: 'terminateGetString',
        parsed: true,
        allowStar: true,
        allowDblStar: true,
        allowTriStar: true,
        display: true,
        options: ['context', 'reduced']
    },
    '[`': {
        type: 'math',
        parseComments: true,
        parseSubstitutions: true,
        terminator: RegExp('\\`\\]'),
        terminateMethod: 'terminateGetString'
    },
    '[``': {
        type: 'math',
        parseComments: true,
        parseSubstitutions: true,
        terminator: RegExp('\\`\\`\\]'),
        terminateMethod: 'terminateGetString',
        displaystyle: true
    },
    '[```': {
        type: 'math',
        parseComments: true,
        parseSubstitutions: true,
        terminator: RegExp('\\`\\`\\`\\]'),
        terminateMethod: 'terminateGetString',
        display: true
    },
    '[!': {
        type: 'image',
        parseComments: true,
        parseSubstitutions: true,
        terminator: RegExp('!]'),
        terminateMethod: 'terminateGetString',
        cancelNL: true,
        options: ['source', 'width', 'height', 'image_options']
    },
    '[<': {
        type: 'tag',
        parseAll: true,
        isContainer: true,
        terminator: RegExp('>\\]'),
        options: ['html', 'tex', 'ptx']
    },
    '[%': { type: 'comment', parseComments: true, terminator: RegExp('%\\]'), allowPar: true },
    '[@': {
        type: 'command',
        parseComments: true,
        parseSubstitutions: true,
        parseQuoted: true,
        terminator: RegExp('@\\]'),
        terminateMethod: 'terminateGetString',
        balance: RegExp('[{\'"]'),
        allowStar: true,
        allowDblStar: true,
        allowTriStar: true
    },
    '[$': {
        type: 'variable',
        parseComments: true,
        parseSubstitutions: false,
        terminator: RegExp('\\$?\\]'),
        terminateMethod: 'terminateGetString',
        balance: balanceAll,
        cancelUnbalanced: true,
        cancelNL: true,
        allowStar: true,
        allowDblStar: true,
        allowTriStar: true
    },
    ' [|': { type: 'verbatim', cancelNL: true, allowStar: true, terminateMethod: 'terminateGetString' },
    ' {': {
        type: 'options',
        parseComments: true,
        parseSubstitutions: false,
        terminator: RegExp('\\}'),
        balance: balanceAll,
        cancelUnbalanced: true,
        terminateMethod: 'terminateOptions'
    },
    '{': {
        type: 'balance',
        parseComments: true,
        parseSubstitutions: true,
        terminator: RegExp('\\}'),
        balance: balanceAll,
        cancelUnbalanced: true
    },
    '[': {
        type: 'balance',
        parseComments: true,
        parseSubstitutions: true,
        terminator: RegExp('\\]'),
        balance: balanceAll,
        cancelUnbalanced: true
    },
    "'": { type: 'balance', terminator: RegExp("'"), terminateMethod: 'terminateBalance' },
    '"': { type: 'balance', terminator: RegExp('\\"'), terminateMethod: 'terminateBalance' },
    '```': { type: 'code', terminator: RegExp('```'), terminateMethod: 'terminateCode', allowPar: true },
    ':   ': {
        type: 'pre',
        parseAll: true,
        terminator: RegExp('\\n+'),
        terminateMethod: 'terminatePre',
        noIndent: -1
    },
    '>>': {
        type: 'align',
        parseAll: true,
        align: 'right',
        noIndent: -1
    },
    '#': { type: 'heading', parseAll: true },
    '*': { type: 'bold', parseAll: true, cancelPar: true },
    _: { type: 'italic', parseAll: true, cancelPar: true },
    bullet: { type: 'bullet', parseAll: true },
    list: { type: 'list', parseAll: true, combine: { list: 'bullet', par: true }, noIndent: -1 }
};
const pgmlShow = (input) => {
    return new PGMLParse(input).root?.show() ?? '';
};

var Type$1;
(function (Type) {
    Type[Type["PGMLContent"] = 1] = "PGMLContent";
    Type[Type["Paragraph"] = 2] = "Paragraph";
    Type[Type["Align"] = 3] = "Align";
    Type[Type["AlignMark"] = 4] = "AlignMark";
    Type[Type["AnswerRule"] = 5] = "AnswerRule";
    Type[Type["BulletList"] = 6] = "BulletList";
    Type[Type["Code"] = 7] = "Code";
    Type[Type["CodeClass"] = 8] = "CodeClass";
    Type[Type["CodeMark"] = 9] = "CodeMark";
    Type[Type["CodeText"] = 10] = "CodeText";
    Type[Type["Comment"] = 11] = "Comment";
    Type[Type["Emphasis"] = 12] = "Emphasis";
    Type[Type["EmphasisMark"] = 13] = "EmphasisMark";
    Type[Type["EscapeSequence"] = 14] = "EscapeSequence";
    Type[Type["HeaderMark"] = 15] = "HeaderMark";
    Type[Type["Heading1"] = 16] = "Heading1";
    Type[Type["Heading2"] = 17] = "Heading2";
    Type[Type["Heading3"] = 18] = "Heading3";
    Type[Type["Heading4"] = 19] = "Heading4";
    Type[Type["Heading5"] = 20] = "Heading5";
    Type[Type["Heading6"] = 21] = "Heading6";
    Type[Type["HorizontalRule"] = 22] = "HorizontalRule";
    Type[Type["Image"] = 23] = "Image";
    Type[Type["ImageMark"] = 24] = "ImageMark";
    Type[Type["ListItem"] = 25] = "ListItem";
    Type[Type["ListMark"] = 26] = "ListMark";
    Type[Type["MathMode"] = 27] = "MathMode";
    Type[Type["MathModeMark"] = 28] = "MathModeMark";
    Type[Type["Option"] = 29] = "Option";
    Type[Type["OptionMark"] = 30] = "OptionMark";
    Type[Type["OrderedList"] = 31] = "OrderedList";
    Type[Type["PerlCommand"] = 32] = "PerlCommand";
    Type[Type["PerlCommandMark"] = 33] = "PerlCommandMark";
    Type[Type["Pre"] = 34] = "Pre";
    Type[Type["PreMark"] = 35] = "PreMark";
    Type[Type["StarOption"] = 36] = "StarOption";
    Type[Type["StrongEmphasis"] = 37] = "StrongEmphasis";
    Type[Type["Table"] = 38] = "Table";
    Type[Type["TableCell"] = 39] = "TableCell";
    Type[Type["TableCellMark"] = 40] = "TableCellMark";
    Type[Type["TableMark"] = 41] = "TableMark";
    Type[Type["Tag"] = 42] = "Tag";
    Type[Type["TagMark"] = 43] = "TagMark";
    Type[Type["Variable"] = 44] = "Variable";
    Type[Type["VariableMark"] = 45] = "VariableMark";
    Type[Type["Verbatim"] = 46] = "Verbatim";
    Type[Type["VerbatimMark"] = 47] = "VerbatimMark";
    Type[Type["PGMLError"] = 48] = "PGMLError";
})(Type$1 || (Type$1 = {}));
const nodeProps$1 = new Map([
    [Type$1.PGMLContent, [[NodeProp.group, ['Block', 'BlockContext']]]],
    [Type$1.Paragraph, [[NodeProp.group, ['Block']]]]
]);
const nodeTypes$1 = [NodeType.none];
for (let i = 1, name; (name = Type$1[i]); ++i) {
    nodeTypes$1[i] = NodeType.define({
        id: i,
        name,
        props: nodeProps$1.get(i) ?? [],
        top: name === 'PGMLContent',
        error: name === 'PGMLError'
    });
}
// Block-level parsing functions get access to this context object.
let BlockContext$1 = class BlockContext {
    parser;
    input;
    ranges;
    block;
    to;
    reusePlaceholders = new Map();
    stoppedAt = null;
    from;
    pos; // The currently parsed position.
    content = ''; // The current range content.
    // The start and end of the current range content.
    contentStart;
    contentEnd;
    // The current parsed postion in the content relative to contentStart,
    // i.e., the position in the current content string.
    contentPos = 0;
    rangeI = 0; // The range index that contentStart points into
    constructor(
    // The parser configuration used.
    parser, input, _fragments, ranges) {
        this.parser = parser;
        this.input = input;
        this.ranges = ranges;
        this.from = this.pos = ranges[0].from;
        this.to = ranges[ranges.length - 1].to;
        this.contentStart = this.contentEnd = ranges[0].from;
        this.block = CompositeBlock.create(Type$1.PGMLContent, this.pos, 0, 0);
        this.readContent();
    }
    get parsedPos() {
        return this.pos;
    }
    advance() {
        if ((this.stoppedAt != null && this.contentStart > this.stoppedAt) ||
            (this.contentPos >= this.content.length && !this.nextRangeContent()))
            return this.finish();
        for (const item of new PGMLParse(this.content).root?.stack ?? []) {
            if (!(item instanceof Item))
                continue;
            for (const elt of pgmlFormat(item, this.from)) {
                this.addNode(elt.toTree(this.parser.nodeSet), elt.from);
            }
        }
        this.contentPos += this.content.length;
        this.pos += this.content.length;
        return null;
    }
    stopAt(pos) {
        if (this.stoppedAt != null && this.stoppedAt < pos)
            throw new RangeError("Can't move stoppedAt forward");
        this.stoppedAt = pos;
    }
    // Move to the next input range content.
    nextRangeContent() {
        this.contentStart += this.content.length;
        if (this.contentEnd >= this.to) {
            this.contentStart = this.contentEnd;
            return false;
        }
        else {
            this.contentStart = this.contentEnd + 1;
            this.moveRangeI();
            this.readContent();
            return true;
        }
    }
    moveRangeI() {
        while (this.rangeI < this.ranges.length - 1 && this.contentStart >= this.ranges[this.rangeI].to) {
            ++this.rangeI;
            this.contentStart = Math.max(this.contentStart, this.ranges[this.rangeI].from);
        }
    }
    readContent() {
        let end = this.contentStart;
        let text = '';
        if (this.contentStart < this.to) {
            text = this.contentInRange(this.contentStart, this.ranges[this.rangeI].to);
            end += text.length;
            if (this.ranges.length > 1) {
                let textOffset = this.contentStart, rangeI = this.rangeI;
                while (this.ranges[rangeI].to < end) {
                    ++rangeI;
                    const after = this.contentInRange(this.ranges[rangeI].from, this.ranges[rangeI].to);
                    end = this.ranges[rangeI].from + after.length;
                    text = text.slice(0, this.ranges[rangeI - 1].to - textOffset) + after;
                    textOffset = end - text.length;
                }
            }
        }
        this.contentEnd = end;
        this.content = text;
        this.contentPos = 0;
    }
    contentInRange(start, end) {
        const text = this.input.read(start, end);
        return start + text.length > this.to ? text.slice(0, this.to - start) : text;
    }
    addNode(block, from) {
        this.block.addChild(block, from - this.block.from);
    }
    finish() {
        return this.block.toTree(this.parser.nodeSet, this.contentStart);
    }
};
const pgmlFormat = (block, offset) => {
    if (['text', 'par', 'break', 'quote', 'forced', 'balance'].includes(block.type))
        return [];
    const children = [];
    for (const item of block.stack ?? block.children ?? []) {
        if (!(item instanceof Item) || item.type === 'text')
            continue;
        children.push(...pgmlFormat(item, offset));
    }
    const options = [];
    for (const item of block.optionStack ?? []) {
        options.push(...pgmlFormat(item, offset));
    }
    if (block.type === 'indent') {
        return children.length ? [elt(Type$1.Paragraph, block.from + offset, block.to + offset, children)] : [];
    }
    else if (block.type === 'variable') {
        children.unshift(elt(Type$1.VariableMark, block.from + offset, block.from + 1 + offset));
        if (block.text?.length)
            children.push(new TreeElement(pgPerlParser.parse(`$${block.text}`), block.from + 1 + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.VariableMark, 
            // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
            block.to - 1 - (block.hasStar ? block.hasStar : block.hasDblStar ? 2 : 0) + offset, block.to + offset));
        else
            children.push(elt(Type$1.PGMLError, block.to + offset, block.to + offset));
        return [elt(Type$1.Variable, block.from + offset, block.to + offset, children)];
    }
    else if (block.type === 'command') {
        children.unshift(elt(Type$1.PerlCommandMark, block.from + offset, block.from + 2 + offset));
        children.push(new TreeElement(pgPerlParser.parse(block.text ?? ''), (block.textFrom ?? block.from + 2) + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.PerlCommandMark, 
            // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
            block.to - (block.hasStar ? block.hasStar : block.hasDblStar ? 2 : 0) - 2 + offset, block.to + offset));
        else
            children.push(elt(Type$1.PGMLError, block.to + offset, block.to + offset));
        return [elt(Type$1.PerlCommand, block.from + offset, block.to + offset, children)];
    }
    else if (block.type === 'pre') {
        children.unshift(elt(Type$1.PreMark, block.from + offset, block.from + (block.token?.length ?? 1) + offset));
        return [elt(Type$1.Pre, block.from + offset, block.to + offset, children)];
    }
    else if (block.type === 'verbatim') {
        children.unshift(elt(Type$1.VerbatimMark, block.from + offset, block.from + (block.token?.length ?? 2) + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.VerbatimMark, block.to - block.terminator.length - (block.hasStar ?? 0) + offset, block.to + offset));
        else
            children.push(elt(Type$1.PGMLError, block.to + offset, block.to + offset));
        return [elt(Type$1.Verbatim, block.from + offset, block.to + offset, children)];
    }
    else if (block.type === 'comment') {
        return [
            elt(Type$1.Comment, block.from + offset, block.to + offset, block.terminator instanceof RegExp ? [elt(Type$1.PGMLError, block.to + offset, block.to + offset)] : [])
        ];
    }
    else if (block.type === 'answer') {
        const firstOptionBlock = options.at(0);
        return [
            elt(Type$1.AnswerRule, block.from + offset, firstOptionBlock ? firstOptionBlock.from : block.to + offset),
            ...options
        ];
    }
    else if (block.type === 'rule') {
        const firstOptionBlock = options.at(0);
        return [
            elt(Type$1.HorizontalRule, block.from + offset, firstOptionBlock ? firstOptionBlock.from : block.to + offset),
            ...options
        ];
    }
    else if (block.type === 'math') {
        const firstOptionBlock = options.at(0);
        const to = firstOptionBlock ? firstOptionBlock.from : block.to + offset;
        children.unshift(elt(Type$1.MathModeMark, block.from + offset, block.from + (block.token?.length ?? 0) + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.MathModeMark, 
            // eslint-disable-next-line @typescript-eslint/prefer-nullish-coalescing
            to - block.terminator.length - (block.hasStar ? block.hasStar : block.hasDblStar ? 2 : 0), to));
        else
            children.push(elt(Type$1.PGMLError, to + offset, to + offset));
        return [elt(Type$1.MathMode, block.from + offset, to, children), ...options];
    }
    else if (block.type === 'table') {
        const firstOptionBlock = options.at(0);
        const to = firstOptionBlock ? firstOptionBlock.from : block.to + offset;
        children.unshift(elt(Type$1.TableMark, block.from + offset, block.from + (block.token?.length ?? 0) + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.TableMark, to - block.terminator.length - (block.hasStar ?? 0), to));
        else
            children.push(elt(Type$1.PGMLError, to, to));
        return [elt(Type$1.Table, block.from + offset, to, children), ...options];
    }
    else if (block.type === 'table-cell') {
        const firstOptionBlock = options.at(0);
        const to = firstOptionBlock ? firstOptionBlock.from : block.to + offset;
        children.unshift(elt(Type$1.TableCellMark, block.from + offset, block.from + (block.token?.length ?? 0) + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.TableCellMark, to - block.terminator.length - (block.hasStar ?? 0), to));
        else
            children.push(elt(Type$1.PGMLError, to, to));
        return [elt(Type$1.TableCell, block.from + offset, to, children), ...options];
    }
    else if (block.type === 'image') {
        const firstOptionBlock = options.at(0);
        const to = firstOptionBlock ? firstOptionBlock.from : block.to + offset;
        children.unshift(elt(Type$1.ImageMark, block.from + offset, block.from + 2 + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.ImageMark, to - 2, to));
        else
            children.push(elt(Type$1.PGMLError, to, to));
        return [elt(Type$1.Image, block.from + offset, to, children), ...options];
    }
    else if (block.type === 'tag') {
        const firstOptionBlock = options.at(0);
        const to = firstOptionBlock ? firstOptionBlock.from : block.to + offset;
        children.unshift(elt(Type$1.TagMark, block.from + offset, block.from + 2 + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.TagMark, to - 2, to));
        else
            children.push(elt(Type$1.PGMLError, to, to));
        return [elt(Type$1.Tag, block.from + offset, to, children), ...options];
    }
    else if (block.type === 'options') {
        children.unshift(elt(Type$1.OptionMark, block.from + offset, block.from + 1 + offset));
        if (block.text)
            children.push(new TreeElement(pgPerlParser.parse(block.text), (block.textFrom ?? block.from + 1) + offset));
        children.push(elt(Type$1.OptionMark, block.to - 1 + offset, block.to + offset));
        return [elt(Type$1.Option, block.from + offset, block.to + offset, children)];
    }
    else if (block.type === 'bold' || block.type === 'italic') {
        children.unshift(elt(Type$1.EmphasisMark, block.from + offset, block.from + (block.token?.length ?? 1) + offset));
        const lastChild = block.stack?.at(-1);
        if (lastChild instanceof Item && block.to === lastChild.to)
            children.push(elt(Type$1.PGMLError, block.to + offset, block.to + offset));
        else
            children.push(elt(Type$1.EmphasisMark, (lastChild instanceof Item ? lastChild.to : block.to - (block.token?.length ?? 1)) + offset, block.to + offset));
        return [
            elt(block.type === 'bold' ? Type$1.StrongEmphasis : Type$1.Emphasis, block.from + offset, block.to + offset, children)
        ];
    }
    else if (block.type === 'align') {
        children.unshift(elt(Type$1.AlignMark, block.from + offset, block.from + (block.token?.length ?? 2) + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.AlignMark, block.to - block.terminator.length + offset, block.to + offset));
        return [elt(Type$1.Align, block.from + offset, block.to + offset, children)];
    }
    else if (block.type === 'heading') {
        children.unshift(elt(Type$1.HeaderMark, block.from + offset, block.from + (block.token?.length ?? 1) + offset));
        if (typeof block.terminator === 'string')
            children.push(elt(Type$1.HeaderMark, block.to - block.terminator.length + offset, block.to + offset));
        return [
            elt((Type$1.Heading1.valueOf() - 1 + (block.n ?? 1)), block.from + offset, block.to + offset, children)
        ];
    }
    else if (block.type === 'list') {
        return [
            elt(block.bullet && ['disc', 'square', 'circle', 'bullet'].includes(block.bullet)
                ? Type$1.BulletList
                : Type$1.OrderedList, block.from + offset, block.to + offset, children)
        ];
    }
    else if (block.type === 'bullet') {
        return [
            elt(Type$1.ListItem, block.from + offset, block.to + offset, [
                elt(Type$1.ListMark, block.from + offset, block.from + (block.token?.length ?? 1) + offset),
                elt(Type$1.Paragraph, block.from + (block.token?.length ?? 1) + offset, block.to + offset, children)
            ])
        ];
    }
    else if (block.type === 'code') {
        const codeText = block.stack?.[0];
        return [
            elt(Type$1.Code, block.from + offset, block.to + offset, [
                elt(Type$1.CodeMark, block.from + offset, block.from + (block.token?.length ?? 1) + offset),
                ...(block.class
                    ? [
                        elt(Type$1.CodeClass, block.from + (block.token?.length ?? 1) + offset, block.from + (block.token?.length ?? 1) + block.class.length + offset)
                    ]
                    : []),
                ...(codeText instanceof Item ? [elt(Type$1.CodeText, codeText.from + offset, codeText.to + offset)] : []),
                typeof block.terminator === 'string'
                    ? elt(Type$1.CodeMark, block.to - block.terminator.length + offset, block.to + offset)
                    : elt(Type$1.PGMLError, block.to + offset, block.to + offset)
            ])
        ];
    }
    else if (block.type === 'slash') {
        return [elt(Type$1.EscapeSequence, block.from + offset, block.to + offset)];
    }
    console.log(`unhandled ${block.type}`);
    return children;
};
const pgmlHighlighting = styleTags({
    Paragraph: tags.content,
    'AlignMark CodeMark EmphasisMark HeaderMark ImageMark ListMark MathModeMark OptionMark': tags.processingInstruction,
    'PreMark PerlCommandMark TableMark TableCellMark TagMark VariableMark VerbatimMark': tags.processingInstruction,
    CodeText: tags.monospace,
    CodeClass: tags.labelName,
    HorizontalRule: tags.contentSeparator,
    EscapeSequence: tags.escape,
    'AnswerRule Image MathMode': tags.atom,
    'Heading1/...': tags.heading1,
    'Heading2/...': tags.heading2,
    'Heading3/...': tags.heading3,
    'Heading4/...': tags.heading4,
    'Heading5/...': tags.heading5,
    'Heading6/...': tags.heading6,
    'OrderedList/... BulletList/...': tags.list,
    Comment: tags.lineComment,
    'Emphasis/...': tags.emphasis,
    'StrongEmphasis/...': tags.strong,
    StarOption: tags.controlOperator,
    PGMLError: tags.invalid
});
// A PGML parser configuration.
class PGMLParser extends Parser {
    nodeTypes = Object.create(null);
    nodeSet;
    // The parser's syntax [node types](https://lezer.codemirror.net/docs/ref/#common.NodeSet).
    constructor(props) {
        super();
        this.nodeSet = new NodeSet(nodeTypes$1).extend(pgmlHighlighting, ...(props?.length ? props : []));
        for (const t of this.nodeSet.types)
            this.nodeTypes[t.name] = t.id;
    }
    createParse(input, fragments, ranges) {
        return new BlockContext$1(this, input, fragments, ranges);
    }
    getNodeType(name) {
        return this.nodeTypes[name];
    }
}

const pgmlIndent = {
    PGMLContent: () => null,
    PerlCommand: delimitedIndent({ closing: '@]' }),
    Table: delimitedIndent({ closing: '#]' }),
    Tag: delimitedIndent({ closing: '>]' })
};
let i$1 = 99;
let rank = 1;
const answerRuleSnippets = [
    snippetCompletion('[_${}]${}{${$answer}}${}', {
        label: '[_]{ }',
        info: 'answer rule',
        section: { name: 'answer', rank },
        boost: i$1--
    }),
    snippetCompletion('[_${}]*{${$answer}}${}', {
        label: '[_]*{ }',
        info: 'array answer rule',
        section: { name: 'answer', rank },
        boost: i$1--
    })
];
++rank;
const mathModeSnippets = [
    ['[` `]', 'inline math'],
    ['[`` ``]', 'display style math'],
    ['[``` ```]', 'block display math'],
    ['[: :]', 'parsed inline math'],
    ['[:: ::]', 'parsed display style math'],
    ['[::: :::]', 'parsed block display math'],
    ['[: :]*', 'parsed inline math in active context'],
    ['[:: ::]*', 'parsed display style math in active context'],
    ['[::: :::]*', 'parsed block display math in active context'],
    ['[: :]**', 'parsed and reduced inline math in active context'],
    ['[:: ::]**', 'parsed and reduced display style math in active context'],
    ['[::: :::]**', 'parsed and reduced block display math in active context']
].map(([label, info]) => {
    return snippetCompletion(label.replace(' ', '${ }') + '${}', {
        label,
        info,
        type: 'function',
        section: { name: 'math', rank },
        boost: i$1--
    });
});
const variableSnippets = [
    snippetCompletion('[$${ }]${}', {
        label: '[$ ]',
        info: 'variable',
        section: { name: 'substitution', rank: ++rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[$${ }]*${}', {
        label: '[$ ]*',
        info: 'variable with value not HTML escaped',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[$${ }]**${}', {
        label: '[$ ]**',
        info: 'variable with value PGML parsed',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[$${ }]***${}', {
        label: '[$ ]***',
        info: 'variable with LaTeX value not HTML escaped',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    })
];
const perlCommandSnippets = [
    snippetCompletion('[@ ${ } @]${}', {
        label: '[@ @]',
        info: 'perl command',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[@ ${ } @]*${}', {
        label: '[@ @]*',
        info: 'perl command with result not HTML escaped',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[@ ${ } @]**${}', {
        label: '[@ @]**',
        info: 'perl command with result PGML parsed',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[@ ${ } @]***${}', {
        label: '[@ @]***',
        info: 'perl command with LaTeX result not HTML escaped',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[@\n\t${ }\n@]${}', {
        label: '[@ @]',
        info: 'perl command (multi line)',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[@\n\t${ }\n@]*${}', {
        label: '[@ @]*',
        info: 'perl command with result not HTML escaped (multi line)',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[@\n\t${ }\n@]**${}', {
        label: '[@ @]**',
        info: 'perl command with result PGML parsed (multi line)',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    }),
    snippetCompletion('[@\n\t${ }\n@]***${}', {
        label: '[@ @]***',
        info: 'perl command with LaTeX result not HTML escaped (multi line)',
        section: { name: 'substitution', rank },
        type: 'variable',
        boost: i$1--
    })
];
const imageSnippet = snippetCompletion('[!${alt text}!]{${$source}}${}', {
    label: '[! !]{ }',
    info: 'image',
    type: 'type',
    section: { name: 'substitution', rank },
    boost: i$1--
});
const tagSnippets = [
    snippetCompletion('[<\n\t${ }\n>]${}', {
        label: '[< >]',
        info: 'html tag (multi line)',
        type: 'type',
        section: { name: 'substitution', rank: ++rank },
        boost: i$1--
    }),
    snippetCompletion('[<${ }>]${}', {
        label: '[< >]',
        info: 'html tag (single line)',
        type: 'type',
        section: { name: 'substitution', rank },
        boost: i$1--
    })
];
const commentSnippet = snippetCompletion('[% ${ } %]${}', {
    label: '[% %]',
    info: 'comment',
    type: 'type',
    section: { name: 'substitution', rank },
    boost: i$1--
});
const tableSnippets = [
    snippetCompletion('[#\n\t[.${ }.]${}\n#]${}', {
        label: '[# [. .] #]',
        info: 'table (multi line)',
        section: { name: 'table', rank: ++rank },
        type: 'type',
        boost: i$1--
    }),
    snippetCompletion('[#\n\t[.${ }.]${}\n#]*${}', {
        label: '[# [. .] #]*',
        info: 'layout table (multi line)',
        section: { name: 'table', rank },
        type: 'type',
        boost: i$1--
    }),
    snippetCompletion('[# [.${ }.]${} #]${}', {
        label: '[# [. .] #]',
        info: 'table (single line)',
        section: { name: 'table', rank },
        type: 'type',
        boost: i$1--
    }),
    snippetCompletion('[# [.${ }.]${} #]*${}', {
        label: '[# [. .] #]*',
        info: 'layout table (single line)',
        section: { name: 'table', rank },
        type: 'type',
        boost: i$1--
    })
];
const tableCellSnippets = [
    snippetCompletion('[.${ }.]${}', {
        label: '[. .]',
        info: 'table cell',
        section: { name: 'table', rank },
        type: 'type',
        boost: i$1--
    }),
    snippetCompletion('[.${ }.]*${}', {
        label: '[. .]',
        info: 'table cell at end of table row',
        section: { name: 'table', rank },
        type: 'type',
        boost: i$1--
    })
];
++rank;
const verbatimSnippets = [
    ['[| |]', 'verbatim'],
    ['[| |]*', 'verbatim code'],
    ['[|| ||]', 'verbatim with unprocessed inner verbatim'],
    ['[|| ||]*', 'verbatim code with unprocessed inner verbatim']
].map(([label, info]) => {
    return snippetCompletion(label.replace(' ', '${ }') + '${}', {
        label,
        info,
        type: 'type',
        section: { name: 'formatting', rank },
        boost: i$1--
    });
});
const preSnippet = snippetCompletion(':   ${}', {
    label: ':   ',
    info: 'pre',
    section: { name: 'formatting', rank },
    type: 'type',
    boost: i$1--
});
const pgmlLanguageData = {
    PGMLContent: defineLanguageFacet({
        commentTokens: { block: { open: '[%', close: '%]' } },
        autocomplete: (context) => {
            const nodeAt = syntaxTree(context.state).resolveInner(context.pos, 0);
            const currentLine = context.state.doc.lineAt(context.pos);
            const textBefore = currentLine.text.slice(0, context.pos - currentLine.from);
            const textAfter = currentLine.text.slice(context.pos - currentLine.from);
            // This returns 2 if inside a complete node, 1 if inside an incomplete node, and false otherwise.
            const inside = (nodeNames) => {
                for (let pos = nodeAt; pos; pos = pos.parent) {
                    if ((nodeNames instanceof Array && nodeNames.includes(pos.name)) || nodeNames === pos.name)
                        return (pos.name === 'Comment' && !pos.lastChild) ||
                            (pos.name !== 'Comment' && pos.lastChild && pos.lastChild.name !== 'PGMLError')
                            ? 2
                            : 1;
                    if (pos.type.isTop)
                        break;
                }
                return false;
            };
            if (inside(['Comment', 'PerlCommand', 'Variable', 'Option', 'Verbatim']) === 2)
                return;
            const insideTable = inside('Table');
            if ((insideTable == 2 || (insideTable == 1 && (!textBefore.endsWith('[#') || /^\s*#\]/.test(textAfter)))) &&
                ((!inside('TableCell') && !textBefore.endsWith('.]*')) ||
                    (textBefore.endsWith('[.') && !/^[ \t]*\.\]/.test(textAfter)))) {
                const previous = context.matchBefore(/\[|\[\./);
                return {
                    from: previous?.from ?? context.pos,
                    options: tableCellSnippets
                };
            }
            const insideMathMode = inside('MathMode');
            if (insideMathMode == 2 ||
                (insideMathMode == 1 &&
                    (!/\[`{1,3}$/.test(textBefore) || /^\s*`{1,3}\]/.test(textAfter)) &&
                    (!/\[:{1,3}$/.test(textBefore) || /^\s*:{1,3}\]/.test(textAfter)))) {
                const previous = context.matchBefore(/\[|\[\$|\[@/);
                if (!previous && !context.explicit)
                    return;
                return {
                    from: previous?.from ?? context.pos,
                    options: [...variableSnippets, ...perlCommandSnippets]
                };
            }
            const insideImage = inside('Image');
            if (insideImage == 2 || (insideImage == 1 && (!textBefore.endsWith('[!') || /^\s*!\]/.test(textAfter)))) {
                const previous = context.matchBefore(/\[[$@]?\s*$/);
                if (!previous && !context.explicit)
                    return;
                return {
                    from: previous?.from ?? context.pos,
                    options: [...variableSnippets, ...perlCommandSnippets]
                };
            }
            const insideVerbatim = inside('Verbatim');
            if (insideVerbatim == 2 ||
                (insideVerbatim == 1 && (!/\[\|*$/.test(textBefore) || /^\s*\|*\]/.test(textAfter)))) {
                return null;
            }
            if (/\[(`{1,3}|:{1,3}|\|{1,}|[_$@!<#%])?[\t ]*$/.test(textBefore)) {
                const previous = context.matchBefore(/\[(`{1,3}|:{1,3}|\|{1,}|[_$@!<#%])?[\t ]*$/);
                if (!previous && !context.explicit)
                    return;
                return {
                    from: previous?.from ?? context.pos,
                    options: [
                        ...answerRuleSnippets,
                        ...mathModeSnippets,
                        ...variableSnippets,
                        ...perlCommandSnippets,
                        imageSnippet,
                        ...tagSnippets,
                        commentSnippet,
                        ...tableSnippets,
                        ...verbatimSnippets
                    ]
                };
            }
            if (/^(\s{4})*:\s*/.test(textBefore)) {
                const previous = context.matchBefore(/:\s*/);
                return {
                    from: previous?.from ?? context.pos,
                    options: [preSnippet]
                };
            }
            if (!context.explicit)
                return;
            return {
                from: context.pos,
                options: [
                    ...answerRuleSnippets,
                    ...mathModeSnippets,
                    ...variableSnippets,
                    ...perlCommandSnippets,
                    imageSnippet,
                    ...tagSnippets,
                    commentSnippet,
                    ...tableSnippets,
                    ...verbatimSnippets,
                    preSnippet
                ]
            };
        },
        // '[' is removed from this list because it interferes with the above snippets.
        closeBrackets: { brackets: ['(', '{', "'", '"'] }
    })
};
const pgmlNodeProps = [indentNodeProp.add(pgmlIndent), languageDataProp.add(pgmlLanguageData)];

var Type;
(function (Type) {
    Type[Type["PGTextContent"] = 1] = "PGTextContent";
    Type[Type["DisplayMathMode"] = 2] = "DisplayMathMode";
    Type[Type["Emphasis"] = 3] = "Emphasis";
    Type[Type["EmphasisMark"] = 4] = "EmphasisMark";
    Type[Type["InlineMathMode"] = 5] = "InlineMathMode";
    Type[Type["MathModeMark"] = 6] = "MathModeMark";
    Type[Type["ParsedMathMode"] = 7] = "ParsedMathMode";
    Type[Type["ParsedMathModeMark"] = 8] = "ParsedMathModeMark";
    Type[Type["PerlCommand"] = 9] = "PerlCommand";
    Type[Type["PerlCommandMark"] = 10] = "PerlCommandMark";
    Type[Type["StrongEmphasis"] = 11] = "StrongEmphasis";
    Type[Type["PerlInterpolation"] = 12] = "PerlInterpolation";
    Type[Type["PGTextError"] = 13] = "PGTextError";
})(Type || (Type = {}));
const nodeProps = new Map([
    [Type.PGTextContent, [[NodeProp.group, ['Block', 'BlockContext']]]]
]);
const nodeTypes = [NodeType.none];
for (let i = 1, name; (name = Type[i]); ++i) {
    nodeTypes[i] = NodeType.define({
        id: i,
        name,
        props: nodeProps.get(i) ?? [],
        top: name === 'PGTextContent',
        error: name === 'PGTextError'
    });
}
// Block-level parsing functions get access to this context object.
class BlockContext {
    parser;
    input;
    ranges;
    block;
    line = '';
    to;
    stoppedAt = null;
    // The start and end of the current line.
    lineStart;
    lineEnd;
    // The range index that lineStart points into
    rangeI = 0;
    constructor(parser, input, _fragments, ranges) {
        this.parser = parser;
        this.input = input;
        this.ranges = ranges;
        this.to = ranges[ranges.length - 1].to;
        this.lineStart = this.lineEnd = ranges[0].from;
        this.block = CompositeBlock.create(Type.PGTextContent, this.lineStart, 0, 0);
        if (this.input.read(ranges[0].from, ranges[0].from + 1) == '\n')
            this.readLine();
    }
    get parsedPos() {
        return this.lineStart;
    }
    advance() {
        if ((this.stoppedAt != null && this.lineStart > this.stoppedAt) || !this.nextLine())
            return this.finish();
        const cx = new InlineContext(this, this.line, this.lineStart);
        outer: for (let pos = this.lineStart; pos < cx.end;) {
            const next = cx.char(pos);
            for (const token of InlineParsers) {
                const result = token(cx, next, pos);
                if (result >= 0) {
                    pos = result;
                    if (!cx.delimitersResolved() && pos >= cx.end)
                        cx.nextLine();
                    continue outer;
                }
            }
            ++pos;
            if (!cx.delimitersResolved() && pos >= cx.end)
                cx.nextLine();
        }
        for (const elt of cx.takeContent(0, true)) {
            this.addNode(elt.toTree(this.parser.nodeSet), elt.from);
        }
        return null;
    }
    stopAt(pos) {
        if (this.stoppedAt != null && this.stoppedAt < pos)
            throw new RangeError("Can't move stoppedAt forward");
        this.stoppedAt = pos;
    }
    // Move to the next input line.
    nextLine() {
        if (this.lineEnd >= this.to) {
            this.lineStart = this.lineEnd;
            return false;
        }
        else {
            this.lineStart = this.lineEnd + 1;
            this.moveRangeI();
            this.readLine();
            return true;
        }
    }
    moveRangeI() {
        while (this.rangeI < this.ranges.length - 1 && this.lineStart >= this.ranges[this.rangeI].to) {
            ++this.rangeI;
            this.lineStart = Math.max(this.lineStart, this.ranges[this.rangeI].from);
        }
    }
    readLine() {
        let text = '';
        let end = this.lineStart;
        if (this.lineStart < this.to) {
            text = this.lineChunkAt(this.lineStart);
            end += text.length;
            if (this.ranges.length > 1) {
                let textOffset = this.lineStart, rangeI = this.rangeI;
                while (this.ranges[rangeI].to < end) {
                    ++rangeI;
                    const nextFrom = this.ranges[rangeI].from;
                    const after = this.lineChunkAt(nextFrom);
                    end = nextFrom + after.length;
                    text = text.slice(0, this.ranges[rangeI - 1].to - textOffset) + after;
                    textOffset = end - text.length;
                }
            }
        }
        this.lineEnd = end;
        this.line = text;
    }
    lineChunkAt(pos) {
        let text = this.input.chunk(pos);
        if (!this.input.lineChunks) {
            const eol = text.indexOf('\n');
            text = eol < 0 ? text : text.slice(0, eol);
        }
        else if (text === '\n') {
            text = '';
        }
        return pos + text.length > this.to ? text.slice(0, this.to - pos) : text;
    }
    addNode(block, from) {
        this.block.addChild(block, from - this.block.from);
    }
    finish() {
        return this.block.toTree(this.parser.nodeSet, this.lineStart);
    }
}
const Delimiters = {
    InlineMathMode: { nodeType: Type.InlineMathMode, mark: Type.MathModeMark },
    DisplayMathMode: { nodeType: Type.DisplayMathMode, mark: Type.MathModeMark },
    ParsedMathMode: { nodeType: Type.ParsedMathMode, mark: Type.ParsedMathModeMark },
    Emphasis: { nodeType: Type.Emphasis, mark: Type.EmphasisMark },
    StrongEmphasis: { nodeType: Type.StrongEmphasis, mark: Type.EmphasisMark },
    PerlInterpolation: { nodeType: Type.PerlInterpolation },
    // These are only allowed to occur in a perl interpolation and do not resolve to actual elements.
    Brace: {},
    Bracket: {}
};
class InlineDelimiter {
    type;
    from;
    to;
    forceResolve;
    interpolationParent;
    constructor(type, from, to, forceResolve = true, interpolationParent) {
        this.type = type;
        this.from = from;
        this.to = to;
        this.forceResolve = forceResolve;
        this.interpolationParent = interpolationParent;
    }
}
const bold = [66, 79, 76, 68]; // BOLD
const italic = [73, 84, 65, 76, 73, 67]; // ITALIC
const InlineParsers = [
    // Perl command
    (cx, next, start) => {
        if (next != 92 /* \\ */ || cx.char(start + 1) != 123 /* { */)
            return -1;
        let pos = start + 2;
        for (; !cx.atEnd(pos) && (cx.char(pos) != 92 /* \ */ || cx.char(pos + 1) != 125) /* } */; ++pos)
            ;
        const end = pos + 2 < cx.end ? pos + 2 : cx.end;
        return cx.append(elt(Type.PerlCommand, start, end, [
            elt(Type.PerlCommandMark, start, start + 2),
            new TreeElement(pgPerlParser.parse(cx.slice(start + 2, pos)), start + 2),
            cx.char(end - 2) == 92 && cx.char(end - 1) == 125
                ? elt(Type.PerlCommandMark, pos, end)
                : elt(Type.PGTextError, end, end)
        ]));
    },
    // Emphasis
    (cx, next, start) => {
        if (next != 36 /* $ */)
            return -1;
        let pos = start + 1;
        const haveBrace = cx.char(pos) == 123; /* { */
        if (haveBrace) {
            ++pos;
            pos = cx.skipSpace(pos);
        }
        // BBOLD
        const isBold = [66, ...bold].every((ch, i) => cx.char(pos + i) == ch);
        // BITALIC
        const isItalic = [66, ...italic].every((ch, i) => cx.char(pos + i) == ch);
        if (isBold)
            pos += 5;
        else if (isItalic)
            pos += 7;
        else
            return -1;
        if (haveBrace) {
            pos = cx.skipSpace(pos);
            if (cx.char(pos) != 125 /* } */)
                return -1;
            ++pos;
        }
        else if (isIdentifierChar(cx.char(pos)))
            return -1;
        return cx.addDelimiter(isBold ? Delimiters.StrongEmphasis : Delimiters.Emphasis, start, pos);
    },
    // Emphasis end
    (cx, next, start) => {
        if (next != 36 /* $ */)
            return -1;
        let pos = start + 1;
        const haveBrace = cx.char(pos) == 123; /* { */
        if (haveBrace) {
            ++pos;
            pos = cx.skipSpace(pos);
        }
        // EBOLD
        const isBold = [69, ...bold].every((ch, i) => cx.char(pos + i) == ch);
        // EITALIC
        const isItalic = [69, ...italic].every((ch, i) => cx.char(pos + i) == ch);
        if (isBold)
            pos += 5;
        else if (isItalic)
            pos += 7;
        else
            return -1;
        if (haveBrace) {
            pos = cx.skipSpace(pos);
            if (cx.char(pos) != 125 /* } */)
                return -1;
            ++pos;
        }
        else if (isIdentifierChar(cx.char(pos)))
            return -1;
        // Scan back to the last emphasis start marker.
        for (let i = cx.parts.length - 1; i >= 0; --i) {
            const part = cx.parts[i];
            if (part instanceof InlineDelimiter &&
                (part.type === Delimiters.Emphasis || part.type === Delimiters.StrongEmphasis) &&
                part.type.nodeType) {
                // Finish the content and replace the entire range in cx.parts with the emphasis node.
                const content = cx.takeContent(i);
                content.unshift(elt(Type.EmphasisMark, part.from, part.to));
                content.push(elt(Type.EmphasisMark, start, pos));
                const emphasis = (cx.parts[i] = elt(part.type.nodeType, part.from, pos, content));
                return emphasis.to;
            }
        }
        return -1;
    },
    // Perl Interpolation
    // FIXME: Quotes and heredocs are not supported in interpolation at this point.
    (cx, next, start) => {
        const haveBrace = cx.char(start + 1) == 123; /* { */
        if (next != 36 /* $ */ || (!isVariableStartChar(cx.char(start + 1)) && !haveBrace))
            return -1;
        if (haveBrace) {
            cx.addDelimiter(Delimiters.PerlInterpolation, start, start + 2, false);
            return cx.addDelimiter(Delimiters.Brace, start + 1, start + 2, true, cx.parts.at(-1));
        }
        let pos = start + 2;
        for (; pos < cx.end && isIdentifierChar(cx.char(pos)); ++pos)
            ;
        return cx.addDelimiter(Delimiters.PerlInterpolation, start, pos, false);
    },
    // Brace or bracket
    (cx, next, start) => {
        if (next != 123 /* { */ && next != 91 /* [ */)
            return -1;
        // Scan back and see if there is an open interpolation that ends here.
        // If not, then ignore the brace or bracket.
        for (let i = cx.parts.length - 1; i >= 0; --i) {
            const part = cx.parts[i];
            if (part instanceof InlineDelimiter &&
                part.type === Delimiters.PerlInterpolation &&
                part.to == start &&
                cx.char(part.from + 1) != 123) {
                ++part.to;
                return cx.addDelimiter(next == 123 ? Delimiters.Brace : Delimiters.Bracket, start, start + 1, true, part);
            }
        }
        return -1;
    },
    // Brace or bracket end (only in perl interpolation)
    (cx, next, start) => {
        if (next != 125 /* } */ && next != 93 /* ] */)
            return -1;
        // Scan back to the matching start marker.
        for (let i = cx.parts.length - 1; i >= 0; --i) {
            const part = cx.parts[i];
            if (part instanceof InlineDelimiter &&
                ((next == 125 && part.type === Delimiters.Brace) || (next == 93 && part.type === Delimiters.Bracket)) &&
                part.interpolationParent) {
                part.interpolationParent.to = start + 1;
                cx.parts.splice(i, 1);
                return start + 1;
            }
        }
        return -1;
    },
    // Dereferencing arrow (only in perl interpolation)
    (cx, next, start) => {
        if (next != 45 /* - */ ||
            cx.char(start + 1) != 62 /* > */ ||
            (cx.char(start + 2) != 123 /* { */ && cx.char(start + 2) != 91) /* [ */)
            return -1;
        // Scan back and see if there is an open interpolation that ends here.
        // If not, then ignore the dereferencing arrow.
        for (let i = cx.parts.length - 1; i >= 0; --i) {
            const part = cx.parts[i];
            if (part instanceof InlineDelimiter &&
                part.type === Delimiters.PerlInterpolation &&
                part.to == start &&
                cx.char(part.from + 1) != 123) {
                part.to += 2;
                return start + 2;
            }
        }
        return -1;
    },
    // Math mode
    (cx, next, start) => {
        if (next != 92 /* \\ */ || (cx.char(start + 1) != 40 /* ( */ && cx.char(start + 1) != 91) /* [ */)
            return -1;
        return cx.addDelimiter(cx.char(start + 1) == 40 ? Delimiters.InlineMathMode : Delimiters.DisplayMathMode, start, start + 2);
    },
    // Math mode end
    (cx, next, start) => {
        if (next != 92 /* \\ */ || (cx.char(start + 1) != 41 /* ) */ && cx.char(start + 1) != 93) /* ] */)
            return -1;
        // Scan back to the last math mode start marker.
        for (let i = cx.parts.length - 1; i >= 0; --i) {
            const part = cx.parts[i];
            if (part instanceof InlineDelimiter &&
                (part.type === Delimiters.InlineMathMode || part.type === Delimiters.DisplayMathMode) &&
                part.type.nodeType) {
                const content = cx.takeContent(i);
                // If there is an in progress math mode, then replace the entire range with an error node.
                if (cx.parts.find((p) => p instanceof InlineDelimiter &&
                    (p.type === Delimiters.InlineMathMode || p.type === Delimiters.DisplayMathMode))) {
                    const errorNode = (cx.parts[i] = elt(Type.PGTextError, part.from, start + 2));
                    return errorNode.to;
                }
                // Finish the content and replace the entire range in cx.parts with the math mode node.
                content.unshift(elt(Type.MathModeMark, part.from, part.to));
                content.push(elt(Type.MathModeMark, start, start + 2));
                const mathMode = (cx.parts[i] = elt(part.type.nodeType, part.from, start + 2, content));
                return mathMode.to;
            }
        }
        return -1;
    },
    // Parsed math mode
    (cx, next, start) => {
        if (next != 96 /* ` */ ||
            (start &&
                cx.char(start - 1) == 96 &&
                (!(cx.parts.at(-1) instanceof Element) || cx.parts.at(-1)?.type !== Type.ParsedMathMode)) ||
            cx.parts.find((p) => p instanceof InlineDelimiter && p.type === Delimiters.ParsedMathMode))
            return -1;
        return cx.addDelimiter(Delimiters.ParsedMathMode, start, start + (cx.char(start + 1) == 96 ? 1 : 0) + 1);
    },
    // Parsed math mode end
    (cx, next, start) => {
        if (next != 96 /* ` */ || (start && cx.char(start - 1) == 96))
            return -1;
        // Scan back to the last parsed math mode start marker.
        for (let i = cx.parts.length - 1; i >= 0; --i) {
            const part = cx.parts[i];
            if (part instanceof InlineDelimiter && part.type === Delimiters.ParsedMathMode && part.type.nodeType) {
                const content = cx.takeContent(i);
                const numBackticks = part.to - part.from;
                if (numBackticks == 2 && cx.char(start + 1) != 96)
                    return -1;
                // Finish the content and replace the entire range in cx.parts with the parsed math mode node.
                const star = cx.char(start + numBackticks) == 42 /* * */ ? 1 : 0;
                content.unshift(elt(Type.ParsedMathModeMark, part.from, part.to));
                content.push(elt(Type.ParsedMathModeMark, start, start + numBackticks + star));
                const mathMode = (cx.parts[i] = elt(part.type.nodeType, part.from, start + numBackticks + star, content));
                return mathMode.to;
            }
        }
        return -1;
    }
];
// Inline parsing functions get access to this context, and use it to read the content and emit syntax nodes.
class InlineContext {
    blockContext;
    text;
    offset;
    parts = [];
    constructor(
    // The current block context.
    blockContext, 
    // The text of this inline section.
    text, 
    // The starting offset of the section in the document.
    offset) {
        this.blockContext = blockContext;
        this.text = text;
        this.offset = offset;
    }
    // Get the character code at the given (document-relative) position.
    char(pos) {
        return pos >= this.end ? -1 : this.text.charCodeAt(pos - this.offset);
    }
    // The position of the end of this inline section.
    get end() {
        return this.offset + this.text.length;
    }
    atEnd(pos) {
        return pos >= this.end && !this.nextLine();
    }
    nextLine() {
        const nextLineExists = this.blockContext.nextLine();
        if (nextLineExists)
            this.text += '\n' + this.blockContext.line;
        return nextLineExists;
    }
    // Get a substring of this inline section. Again uses document-relative positions.
    slice(from, to) {
        return this.text.slice(from - this.offset, to - this.offset);
    }
    append(elt) {
        this.parts.push(elt);
        return elt.to;
    }
    // Add a delimiter at this given position. `open` and `close` indicate whether this delimiter is opening, closing,
    // or both. Returns the end of the delimiter, for convenient returning from parse functions.
    addDelimiter(type, from, to, forceResolve = true, parent) {
        return this.append(new InlineDelimiter(type, from, to, forceResolve, parent));
    }
    delimitersResolved() {
        for (const part of this.parts) {
            if (part instanceof InlineDelimiter && part.forceResolve)
                return false;
        }
        return true;
    }
    // Return element parts from the given start index on as an array of elements.  All unresolved inline delimiters in
    // the range (except a delimiter in the startIndex position if resolveStart is true) are turned into the appropriate
    // node terminated by an error node.
    takeContent(startIndex = 0, resolveStart = false) {
        const content = [];
        for (let i = startIndex; i < this.parts.length; ++i) {
            const part = this.parts[i];
            if (part instanceof Element)
                content.push(part);
            else if ((resolveStart || i > startIndex) && part instanceof InlineDelimiter) {
                if (part.type.mark && part.type.nodeType) {
                    const to = this.parts[i + 1]?.from ?? this.end;
                    content.push(elt(part.type.nodeType, part.from, to, [
                        elt(part.type.mark, part.from, part.to),
                        elt(Type.PGTextError, to, to)
                    ]));
                }
                else if (part.type === Delimiters.PerlInterpolation) {
                    content.push(elt(Type.PerlInterpolation, part.from, part.to, [
                        new TreeElement(pgPerlParser.parse(this.slice(part.from, part.to)), part.from)
                    ]));
                }
            }
        }
        this.parts.length = startIndex;
        return content;
    }
    // Skip space after the given (document) position, returning either the position of the next non-space character or
    // the end of the section.
    skipSpace(from) {
        return skipSpace(this.text, from - this.offset) + this.offset;
    }
}
const pgTextHighlighting = styleTags({
    'MathModeMark ParsedMathModeMark PerlCommandMark': tags.processingInstruction,
    'InlineMathMode DisplayMathMode ParsedMathMode': tags.atom,
    EmphasisMark: tags.special(tags.variableName),
    'Emphasis/...': tags.emphasis,
    'StrongEmphasis/...': tags.strong,
    PGTextError: tags.invalid
});
// A PG Text parser configuration.
class PGTextParser extends Parser {
    nodeTypes = Object.create(null);
    nodeSet;
    // The parser's syntax [node types](https://lezer.codemirror.net/docs/ref/#common.NodeSet).
    constructor(props) {
        super();
        this.nodeSet = new NodeSet(nodeTypes).extend(pgTextHighlighting, ...(props?.length ? props : []));
        for (const t of this.nodeSet.types)
            this.nodeTypes[t.name] = t.id;
    }
    createParse(input, fragments, ranges) {
        return new BlockContext(this, input, fragments, ranges);
    }
    getNodeType(name) {
        return this.nodeTypes[name];
    }
}

const pgTextIndent = { PGTextContent: () => null };
let i = 99;
const perlCommandSnippet = snippetCompletion('\\\\{${ }\\\\}${}', {
    label: '\\{ \\}',
    info: 'perl command',
    type: 'constant',
    boost: i--
});
const inlineMathSnippet = snippetCompletion('\\(${ }\\)${}', {
    label: '\\( \\)',
    info: 'inline math',
    type: 'constant',
    boost: i--
});
const displayMathSnippet = snippetCompletion('\\[${ }\\]${}', {
    label: '\\[ \\]',
    info: 'display math',
    type: 'constant',
    boost: i--
});
const pgTextLanguageData = {
    PGTextContent: defineLanguageFacet({
        autocomplete: (context) => {
            const nodeAt = syntaxTree(context.state).resolveInner(context.pos, -1);
            const currentLine = context.state.doc.lineAt(context.pos);
            const textBefore = currentLine.text.slice(0, context.pos - currentLine.from);
            const textAfter = currentLine.text.slice(context.pos - currentLine.from);
            // This returns 2 if inside a complete node, 1 if inside an incomplete node, and false otherwise.
            const inside = (nodeNames) => {
                for (let pos = nodeAt; pos; pos = pos.parent) {
                    if ((nodeNames instanceof Array && nodeNames.includes(pos.name)) || nodeNames === pos.name)
                        return (pos.name === 'Comment' && !pos.lastChild) ||
                            (pos.name !== 'Comment' && pos.lastChild && pos.lastChild.name !== 'PGTextError')
                            ? 2
                            : 1;
                    if (pos.type.isTop)
                        break;
                }
                return false;
            };
            if (inside('PerlCommand') === 2)
                return;
            const insideInlineMathMode = inside('InlineMathMode');
            const insideDisplayMathMode = inside('DisplayMathMode');
            if (insideInlineMathMode == 2 ||
                (insideInlineMathMode == 1 && (!textBefore.endsWith('\\(') || textAfter.startsWith('\\)'))) ||
                insideDisplayMathMode == 2 ||
                (insideDisplayMathMode == 1 && (!textBefore.endsWith('\\[') || textAfter.startsWith('\\]')))) {
                const previous = context.matchBefore(/\\{?/);
                if (!previous && !context.explicit)
                    return;
                return {
                    from: previous?.from ?? context.pos,
                    options: [perlCommandSnippet]
                };
            }
            const previous = context.matchBefore(/\\[{([]?/);
            if (!previous && !context.explicit)
                return;
            return {
                from: previous?.from ?? context.pos,
                options: [perlCommandSnippet, inlineMathSnippet, displayMathSnippet]
            };
        },
        // '(', '[', and '{' are removed from this list because they interfere with the above snippets.
        closeBrackets: { brackets: ["'", '"'] }
    })
};
const pgTextNodeProps = [indentNodeProp.add(pgTextIndent), languageDataProp.add(pgTextLanguageData)];

const pgmlParser = new PGMLParser(pgmlNodeProps);
const pgmlLanguage = new Language(defineLanguageFacet(), pgmlParser, [], 'pgml');
const pgml = () => new LanguageSupport(pgmlLanguage);
const pgTextParser = new PGTextParser(pgTextNodeProps);
const pgTextLanguage = new Language(defineLanguageFacet(), pgTextParser, [], 'pg-text');
const pgText = () => new LanguageSupport(pgTextLanguage);
const pgLanguage = LRLanguage.define({
    name: 'pg',
    parser: parser.configure({
        props: [
            indentNodeProp.add({
                IfStatement: continuedIndent({ except: /^\s*({|else\b|elsif\b)/ }),
                Block: delimitedIndent({ closing: '}' }),
                'StringSingleQuoted StringQQuoted StringDoubleQuoted StringQqQuoted QWList': () => null,
                'StatementEnd InterpolatedHeredocBody UninterpolatedHeredocBody EndDocument': () => null,
                Statement: continuedIndent(),
                'PGMLBlock PGTextBlock LaTeXImageCode PodStatement': flatIndent
            }),
            foldNodeProp.add({
                'Block Array ArrayRef HashRef': foldInside,
                'InterpolatedHeredocBody UninterpolatedHeredocBody': (node) => {
                    if (node.prevSibling && node.lastChild)
                        return { from: node.prevSibling.to, to: node.lastChild.from };
                    return null;
                },
                'PGMLBlock PGTextBlock': (node) => {
                    if (node.firstChild?.type.name === 'BeginPG' && node.lastChild?.type.name === 'EndPG')
                        return { from: node.firstChild.to - 1, to: node.lastChild.from };
                    return null;
                },
                PodStatement(node) {
                    if (node.firstChild?.type.name === 'PodDirective' &&
                        node.firstChild.nextSibling?.type.name === 'PodContent')
                        return { from: node.firstChild.nextSibling.from, to: node.firstChild.nextSibling.to };
                    return null;
                },
                'EndDocument EndDataStatement': (node) => node.firstChild ? { from: node.firstChild.to, to: node.to } : null
            })
        ],
        wrap: parseMixed((node) => node.name === 'PGMLContent'
            ? { parser: pgmlParser }
            : node.name === 'PGTextContent'
                ? { parser: pgTextParser }
                : null)
    }),
    languageData: {
        commentTokens: { line: '#' },
        autocomplete: pgCompletion(true)
    }
});
const pg = () => new LanguageSupport(pgLanguage, [pgml().support, pgText().support]);

export { pg, pgLanguage, pgText, pgTextLanguage, pgTextParser, pgml, pgmlLanguage, pgmlParser, pgmlShow };
